// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   all_i2c.c   -   Allnet multiplexed I2C bus functions                                                          **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2003 - 2010 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************


#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <sys/time.h>
#include <stdlib.h>
#include "at91gpio.h"
#include "all_i2c.h"



// *****   Globale Variablen   *****

unsigned long int							bus_speed									= 500;				// Verzgerungsziffer
const unsigned char						zp[9]											= {1,2,4,8,16,32,64,128,(unsigned char)256}; // Zweierpotenzen LUT 2^0...8
char													bus_status								= 0;
unsigned char									bus_select_port						= 0;					// Port des gewhlten Busses
unsigned char									bus_select_gruppe					= 0;					// gewhltes 8er Bus-Modul
AT91S_PIO											*all_i2c_pioa;													// AT91 I/O Memory-Handler Port A
AT91S_PIO											*all_i2c_piob;													// AT91 I/O Memory-Handler Port B
AT91S_PIO											*all_i2c_pioc;													// AT91 I/O Memory-Handler Port C
AT91S_PIO											*all_i2c_piod;													// AT91 I/O Memory-Handler Port D
AT91S_PIO											*all_i2c_pioe;													// AT91 I/O Memory-Handler Port E



// **********   Busnummern und I/O-Port Zuordnungen   *****
//
// Bus 0;
// (ALL4500/ALL5000/ALL3651 usw.: interner I2C-Bus, direkt mit dem ALL5001 Board verbunden, keine Leitungstreiber)
// Data: PortA.20
// Clock: PortA.21
//
// Bus1
// (ALL4500 und ALL5000: Externer gemultiplexter Bus, mit Transistoren und Pullups)
// Data in: PortE.15
// Data out: PortE.14
// Clock out: PortE.13
// GroupSelect.0 out: PortE.7
// GroupSelect.1 out: PortE.8
// GroupSelect.2 out: PortE.9
// PortSelect.0 out: PortE.10
// PortSelect.1 out: PortE.11
// PortSelect.2 out: PortE.12
//
// Bus16
// (Buchis Wecker, groes Uhrzeit-Display: interner I2C-Bus, mit 74HC245 als Leitungstreiber, nur Schreiben, kein Lesen mglich)
// Data out: PortB.20
// Data in: PortB.21
//
// Bus17
// (Buchis Wecker, kleines Weckzeit-Display links: interner I2C-Bus, mit 74HC245 als Leitungstreiber, nur Schreiben, kein Lesen mglich)
// Data out: PortB.22
// Data in: PortB.23
//
// Bus18
// (Buchis Wecker, kleines Weckzeit-Display rechts: interner I2C-Bus, mit 74HC245 als Leitungstreiber, nur Schreiben, kein Lesen mglich)
// Data out: PortB.24
// Data in: PortB.25



// *****   i2c_port_group_out - Setzen der Port- und Gruppenselektierungsleitungen auf Ausgang   *****

void i2c_port_group_out(unsigned int busnummer)
	{
	if (busnummer == 0)
		{
		// Bus 0 ist nicht gemultiplext, drum Stub
		}
	if (busnummer == 1)
		{
		gpio_port_init_output(PORT_B,10);										// PortSelect 0
		gpio_port_init_output(PORT_B,11);										// PortSelect 1
		gpio_port_init_output(PORT_B,27);										// PortSelect 2
		gpio_port_init_output(PORT_B,25);										// GroupSelect 0
		gpio_port_init_output(PORT_B,24);										// GroupSelect 1
		gpio_port_init_output(PORT_B,23);										// GroupSelect 2
		gpio_port_init_output(PORT_B,29);										// I2C_Master_Data_Out
		gpio_port_init_output(PORT_B,30);										// I2C_Master_Data_In
		gpio_port_write(PORT_B,30,1);												// PE15
		gpio_port_init_input(PORT_B,30);										// I2C_Master_Data_In
		gpio_port_init_output(PORT_B,28);										// I2C_Master_Clock_Out
		}
	if (busnummer == 2)
		{
		// Bus 2 ist nicht gemultiplext, drum Stub
		}
	if (busnummer == 16)
		{
		// Bus 16 ist nicht gemultiplext, drum Stub
		}
	if (busnummer == 17)
		{
		// Bus 17 ist nicht gemultiplext, drum Stub
		}
	if (busnummer == 18)
		{
		// Bus 18 ist nicht gemultiplext, drum Stub
		}
	}



// *****   selektiere_port - Auswhlen eines bestimmten Ports innerhalb einer Gruppe   *****

void selektiere_port(unsigned int busnummer,unsigned char nummer)
	{
	if (busnummer == 0)
		{
		// am internen Bus gibt es keine Multiplex-Funktionen
		// darum muss auch keine Portnummer ausgewhlt werden
		// --> Stub
		}

	if (busnummer == 1)
		{
		if ((nummer & 1) != 0)															// Bit 0
			{
			gpio_port_write(PORT_B,10,1);											// PortSelect0 liegt auf PB10
			}
		else
			{
			gpio_port_write(PORT_B,10,0);											// PortSelect0 liegt auf PB10
			}
		if ((nummer & 2) != 0)															// Bit 1
			{
			gpio_port_write(PORT_B,11,1);											// PortSelect1 liegt auf PB11
			}
		else
			{
			gpio_port_write(PORT_B,11,0);											// PortSelect1 liegt auf PB11
			}
		if ((nummer & 4) != 0)															// Bit 2
			{
			gpio_port_write(PORT_B,27,1);											// PortSelect2 liegt auf PB27
			}
		else
			{
			gpio_port_write(PORT_B,27,0);											// PortSelect2 liegt auf PB27
			}
		}

	if (busnummer == 2)
		{
		// am internen Bus gibt es keine Multiplex-Funktionen
		// darum muss auch keine Portnummer ausgewhlt werden
		// --> Stub
		}
	if (busnummer == 16)
		{
		// Bus 16 ist nicht gemultiplext, drum Stub
		}
	if (busnummer == 17)
		{
		// Bus 17 ist nicht gemultiplext, drum Stub
		}
	if (busnummer == 18)
		{
		// Bus 18 ist nicht gemultiplext, drum Stub
		}
	}



// *****   selektiere_gruppe - Auswhlen einer bestimmten Portgruppe   *****

void selektiere_gruppe(unsigned int busnummer,unsigned char nummer)
	{
	if (busnummer == 0)
		{
		// am internen Bus gibt es keine Multiplex-Funktionen
		// darum muss auch keine Gruppennummer ausgewhlt werden
		// --> Stub
		}

	if (busnummer == 1)
		{
		if ((nummer & 1) != 0)															// Bit 0
			{
			gpio_port_write(PORT_B,25,1);											// GroupSelect0 liegt auf PB25
			}
		else
			{
			gpio_port_write(PORT_B,25,0);											// GroupSelect0 liegt auf PB25
			}
		if ((nummer & 2) != 0)															// Bit 1
			{
			gpio_port_write(PORT_B,24,1);											// GroupSelect1 liegt auf PB24
			}
		else
			{
			gpio_port_write(PORT_B,24,0);											// GroupSelect1 liegt auf PB24
			}
		if ((nummer & 4) != 0)															// Bit 2
			{
			gpio_port_write(PORT_B,23,1);											// GroupSelect2 liegt auf PB23
			}
		else
			{
			gpio_port_write(PORT_B,23,0);											// GroupSelect2 liegt auf PB23
			}
		}
	if (busnummer == 2)
		{
		// am internen Bus gibt es keine Multiplex-Funktionen
		// darum muss auch keine Gruppennummer ausgewhlt werden
		// --> Stub
		}
	if (busnummer == 16)
		{
		// Bus 16 ist nicht gemultiplext, drum Stub
		}
	if (busnummer == 17)
		{
		// Bus 17 ist nicht gemultiplext, drum Stub
		}
	if (busnummer == 18)
		{
		// Bus 18 ist nicht gemultiplext, drum Stub
		}
	}



// *****   i2c_ruhe - Ruhe am Bus   *****

void i2c_ruhe(unsigned int busnummer)
	{
	i2c_port_group_out(busnummer);												// Port- und Gruppenleitungen Ausgang
	i2c_clock_out(busnummer);															// Clockleitung Ausgang
	i2c_data_out(busnummer);															// Datenleitung Ausgang
	i2c_clock_high(busnummer);														// Clockleitung H
	i2c_data_high(busnummer);															// Datenleitung H
	}



// *****   i2c_clock_out() - CLK auf Ausgang setzen   *****

void i2c_clock_out(unsigned int busnummer)
	{
	if (busnummer == 0)
		{
		gpio_port_init_output(PORT_A,21);										// I2C Clock ist Ausgang
		}
	if (busnummer == 1)
		{
		// Datenrichtungen wurden bereits in i2c_port_group_out() gesetzt
		// also ist hier nichts zu tun.
		}
	if (busnummer == 2)
		{
		gpio_port_init_output(PORT_A,26);										// I2C Clock ist Ausgang
		}
	if (busnummer == 16)
		{
		gpio_port_init_output(PORT_B,21);										// I2C Clock ist Ausgang
		}
	if (busnummer == 17)
		{
		gpio_port_init_output(PORT_B,23);										// I2C Clock ist Ausgang
		}
	if (busnummer == 18)
		{
		gpio_port_init_output(PORT_B,25);										// I2C Clock ist Ausgang
		}
	}



// *****   i2c_data_out() - DATA auf Ausgang setzen   *****

void i2c_data_out(unsigned int busnummer)
	{
	if (busnummer == 0)
		{
		gpio_port_init_output(PORT_A,20);										// I2C DATA ist Ausgang
		}	
	if (busnummer == 1)
		{
		// Datenrichtungen wurden bereits in i2c_port_group_out() gesetzt
		// also ist hier nichts zu tun.
		}
	if (busnummer == 2)
		{
		gpio_port_init_output(PORT_D,29);										// I2C DATA ist Ausgang
		}	
	if (busnummer == 16)
		{
		gpio_port_init_output(PORT_B,20);										// I2C DATA ist Ausgang
		}	
	if (busnummer == 17)
		{
		gpio_port_init_output(PORT_B,22);										// I2C DATA ist Ausgang
		}	
	if (busnummer == 16)
		{
		gpio_port_init_output(PORT_B,24);										// I2C DATA ist Ausgang
		}	
	}



// *****   i2c_data_in() - DATA auf Eingang setzen   *****

void i2c_data_in(unsigned int busnummer)
	{
	if (busnummer == 0)
		{
		gpio_port_init_input(PORT_A,20);										// I2C DATA ist Eingang
		}
	if (busnummer == 1)
		{
		// Datenrichtungen wurden bereits in i2c_port_group_out() gesetzt
		// aber wir setzen den Ausgang auf H, damit der Eingang gelesen werden kann.
		i2c_data_high(busnummer);
		}
	if (busnummer == 2)
		{
		gpio_port_init_input(PORT_D,29);										// I2C DATA ist Eingang
		}
	if (busnummer == 16)
		{
		gpio_port_write(PORT_B,20,0);											// I2C DATA kann nicht als Eingang gesetzt werden, drum setzen wir DATA einfach auf H
		}
	if (busnummer == 17)
		{
		gpio_port_write(PORT_B,22,0);											// I2C DATA kann nicht als Eingang gesetzt werden, drum setzen wir DATA einfach auf H
		}
	if (busnummer == 18)
		{
		gpio_port_write(PORT_B,24,0);											// I2C DATA kann nicht als Eingang gesetzt werden, drum setzen wir DATA einfach auf H
		}
	}



// *****   i2c_clock_low() - CLK auf Low legen   *****

void i2c_clock_low(unsigned int busnummer)
	{
	if (busnummer == 0)
		{
		gpio_port_write(PORT_A,21,0);												// SCL liegt auf PA21
		}
	if (busnummer == 1)
		{
		gpio_port_write(PORT_B,28,1);												// /SCL liegt auf PB28
		}
	if (busnummer == 2)
		{
		gpio_port_write(PORT_A,26,0);												// SCL liegt auf PC10
		}
	if (busnummer == 16)
		{
		gpio_port_write(PORT_B,21,0);												// SCL = L
		}
	if (busnummer == 17)
		{
		gpio_port_write(PORT_B,23,0);												// SCL = L
		}
	if (busnummer == 18)
		{
		gpio_port_write(PORT_B,25,0);												// SCL = L
		}
	}



// *****   i2c_clock_high() - CLK auf High legen   *****

void i2c_clock_high(unsigned int busnummer)
	{
	if (busnummer == 0)
		{
		gpio_port_write(PORT_A,21,1);												// SCL liegt auf PA21
		}
	if (busnummer == 1)
		{
		gpio_port_write(PORT_B,28,0);												// /SCL liegt auf PB28
		}
	if (busnummer == 2)
		{
		gpio_port_write(PORT_A,26,1);												// SCL liegt auf PC10
		}
	if (busnummer == 16)
		{
		gpio_port_write(PORT_B,21,1);												// SCL = H
		}
	if (busnummer == 17)
		{
		gpio_port_write(PORT_B,23,1);												// SCL = H
		}
	if (busnummer == 18)
		{
		gpio_port_write(PORT_B,25,1);												// SCL = H
		}
	}



// *****   int_i2c_data_low() - DATA auf Low legen   *****

void i2c_data_low(unsigned int busnummer)
	{
	if (busnummer == 0)
		{
		gpio_port_write(PORT_A,20,0);												// SDA liegt auf PA20
		}
	if (busnummer == 1)
		{
		gpio_port_write(PORT_B,29,1);												// /SDA liegt auf PB29
		}
	if (busnummer == 2)
		{
		gpio_port_write(PORT_D,29,0);												// SDA liegt auf PD29
		}
	if (busnummer == 16)
		{
		gpio_port_write(PORT_B,20,0);												// SDA = L
		}
	if (busnummer == 17)
		{
		gpio_port_write(PORT_B,22,0);												// SDA = L
		}
	if (busnummer == 18)
		{
		gpio_port_write(PORT_B,24,0);												// SDA = L
		}
	}



// *****   i2c_data_high() - DATA auf High legen   *****

void i2c_data_high(unsigned int busnummer)
	{
	if (busnummer == 0)
		{
		gpio_port_write(PORT_A,20,1);												// SDA liegt auf PA20
		}
	if (busnummer == 1)
		{
		gpio_port_write(PORT_B,29,0);												// /SDA liegt auf PB29
		}
	if (busnummer == 2)
		{
		gpio_port_write(PORT_D,29,1);												// SDA liegt auf PD29
		}
	if (busnummer == 16)
		{
		gpio_port_write(PORT_B,20,1);												// SDA = H
		}
	if (busnummer == 17)
		{
		gpio_port_write(PORT_B,22,1);												// SDA = H
		}
	if (busnummer == 18)
		{
		gpio_port_write(PORT_B,24,1);												// SDA = H
		}
	}



// *****   i2c_read_data() - Abfragen der DATA-Leitung   *****

unsigned char i2c_read_data(unsigned int busnummer)
	{
	unsigned char rw;
	unsigned char messwert = 0;


	if (busnummer == 0)
		{
		messwert = gpio_port_read(PORT_A,20);								// SDA-Pin ist nicht invertiert !
		}
	if (busnummer == 1)
		{
		messwert = gpio_port_read(PORT_B,30);								// SDA_IN-Pin PB30 ist nicht invertiert !
		}
	if (busnummer == 2)
		{
		messwert = gpio_port_read(PORT_D,29);								// SDA-Pin ist nicht invertiert !
		}
	if (busnummer == 16)
		{
		messwert = 0;																				// SDA kann nicht gelesen werden, wir liefern L zurck
		}
	if (busnummer == 17)
		{
		messwert = 0;																				// SDA kann nicht gelesen werden, wir liefern L zurck
		}
	if (busnummer == 18)
		{
		messwert = 0;																				// SDA kann nicht gelesen werden, wir liefern L zurck
		}

		if (messwert != 0)
		{
		rw = 1;
		}
	else
		{
		rw = 0;
		}
	return(rw);
	}



// *****   i2c_debug_ein() - Debug-LED einschalten   *****

void i2c_debug_ein(void)
	{
	gpio_port_write(PORT_D,23,1);													// H=Ein
	}



// *****   debug_aus() - Debug-LED ausschalten   *****

void i2c_debug_aus(void)
	{
	gpio_port_write(PORT_D,23,0);													// L=Aus
	}



// *****   i2c_send_byte - Senden eines Bytes mit ACK/NAK   *****

unsigned char i2c_send_byte(unsigned int busnummer,unsigned char b,unsigned char ack)
	{
	unsigned char acknack = 1;
	unsigned char rw;

	if (ack == 0) acknack = 0;
	if (ack == 1) acknack = 1;
	if (ack == 2) acknack = 2;

	// printf("-- i2c_send_byte(%d,%d): ",b,ack);
	i2c_byte_send(busnummer,b);														// Das bergebene Byte ber den Bus senden
 	i2c_delay(6);

	if (acknack == 0)																			// 0: NAK senden
		{
		i2c_send_nak(busnummer);
		rw = 'N';
		goto ende;
		}
	if (acknack == 1)																			// 1: ACK senden
		{
		i2c_send_ack(busnummer);
		rw = 'A';
		goto ende;
		}
	if (acknack == 2)																			// 2: ACK/NACK vom Slave abwarten
		{
		rw = i2c_wait_for_ack(busnummer);
		goto ende;
		}

ende:
	// printf("ACKNACK_RW:%d\n",rw);
	return(rw);
	}



// *****   i2c_receive_byte - Empfangen eines Bytes mit ACK/NAK   *****

unsigned char i2c_receive_byte(unsigned int busnummer,unsigned char ack)
	{
	unsigned char acknack = 1;
	unsigned char rw;

	if (ack == 0) acknack = 0;
	if (ack == 1) acknack = 1;

	rw = i2c_byte_receive(busnummer);											// Das Byte vom Bus holen

	if (acknack == 0)																			// 0: NAK senden
		{
		i2c_send_nak(busnummer);
		goto ende;
		}
	if (acknack == 1)																			// 1: ACK senden
		{
		i2c_send_ack(busnummer);
		goto ende;
		}

ende:
	return(rw);
	}



// *****   i2c_byte_send() - Senden eines Bytes   *****

void i2c_byte_send(unsigned int busnummer,unsigned char b)
	{
	signed char t;

	i2c_data_out(busnummer);

	for (t = 7;t >= 0;t--)																// MSB first
		{
		if ((b & zp[(unsigned char)t]) != 0)								// Bit auf den Bus legen
			{
			i2c_data_high(busnummer);													// pin_high(RE,i);
			}
		else
			{
			i2c_data_low(busnummer);													// pin_low(RE,i);
			}
	 	i2c_delay(1); // 1
		i2c_clock_high(busnummer);													// einen Clock-Impuls senden
	 	i2c_delay(2);
		i2c_clock_low(busnummer);
	 	i2c_delay(1);
		}
	i2c_data_low(busnummer);															// Datenleitung am Ende der betragung immer auf L setzen
 	// i2c_delay(1);
	}



// *****   i2c_byte_receive() - Empfangen eines Bytes   *****

unsigned char i2c_byte_receive(unsigned int busnummer)
	{
	signed char t;
	unsigned char i;

	i = 0;

	i2c_data_in(busnummer);

	for (t = 7;t >= 0;t--)																// MSB first
		{
		i2c_delay(1);
		i2c_clock_high(busnummer);
		i2c_delay(1);
		if (i2c_read_data(busnummer) != 0)									// Bit einlesen
			{
			i += zp[(unsigned char)t];
			}
		// i2c_delay(1);
		i2c_clock_low(busnummer);
		// i2c_delay(1);
 		}
	return(i);
	}



// *****   i2c_wait_for_ack() - Warten auf ACK von den Slaves   *****

unsigned char i2c_wait_for_ack(unsigned int busnummer)
	{
	unsigned char rw = 0;

	i2c_data_in(busnummer);
	i2c_data_high(busnummer);															// auf H setzen, um Einlesen zu ermglichen
	
 	i2c_delay(2);
	if (i2c_read_data(busnummer) != 0)										// Bit einlesen
		{
		// printf("*NACK*");
		rw = 1;
		}
	else
		{
		// printf("*ACK*");
		rw = 0;
		}

	i2c_clock_high(busnummer);
 	i2c_delay(1);
	i2c_clock_low(busnummer);
 	i2c_delay(2);
	return(rw);
	}
	


// *****   i2c_send_ack() - ACK an den Bus senden   *****

void i2c_send_ack(unsigned int busnummer)
	{
	i2c_data_out(busnummer);
	i2c_data_low(busnummer);															// Datenleitung low setzen

 	i2c_delay(2);
	// einfach nur einen Impuls senden
	i2c_clock_high(busnummer);
 	i2c_delay(2);
	i2c_clock_low(busnummer);
	i2c_data_high(busnummer);															// Datenleitung High setzen (freigeben)
 	i2c_delay(2);
	}



// *****   i2c_send_nak() - NAK an den Bus senden   *****

void i2c_send_nak(unsigned int busnummer)
	{
	i2c_data_out(busnummer);
	i2c_data_high(busnummer);															// Datenleitung High setzen

 	i2c_delay(1);
	// einfach nur einen Impuls senden
	i2c_clock_high(busnummer);
 	i2c_delay(2);
	i2c_clock_low(busnummer);
 	i2c_delay(2);
	}



// *****   i2c_send_stop() - Bus freigeben (STOP condition)   *****

void i2c_send_stop(unsigned int busnummer)
	{
	/*
	// Alte Version
 	i2c_delay(2);																					// Pre-Delay
	i2c_data_out(busnummer);
	i2c_data_low(busnummer);															// Datenleitung Low setzen
 	i2c_delay(1);

	i2c_clock_high(busnummer);														// Clockleitung High setzen
 	i2c_delay(2);

	i2c_data_high(busnummer);															// Datenleitung High setzen
 	i2c_delay(4);
	*/

	// Neue Version
	i2c_clock_high(busnummer);														// Clockleitung High setzen, falls diese nicht H ist
	i2c_data_out(busnummer);
	i2c_data_low(busnummer);															// Datenleitung Low setzen, falls dies nicht der Fall ist
 	i2c_delay(2);																					// Pre-Delay
	i2c_data_high(busnummer);															// Datenleitung High setzen
 	i2c_delay(4);

	}



// *****   i2c_send_start() - Start-Signal senden   *****

void i2c_send_start(unsigned int busnummer)
	{
	/*
	// Alte Version
 	i2c_delay(2);																					// Pre-Delay
	i2c_data_out(busnummer);
	i2c_data_low(busnummer);															// Datenleitung Low setzen
 	i2c_delay(2);																					// kurz warten
	i2c_clock_low(busnummer);															// Clock-Leitung Low setzen
 	i2c_delay(4);																					// Warten, damit die Start Condition ueberall bekannt wird
	*/


	// Neue Version
	i2c_clock_high(busnummer);														// Clock-Leitung High setzen, falls sie nicht H ist
	i2c_data_out(busnummer);
	i2c_data_high(busnummer);															// Datenleitung High setzen, falls sie nicht H ist
 	i2c_delay(2);																					// Pre-Delay
	i2c_data_low(busnummer);															// Datenleitung Low setzen
 	i2c_delay(2);																					// kurz warten
	i2c_clock_low(busnummer);															// Clock-Leitung Low setzen
 	i2c_delay(4);																					// Warten, damit die Start Condition ueberall bekannt wird

	}



// *****   i2c_send_start_sht() - nicht I2C konformes Spezial-Startsignal fr SHT71 senden   *****
//         Auszug aus dem SHT71 Datenblatt:
//         To initiate a transmission, a "Transmission Start" sequence 
//         has to be issued. It consists of a lowering of the DATA line 
//         while SCK is high, followed by a low pulse on SCK and raising
//         DATA again while SCK is still high.
//         --- Warum habt Ihr das getan ? Htte ein normaler I2C Start nicht gereicht ???

void i2c_send_start_sht(unsigned int busnummer)
	{																											// zu Anfang sind CLK und Data normalerweise beide H
	i2c_data_out(busnummer);
	i2c_data_high(busnummer);															// Datenleitung High setzen (sollte sie eh sein)
 	i2c_delay(2);

	i2c_clock_high(busnummer);														// Clock-Leitung High setzen
 	i2c_delay(3);

	i2c_data_low(busnummer);															// Datenleitung Low setzen
 	i2c_delay(3);
	i2c_clock_low(busnummer);															// Clock-Leitung Low setzen
 	i2c_delay(2);
	i2c_clock_high(busnummer);														// Clock-Leitung High setzen
 	i2c_delay(3);
	i2c_data_high(busnummer);															// Datenleitung High setzen
 	i2c_delay(2);
	i2c_clock_low(busnummer);															// Clock-Leitung Low setzen
 	i2c_delay(10);
	}



// *****   get_10us_now()   -   aktuelle Zeit in 10us ermitteln   *****

unsigned long get_10us_now(void)
	{
	unsigned long rw;
	const long us10divider = CLOCKS_PER_SEC / 100000;			// Teiler um von Clockticks auf 10us zu kommen
  struct timeval tv;
	time_t curtime;
	struct tm *myTm;


	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln 
  curtime = tv.tv_sec;																	// Lores-Zeit in s reusziehen
	myTm = localtime(&curtime);														// ...und berfhren in tm-Struktur
	rw = (myTm->tm_hour * 3600 + myTm->tm_min * 60 + myTm->tm_sec) * 1000 + tv.tv_usec / us10divider;
	return(rw);
	}



// *****   i2c_delay()  -  Warteschleife; Zeitangabe in Mikrosekunden   *****

void i2c_delay(unsigned long d)
	{

	unsigned long t;
	unsigned long startzeit;
	unsigned long endzeit;	
  struct timeval tv;


	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln 
	startzeit =  tv.tv_sec * 1000000 + tv.tv_usec;
	endzeit = startzeit + d * bus_speed;									// Endzeit ermitteln

schleife:
	gettimeofday(&tv,NULL);																// Ist-Zeit ermitteln 
	t =  tv.tv_sec * 1000000 + tv.tv_usec;
	if (t < startzeit) goto ende;													// um Mitternacht einfach abbrechen
	if (t < endzeit) goto schleife;												// bis zur Endzeit warten
ende:
	;
	}



// *****   gpio_port_init_output(Portnummer,Bitmaske)   -   Datenrichtung eines I/O-Ports auf Ausgang setzen   *****

void gpio_port_init_output(unsigned int portnummer,unsigned char bitnummer)
	{
	unsigned int bitmaske;
	
	bitmaske = 1 << bitnummer;
	
	if (portnummer == PORT_A)
		{
		pio_enable(all_i2c_pioa,bitmaske);
		pio_disable_irq(all_i2c_pioa,bitmaske);
		pio_disable_multiple_driver(all_i2c_pioa,bitmaske);
		pio_disable_pull_ups(all_i2c_pioa,bitmaske);
		pio_synchronous_data_output(all_i2c_pioa,bitmaske);
		pio_output_enable(all_i2c_pioa,bitmaske);
		}
	if (portnummer == PORT_B)
		{
		pio_enable(all_i2c_piob,bitmaske);
		pio_disable_irq(all_i2c_piob,bitmaske);
		pio_disable_multiple_driver(all_i2c_piob,bitmaske);
		pio_disable_pull_ups(all_i2c_piob,bitmaske);
		pio_synchronous_data_output(all_i2c_piob,bitmaske);
		pio_output_enable(all_i2c_piob,bitmaske);
		}
	if (portnummer == PORT_C)
		{
		pio_enable(all_i2c_pioc,bitmaske);
		pio_disable_irq(all_i2c_pioc,bitmaske);
		pio_disable_multiple_driver(all_i2c_pioc,bitmaske);
		pio_disable_pull_ups(all_i2c_pioc,bitmaske);
		pio_synchronous_data_output(all_i2c_pioc,bitmaske);
		pio_output_enable(all_i2c_pioc,bitmaske);
		}
	if (portnummer == PORT_D)
		{
		pio_enable(all_i2c_piod,bitmaske);
		pio_disable_irq(all_i2c_piod,bitmaske);
		pio_disable_multiple_driver(all_i2c_piod,bitmaske);
		pio_disable_pull_ups(all_i2c_piod,bitmaske);
		pio_synchronous_data_output(all_i2c_piod,bitmaske);
		pio_output_enable(all_i2c_piod,bitmaske);
		}
	if (portnummer == PORT_E)
		{
		pio_enable(all_i2c_pioe,bitmaske);
		pio_disable_irq(all_i2c_pioe,bitmaske);
		pio_disable_multiple_driver(all_i2c_pioe,bitmaske);
		pio_disable_pull_ups(all_i2c_pioe,bitmaske);
		pio_synchronous_data_output(all_i2c_pioe,bitmaske);
		pio_output_enable(all_i2c_pioe,bitmaske);
		}
	
	}



// *****   gpio_devmem_open   -   Memory-Devices fr die I/O Portbnke ffnen   *****

void gpio_devmem_open(void)
	{
	all_i2c_pioa = pio_map(PIOA_BASE);
	all_i2c_piob = pio_map(PIOB_BASE);
	all_i2c_pioc = pio_map(PIOC_BASE);
	all_i2c_piod = pio_map(PIOD_BASE);
	all_i2c_pioe = pio_map(PIOE_BASE);
	}



// *****   gpio_port_init_input(Portnummer,Bitmaske)   -   Datenrichtung eines I/O-Ports auf Eingang setzen   *****

void gpio_port_init_input(unsigned int portnummer,unsigned char bitnummer)
	{
	unsigned int bitmaske;
	
	bitmaske = 1 << bitnummer;
	
	if (portnummer == PORT_A) pio_input_enable(all_i2c_pioa,bitmaske);
	if (portnummer == PORT_B) pio_input_enable(all_i2c_piob,bitmaske);
	if (portnummer == PORT_C) pio_input_enable(all_i2c_pioc,bitmaske);
	if (portnummer == PORT_D) pio_input_enable(all_i2c_piod,bitmaske);
	if (portnummer == PORT_E) pio_input_enable(all_i2c_pioe,bitmaske);
	}



// *****   gpio_port_read(Portnummer,Bit)   -   Portbit einlesen (RW=0/1)   *****

unsigned char gpio_port_read(unsigned int portnummer,unsigned char bitnummer)
	{
	unsigned char rw;
	
	rw = 0;

	if (portnummer == PORT_A) if (pio_in(all_i2c_pioa,1 << bitnummer) != 0) rw=1;
	if (portnummer == PORT_B) if (pio_in(all_i2c_piob,1 << bitnummer) != 0) rw=1;
	if (portnummer == PORT_C) if (pio_in(all_i2c_pioc,1 << bitnummer) != 0) rw=1;
	if (portnummer == PORT_D) if (pio_in(all_i2c_piod,1 << bitnummer) != 0) rw=1;
	if (portnummer == PORT_E) if (pio_in(all_i2c_pioe,1 << bitnummer) != 0) rw=1;

	return(rw);
	}



// *****   gpio_port_write(Portnummer,Bitnummer,Wert)   -   Portbit schreiben   *****

void gpio_port_write(unsigned int portnummer,unsigned char bitnummer,unsigned char wert)
	{
	if (portnummer == PORT_A) pio_out(all_i2c_pioa,1 << bitnummer,wert);
	if (portnummer == PORT_B) pio_out(all_i2c_piob,1 << bitnummer,wert);
	if (portnummer == PORT_C) pio_out(all_i2c_pioc,1 << bitnummer,wert);
	if (portnummer == PORT_D) pio_out(all_i2c_piod,1 << bitnummer,wert);
	if (portnummer == PORT_E) pio_out(all_i2c_pioe,1 << bitnummer,wert);
	}
