// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   dmx_demon.c   -   Demon zum Senden von DMX512-Daten                                                           **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2011 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   Eintrge in der Konfigurations-Datenbank, die vom Demon ausgewertet werden   **********
//
// /control/powerline/server/comport"										Serielle Schnittstelle, an der das PL-Mastermodul hngt (normalerweise /dev/ttyS4)
// /control/powerline/server/baudrate"									Baudrate dieser Schnittstelle (normalerweise 4800)
// /control/powerline/server/udpport"										UDP-Port fr Steuerkommandos (54009)
// /control/powerline/server/username										zu bergebender Username
// /control/powerline/server/password										zu bergebendes Pawort
// /control/powerline/max_retries												Anzahl Versuche, einen Slave zu erreichen
// /control/powerline/max_slaves"												Anzahl zu verwaltender Slaves
// /control/powerline/reload_counter"										Zhler, um festzustellen ob ein Neuladen der Config ntig ist
// /control/actors/actorXX/value"												gibt den zu schaltenden Wert; wird mit gepolltem Wert upgedated



#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <termios.h>
#include <time.h>
#include <sys/time.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "sqldb.h"
#include "sensor_demon.h"
#include "shm_use.h"
#include "dmx_demon.h"
#include "logging.h"



// *****   Globale Variablen   *****

signed short int			dmx_fd;
char									dmx_device[32];
struct s_shm_dmxdata	*shm_dmxdata;											// DMX512-Daten im Shared-RAM



// *****   dmx_send(fd,array,laenge)   -   Sendet DMX-Daten   *****

signed short int dmx_send(signed short int fd,char array[],signed short int laenge)
	{
	signed short int rw = 0;
	struct termios my_termios;
	char minipuffer[2] = {0,0};
	signed long int t;


	// zu Anfang BREAK mit ca. 100 us senden.
	// normaler BREAK hat baer laut POSIX 250 ms, wre also
	// viel zu lang - drum Umschalten auf 57600 Baud und Senden
	// einer 0, das macht ca. 120 us und wird vom Empfnger als
	// BREAK erkannt - dann kurz warten bis das Signal raus ist
	// und die mindestens 8 us MARK-Pegel wieder da sind.
	// Achtung: nicht tcdrain(fd) benutzen, das dauert wieder
	// ewige 20 ms bis es zurckkehrt !
	tcgetattr(fd,&my_termios);														// TTY-Settings holen
	my_termios.c_cflag = B57600 | CS8  |CREAD | CLOCAL | HUPCL | CSTOPB; // CSTOPB=2 Stopbits
	my_termios.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // Raw processing, kein Echo !
	my_termios.c_iflag &= ~(IXON | IXOFF | IXANY);				// kein Handshaking
	cfsetospeed(&my_termios, B57600);											// langsame Baudrate einstellen
	tcsetattr(fd, TCSANOW, &my_termios);									// TTY-Settings schreiben

	write(fd,minipuffer,1);																// ASCII-0 ausgeben

	for(t = 0;t < 5000;t++)
		{
		rw += t;
		}

	// Prambel gesendet, jetzt knnen wir wieder auf 250000 Baud gehen
	tcgetattr(fd,&my_termios);														// TTY-Settings holen
	my_termios.c_cflag = B460800 | CS8  |CREAD | CLOCAL | HUPCL | CSTOPB; // CSTOPB=2 Stopbits
	my_termios.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);// Raw processing, kein Echo !
	my_termios.c_iflag &= ~(IXON | IXOFF | IXANY);				// kein Handshaking
	cfsetospeed(&my_termios, B460800);										// wieder auf 250 Kbit/s setzen
	tcsetattr(fd, TCSANOW, &my_termios);


	w232(fd,&array[0],laenge,0);													// Datenblock senden
	tcdrain(fd);

	return(rw);
	}



// *****   o232(Benennung) - RS232-Port ffnen   *****

signed short int o232(char sPortName[])
	{
	signed short int fd = 0;
	struct termios my_termios;

	printf("sPortName=%s\n", sPortName);

	fd = open(sPortName, O_RDWR | O_NOCTTY | O_NDELAY);
	if (fd < 0)
		{
		printf("open error %d %s\n", fd, strerror(fd));
		}
	else
		{
		printf("fd is %d\n", fd);
		tcgetattr(fd, &my_termios);
  
		// 460800 wurde im Kernel ersetzt durch 250000 !!!
		my_termios.c_cflag = B460800 | CS8  |CREAD | CLOCAL | HUPCL | CSTOPB; // CSTOPB=2 Stopbits
		my_termios.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // Raw processing, kein Echo !
		my_termios.c_iflag &= ~(IXON | IXOFF | IXANY); // kein Handshaking

		cfsetospeed(&my_termios, B460800);
		tcsetattr(fd, TCSANOW, &my_termios);
		}
	return(fd);
	}



// *****   w232(fd,Zeichenarray[],Laenge)   -   Zeichen seriell ausgeben   *****

signed short int w232(signed short int fd,char* psOutput,unsigned short int laenge,unsigned char delayus)
	{
	signed short int iOut;
	signed long int t;
	char a[2];
	signed long int rw = 0;



	if (fd < 1)
		{
		printf("port is not open\n");
		return(-1);
		}

	if(delayus == 0)
		{
		// keine Zwischenbyte-Verzgerung
		iOut = write(fd,psOutput,laenge);											// hier war statt "laenge" frher "strlen(psOutput)"
		if (iOut < 0)
			{
			printf("write error %d %s\n",iOut,strerror(iOut));
			}
		else
			{
			// printf("wrote %d chars.\n",iOut);
			}
		return(iOut);
		}
	else
		{
		// zwischen den Bytes mu gewartet werden
		for(t = 0;t < laenge;t++)
			{
			a[0] = psOutput[t];
			a[1] = 0;
			iOut = write(fd,a,1);															// 1 Zeichen ausgeben
			if (iOut < 0)
				{
				printf("write error %d %s\n",iOut,strerror(iOut));
				}
			// tcdrain(dmx_fd);																	// warten, bis das Zeichen raus ist
			// usleep(delayus);																	// angegebene Zeit warten

			for(t = 0;t < 600;t++)
				{
				rw += t;
				}

			}
		}

	return(rw);
	}



// *****   r232(fd)   -   Zeichen seriell einlesen   *****

signed short int r232(signed short int fd)
	{
	char tp[2];																						// temporrer Mini-Puffer
	signed short int rw = -1;
	signed long int l;


	l = read(fd,tp,1);																		// Nachschauen, ob ein Zeichen angekommen ist
	if (l > 0)
		{
		if (l > 1) printf("[L=%ld]",l);
		rw = tp[0];
		}
	else
		{
		rw = -1;
		}

	return(rw);
	}



// *****   get_ms_now()   -   aktuelle Zeit in ms ermitteln   *****

signed long int get_ms_now(void)
	{
	signed long int rw;
	const long msdivider = CLOCKS_PER_SEC / 1000;					// Teiler um von Clockticks aus ms zu kommen
  struct timeval tv;
	time_t curtime;
	struct tm *myTm;


	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln 
  curtime = tv.tv_sec;																	// Lores-Zeit in s reusziehen
	myTm = localtime(&curtime);														// ...und berfhren in tm-Struktur
	rw = (myTm->tm_hour * 3600 + myTm->tm_min * 60 + myTm->tm_sec) * 1000 + tv.tv_usec / msdivider;
	return(rw);
	}



// **********   Demon-Configfile einlesen   **********

void dmx_demon_readconfig(void)
	{
	char ep[1024];


	log_level = 3;
	strcpy(dmx_device,"/dev/ttyS1");
	// strcpy(dmx_device,"/dev/ttyS4");

	shm_dmxdata->dmx_laenge = 513;													// 512+1 ist normal

	sprintf(ep,"Global DMX Configuration read. Loglevel:%d, Comport:\"%s\", Laenge:%d",log_level,dmx_device,shm_dmxdata->dmx_laenge);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	}




// *****   Bitreihenfolge umdrehen   *****

char bru(char in)
	{
	char rw = 0;
	unsigned char t;
	char zp[8] = {1,2,4,8,16,32,64,128};

	for(t = 0;t < 8;t++)
		{
		if((in & zp[t]) > 0) rw += zp[7-t];
		}

	return(rw);
	}



// *****   Hauptschleife   *****

int main(int argc,char **argv)
	{
	char									ep[1024];												// Puffer fr Logfile-Meldungen
	int shID;																							// Shared-Memory ID
	unsigned long int t;


	// auf existierendes Shared Memory fr DMX512 zugreifen
	shID = shmget(SHM_ID_DMX,SHM_DMX_SIZE,0666);
	if (shID >= 0)
		{
		shm_dmxdata = shmat(shID, 0, 0);
		if (shm_dmxdata == (struct s_shm_dmxdata *)-1)
			{
			printf("Problem beim Zugriff auf DMX shared memory !\n");
			goto ende;
			}
		else
			{
			printf("SHM_DMXdata OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_DMXdata war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		goto ende;
    }


	log_level = 3;																				// Debuglevel setzen
	dmx_demon_readconfig();


	shm_dmxdata->dmx_array[0] = 0;												// Startbyte auf 0 setzen	
	for (t = 0;t < 512;t++)
		{
		shm_dmxdata->dmx_array[t + 1] = 0;									// Datenbytes auf 0 setzen
		}


	// Logging-System initialisieren
	strcpy (log_filename,"/www/reports/dmx_demon.log");		// Logfilenamen setzen
	sprintf(ep,"----------   dmx_demon V %s started (debuglevel=%d)   ----------",DMX_DEMON_VERSION,log_level);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben
	printf("dmx demon V %s started.\n",DMX_DEMON_VERSION);


	dmx_fd = o232(dmx_device);
	printf("dmx_fd=%d\n",dmx_fd);


hauptschleife:
	dmx_send(dmx_fd,shm_dmxdata->dmx_array,shm_dmxdata->dmx_laenge);
	// usleep(1000);																				// 1 ms warten
	goto hauptschleife;


ende:
	close(dmx_fd);
	return(0);
	}
