// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   dmx_set.c   -   Commandline-Tool zum Senden von DMX512-Daten                                                  **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2011 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************



#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <termios.h>
#include <time.h>
#include <sys/time.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "shm_use.h"



// *****   Globale Variablen   *****

struct s_shm_dmxdata	*shm_dmxdata;											// DMX512-Daten im Shared-RAM



// *****   Hauptschleife   *****

int main(int argc,char **argv)
	{
	int shID;																							// Shared-Memory ID
	signed long int dmx_position;													// Schreibposition (0: Lnge, 1...512: DMX-Value)
	signed long int dmx_value;														// zu setzender Wert


	// auf existierendes Shared Memory fr DMX512 zugreifen
	shID = shmget(SHM_ID_DMX,SHM_DMX_SIZE,0666);
	if (shID >= 0)
		{
		shm_dmxdata = shmat(shID, 0, 0);
		if (shm_dmxdata == (struct s_shm_dmxdata *)-1)
			{
			printf("Problem beim Zugriff auf DMX shared memory !\n");
			goto ende;
			}
		else
			{
			// printf("SHM_DMXdata OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_DMXdata war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		goto ende;
    }


	if (argc != 3)
		{
		printf("ERROR: usage: dmx_set position value\n");
		printf("              position-1: Length of Data block (should be between 24...513, including start byte)\n");
		printf("              position 0: Start byte (should always be 0)\n");
		printf("              position 1...512: position od DMX data byte\n");
		printf("              value (0...255): DMX controller value to be written\n");
		goto ende;
		}

	dmx_position = atol(argv[1]);
	dmx_value = atol(argv[2]);

	if ((dmx_position < -1) || (dmx_position > 512))
		{
		printf("ERROR: dmx_position needs to be within -1...512 !\n");
		goto ende;
		}

	if ((dmx_position != -1) && ((dmx_value < 0) || (dmx_value > 255)))
		{
		printf("ERROR: dmx_value needs to be within 0...255 !\n");
		goto ende;
		}

	if ((dmx_position == -1) && ((dmx_value < 2) || (dmx_value > 513)))
		{
		printf("ERROR: DMX Blocksize needs to be within 2...513 !\n");
		goto ende;
		}

	if (dmx_position == -1)
		{
		shm_dmxdata->dmx_laenge = dmx_value;								// Lnge in shared Memory schreiben
		}
	else
		{
		shm_dmxdata->dmx_array[dmx_position] =dmx_value;		// Datenbyte in shared Memory schreiben
		}

ende:
	return(0);
	}
