// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   i2c_chips.c   -   Treiber fr diverse I2C Bausteine                                                           **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2003 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************



#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <unistd.h>
#include "at91gpio.h"
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "all_i2c.h"
#include "i2c_chips.h"
#include "rgb_demon.h"
#include "rgb_read.h"
#include "logging.h"



// *****   Globale Variablen   *****

#define SENSOR_RETRIES 3

char							dbo = 1;															// Debug Output 0/1
unsigned char			rtc_sekunde = 0;											// Sekundenwert von der RTC
unsigned char			rtc_minute = 0;												// Minutenwert von der RTC
unsigned char			rtc_stunde = 0;												// Stundenwert von der RTC
unsigned char			rtc_tag = 0;													// Tag von der RTC
unsigned char			rtc_monat = 0;												// Monat von der RTC
unsigned short int rtc_jahr = 2009;											// Jahr von der RTC (TODO: bei Silvester weiterzhlen und in File schreiben)

unsigned char 		t_r0 = 0;
unsigned char 		t_g0 = 0;
unsigned char 		t_b0 = 0;
unsigned char 		t_w0 = 0;
unsigned char 		t_r1 = 0;
unsigned char 		t_g1 = 0;
unsigned char 		t_b1 = 0;
unsigned char 		t_w1 = 0;
unsigned char 		t_poti0 = 0;
unsigned char 		t_poti1 = 0;
unsigned char 		t_poti2 = 0;
unsigned char 		t_poti3 = 0;
unsigned char 		t_poti4 = 0;
unsigned char 		t_poti5 = 0;
unsigned char 		t_fw_major = 0;
unsigned char 		t_fw_minor = 0;
unsigned char 		t_ir = 0;
unsigned char 		t_ircounter = 0;
unsigned char 		t_dim0 = 0;
unsigned char 		t_dim1 = 0;
double						t_temperature_int = -2048001;
double						t_temperature_ext = -2048001;
double						t_humidity_ext = -2048001;
signed short int	physical_measurementtype_rotator;
signed short int	physical_measurementtype_counter;
signed short int	physical_measurementtype_cycles;



// *****   dez2bcd - Konvertieren eines normalen Bytes ins BCD-Format   *****

unsigned char dez2bcd(unsigned char natural_binary)
	{
  return(((natural_binary / 10) << 4) + natural_binary%10);
	}



// *****   bcd2dez - Konvertieren eines BCD-Formatierten Bytes in braucbhare Form   *****

unsigned char bcd2dez(unsigned char BCD)
	{
  return(((BCD >> 4) * 10) + (BCD & 0x0f));
	}




// *****   all_i2c_chip_dsdx015a2r_read_pressure - Luftdruck eines DSDX015A2R auslesen   *****

signed long int all_i2c_chip_dsdx015a2r_read_pressure(unsigned char busnummer)
	{
	signed long int rw;
	unsigned char t;
	unsigned char low;
	unsigned char high;
	char ep[80];


	i2c_send_start(busnummer);														// I2C Start senden

	t = i2c_send_byte(busnummer,241,2);										// Byte senden, ACK ermitteln: 3=11110001 = Read from DSDX015A2R
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"dsdx015a2r_read ERROR:no ACK from DSDX015A2R");
			log_append(3,ep);
      goto fehlerende;
      }


	high = i2c_byte_receive(busnummer);										// Byte holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// Byte holen, ACK senden
	i2c_send_nak(busnummer);

	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		//rw = ((signed long int)high << 8) + (signed long int)low - 3960;

		rw = (double)(high * 256 + low);
		// 15 PSI = 1034.214 mbar = 103422 Zhlwerte
		// = ((long)rw - 410L) * 103422L / 3276L + 103422L - nv_sram.hoehenkorrektur;
		rw = (rw - 410L) * 103422 / 3276L;
		// printf ("dsdx L=%d, H=%d, RW=%ld\n",low,high,rw);
		}
	else
		{
		sprintf(ep,"dsdx015a2r_read ERROR:no data from DSDX015A2R");
		log_append(3,ep);
		goto fehlerende;
		}

ende:
	// sprintf(ep,"sht71_readt (RW=%ld)",rw);
	// log_append(3,ep);
	return(rw);

fehlerende:
	rw = -204800000;
	goto ende;
	}



// *****   all_i2c_chip_hdi0811_read_pressure - Luftdruck eines HDI0811ARY8P3 auslesen   *****

signed long int all_i2c_chip_hdi0811_read_pressure(unsigned char busnummer)
	{
	signed long int rw;
	double f;
	unsigned char t;
	unsigned char low;
	unsigned char high;
	char ep[80];
	double mincount;
	double minmbar;
	double maxcount;
	double maxmbar;
	double diffcount;
	double diffmbar;


	i2c_send_start(busnummer);														// I2C Start senden

	t = i2c_send_byte(busnummer,241,2);										// Byte senden, ACK ermitteln: 3=11110001 = Read from HDI
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"hdi0811_read ERROR:no ACK from HDI0811");
			log_append(3,ep);
      goto fehlerende;
      }


	high = i2c_byte_receive(busnummer);										// Byte holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// Byte holen, ACK senden
	i2c_send_nak(busnummer);

	i2c_send_stop(busnummer);															// I2C Stop senden

	// HDI...R...P3 (baromtetric devices)
	// minimum ( 800 mBar):  2731
	// maximum (1100 mBar): 24575
	mincount = 2731;
	minmbar = 800;
	maxcount = 24575;
	maxmbar = 1100;
	diffcount = maxcount - mincount;
	diffmbar = maxmbar - minmbar;

	if ((low != 255) || (high != 255))
		{
		rw = (high * 256 + low);
		f = ((double)rw - mincount) * (diffmbar / diffcount) + minmbar;
		// printf ("HDI L=%d, H=%d, RW=%ld, %4.2f mBar\n",low,high,rw,f);
		rw = f * 100;																				// in 1/100 mbar umrechnen
		}
	else
		{
		sprintf(ep,"hdi0811_read ERROR:no data from HDI0811");
		log_append(3,ep);
		goto fehlerende;
		}

ende:
	sprintf(ep,"hdi0811_read (RW=%ld)",rw);
	log_append(4,ep);
	return(rw);

fehlerende:
	rw = -204800000;
	goto ende;
	}



// *****   all_i2c_chip_bmp085_read_pressure - Luftdruck eines BMP085 auslesen   *****

signed long int all_i2c_chip_bmp085_read_pressure(unsigned char busnummer)
	{
	signed long int rw;
	double f = -2048000.0;
	unsigned char t;
	unsigned char low;
	unsigned char high;
	unsigned char xlow;
	static signed short int AC1;
	static signed short int AC2;
	static signed short int AC3;
	static unsigned short int AC4;
	static unsigned short int AC5;
	static unsigned short int AC6;
	static signed short int B1;
	static signed short int B2;
	static signed short int MB;
	static signed short int MC;
	static signed short int MD;
	// static unsigned short int FLAG = 0;
	signed long int UP;
	signed long int UT;
	signed long int X1;
	signed long int X2;
	signed long int B5;
	signed long int T = 0;
	signed long int B6;
	signed long int B3;
	signed long int X3;
	unsigned long int B4;
	unsigned long int B7;
	signed long int P = 0;
	char ep[80];



	// -----   Kalibrierungs-Werte einlesen   -----
	// AC1
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK011 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,0xAA,2);										// Byte senden, ACK ermitteln Register Address $AA
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK012 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_start(busnummer);														// I2C Restart senden
	t = i2c_send_byte(busnummer,239,2);										// Byte senden, ACK ermitteln: 3=1110 1111 = Read from BMP085 ($EF)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK013 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	high = i2c_byte_receive(busnummer);										// MSB holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// LSB holen, ACK senden
	i2c_send_nak(busnummer);
	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		AC1 = high * 256 + low;
		}
	else
		{
		sprintf(ep,"bmp085_read ERROR:no valid AC1 data from BMP085");
		log_append(3,ep);
		goto fehlerende;
		}
	// AC2
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK021 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,0xAC,2);										// Byte senden, ACK ermitteln Register Address $AC
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK022 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_start(busnummer);														// I2C Restart senden
	t = i2c_send_byte(busnummer,239,2);										// Byte senden, ACK ermitteln: 3=1110 1111 = Read from BMP085 ($EF)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK023 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	high = i2c_byte_receive(busnummer);										// MSB holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// LSB holen, ACK senden
	i2c_send_nak(busnummer);
	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		AC2 = high * 256 + low;
		}
	else
		{
		sprintf(ep,"bmp085_read ERROR:no valid AC2 data from BMP085");
		log_append(3,ep);
		goto fehlerende;
		}
	// AC3
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK031 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,0xAE,2);										// Byte senden, ACK ermitteln Register Address $AE
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK032 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_start(busnummer);														// I2C Restart senden
	t = i2c_send_byte(busnummer,239,2);										// Byte senden, ACK ermitteln: 3=1110 1111 = Read from BMP085 ($EF)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK033 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	high = i2c_byte_receive(busnummer);										// MSB holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// LSB holen, ACK senden
	i2c_send_nak(busnummer);
	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		AC3 = high * 256 + low;
		}
	else
		{
		sprintf(ep,"bmp085_read ERROR:no valid AC3 data from BMP085");
		log_append(3,ep);
		goto fehlerende;
		}
	// AC4
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK041 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,0xB0,2);										// Byte senden, ACK ermitteln Register Address $B0
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK042 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_start(busnummer);														// I2C Restart senden
	t = i2c_send_byte(busnummer,239,2);										// Byte senden, ACK ermitteln: 3=1110 1111 = Read from BMP085 ($EF)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK043 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	high = i2c_byte_receive(busnummer);										// MSB holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// LSB holen, ACK senden
	i2c_send_nak(busnummer);
	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		AC4 = high * 256 + low;
		}
	else
		{
		sprintf(ep,"bmp085_read ERROR:no valid AC4 data from BMP085");
		log_append(3,ep);
		goto fehlerende;
		}
	// AC5
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK051 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,0xB2,2);										// Byte senden, ACK ermitteln Register Address $B2
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK052 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_start(busnummer);														// I2C Restart senden
	t = i2c_send_byte(busnummer,239,2);										// Byte senden, ACK ermitteln: 3=1110 1111 = Read from BMP085 ($EF)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK053 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	high = i2c_byte_receive(busnummer);										// MSB holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// LSB holen, ACK senden
	i2c_send_nak(busnummer);
	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		AC5 = high * 256 + low;
		}
	else
		{
		sprintf(ep,"bmp085_read ERROR:no valid AC5 data from BMP085");
		log_append(3,ep);
		goto fehlerende;
		}
	// AC6
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK061 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,0xB4,2);									// Byte senden, ACK ermitteln Register Address $B4
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK062 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_start(busnummer);														// I2C Restart senden
	t = i2c_send_byte(busnummer,239,2);										// Byte senden, ACK ermitteln: 3=1110 1111 = Read from BMP085 ($EF)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK063 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	high = i2c_byte_receive(busnummer);										// MSB holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// LSB holen, ACK senden
	i2c_send_nak(busnummer);
	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		AC6 = high * 256 + low;
		}
	else
		{
		sprintf(ep,"bmp085_read ERROR:no valid AC6 data from BMP085");
		log_append(3,ep);
		goto fehlerende;
		}
	// B1
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK071 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,0xB6,2);										// Byte senden, ACK ermitteln Register Address $B6
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK072 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_start(busnummer);														// I2C Restart senden
	t = i2c_send_byte(busnummer,239,2);										// Byte senden, ACK ermitteln: 3=1110 1111 = Read from BMP085 ($EF)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK073 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	high = i2c_byte_receive(busnummer);										// MSB holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// LSB holen, ACK senden
	i2c_send_nak(busnummer);
	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		B1 = high * 256 + low;
		}
	else
		{
		sprintf(ep,"bmp085_read ERROR:no valid B6 data from BMP085");
		log_append(3,ep);
		goto fehlerende;
		}
	// B2
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK081 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,0xB8,2);										// Byte senden, ACK ermitteln Register Address $B8
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK082 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_start(busnummer);														// I2C Restart senden
	t = i2c_send_byte(busnummer,239,2);										// Byte senden, ACK ermitteln: 3=1110 1111 = Read from BMP085 ($EF)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK083 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	high = i2c_byte_receive(busnummer);										// MSB holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// LSB holen, ACK senden
	i2c_send_nak(busnummer);
	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		B2 = high * 256 + low;
		}
	else
		{
		sprintf(ep,"bmp085_read ERROR:no valid B2 data from BMP085");
		log_append(3,ep);
		goto fehlerende;
		}
	// MB
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK091 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,0xBA,2);										// Byte senden, ACK ermitteln Register Address $BA
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK092 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_start(busnummer);														// I2C Restart senden
	t = i2c_send_byte(busnummer,239,2);										// Byte senden, ACK ermitteln: 3=1110 1111 = Read from BMP085 ($EF)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK093 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	high = i2c_byte_receive(busnummer);										// MSB holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// LSB holen, ACK senden
	i2c_send_nak(busnummer);
	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		MB = high * 256 + low;
		}
	else
		{
		sprintf(ep,"bmp085_read ERROR:no valid MB data from BMP085");
		log_append(3,ep);
		goto fehlerende;
		}
	// MC
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK0A1 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,0xBC,2);										// Byte senden, ACK ermitteln Register Address $BC
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK0A2 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_start(busnummer);														// I2C Restart senden
	t = i2c_send_byte(busnummer,239,2);										// Byte senden, ACK ermitteln: 3=1110 1111 = Read from BMP085 ($EF)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK0A3 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	high = i2c_byte_receive(busnummer);										// MSB holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// LSB holen, ACK senden
	i2c_send_nak(busnummer);
	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		MC = high * 256 + low;
		}
	else
		{
		sprintf(ep,"bmp085_read ERROR:no valid MC data from BMP085");
		log_append(3,ep);
		goto fehlerende;
		}
	// MD
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK0B1 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,0xBE,2);										// Byte senden, ACK ermitteln Register Address $BE
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK0B2 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_start(busnummer);														// I2C Restart senden
	t = i2c_send_byte(busnummer,239,2);										// Byte senden, ACK ermitteln: 3=1110 1111 = Read from BMP085 ($EF)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK0B3 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	high = i2c_byte_receive(busnummer);										// MSB holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// LSB holen, ACK senden
	i2c_send_nak(busnummer);
	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		MD = high * 256 + low;
		}
	else
		{
		sprintf(ep,"bmp085_read ERROR:no valid MD data from BMP085");
		log_append(3,ep);
		goto fehlerende;
		}

	// printf("BMP085 AC1:%d, AC2:%d, AC3:%d, AC4:%d, AC5:%d, AC6:%d, B1:%d, B2:%d, MB=%d, MC=%d, MD=%d\n",AC1,AC2,AC3,AC4,AC5,AC6,B1,B2,MB,MC,MD);

	// -----   Luftdruck-Messung starten mit hchstmglicher Genauigkeit   -----
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK501 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,244,2);										// Byte senden, ACK ermitteln Register Address $F4 (Control Register)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK502 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,244,2);										// Byte senden, ACK ermitteln Register Data $F4 (Pressure osrs=3)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK503 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_stop(busnummer);															// I2C Stop senden
	usleep(26000);																				// 26 ms warten (max. 25.5 ms laut Bosch DB Rev. 1.2 von 2009)

	// -----   Luftdruck-Messwert abholen   -----
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK601 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,246,2);										// Byte senden, ACK ermitteln Register Address $F6 (ADC MSB)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK602 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_start(busnummer);														// I2C Restart senden
	t = i2c_send_byte(busnummer,239,2);										// Byte senden, ACK ermitteln: 3=1110 1111 = Read from BMP085 ($EF)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK603 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	high = i2c_byte_receive(busnummer);										// MSB holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// LSB holen, ACK senden
	i2c_send_ack(busnummer);
	xlow = i2c_byte_receive(busnummer);										// XLSB holen, NACK senden
	i2c_send_nak(busnummer);
	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		UP = high * 256 * 8 + low * 8 + xlow / 32;
		// printf ("BMP085 XL=%d, L=%d, H=%d, UP=%ld\n",xlow,low,high,UP);
		}
	else
		{
		sprintf(ep,"bmp085_read ERROR:no valid UP data from BMP085");
		log_append(3,ep);
		goto fehlerende;
		}

	// -----   Temperatur-Messung starten   -----
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK701 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,244,2);										// Byte senden, ACK ermitteln Register Address $F4 (Control Register)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK702 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,46,2);										// Byte senden, ACK ermitteln Register Data $2E (Temperature)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK703 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_stop(busnummer);															// I2C Stop senden
	usleep(5000);																					// 5 ms warten (max. 4.5 ms laut Bosch DB Rev. 1.2 von 2009)

	// -----   Temperatur-Messwert abholen   -----
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,238,2);										// Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK801 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	t = i2c_send_byte(busnummer,246,2);										// Byte senden, ACK ermitteln Register Address $F6 (ADC MSB)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK802 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	i2c_send_start(busnummer);														// I2C Restart senden
	t = i2c_send_byte(busnummer,239,2);										// Byte senden, ACK ermitteln: 3=1110 1111 = Read from BMP085 ($EF)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"bmp085_read ERROR:no ACK803 from BMP085");
			log_append(3,ep);
      goto fehlerende;
      }
	high = i2c_byte_receive(busnummer);										// MSB holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// LSB holen, ACK senden
	i2c_send_nak(busnummer);
	i2c_send_stop(busnummer);															// I2C Stop senden


	if ((low != 255) || (high != 255))
		{
		UT = high * 256 + low;
		// printf ("BMP085 L=%d, H=%d, UT=%ld\n",low,high,UT);
		rw = f * 100;																				// in 1/100 mbar umrechnen
		}
	else
		{
		sprintf(ep,"bmp085_read ERROR:no valid UT data from BMP085");
		log_append(3,ep);
		goto fehlerende;
		}

	// -----   Temperatur ausrechnen   -----
	X1 = (UT - AC6) * AC5 / 32768; // printf("X1:%ld",X1);
	X2 = MC * 2048 / (X1 + MD); // printf(", X2:%ld",X2);
	B5 = X1 + X2; // printf(", B5:%ld\n",B5);
	T = (B5 + 8) / 16;

	// -----   Luftdruck ausrechnen   -----
	B6 = B5 - 4000; // printf("B6:%ld",B6);
	X1 = (B2 * (B6 / 4096)) / 2048; // printf("X1:%ld",X1);
	X2 = AC2 * B6 / 2048; // printf("X2:%ld",X2);
	X3 = X1 + X2; // printf("X3:%ld",X3);
	B3 = ((AC1 * 4 + X3) * 8 + 2) / 4; // printf("B3:%ld",B3);
	X1 = AC3 * B6 / 8192; // printf("X1:%ld",X1);
	X2 = (B1 * (B6 * B6 / 4096)) / 65536; // printf("X2:%ld",X2);
	X3 = ((X1 + X2) + 2) / 4; // printf("X3:%ld",X3);
	B4 = AC4 *(unsigned long)(X3 + 32768) / 32768; // printf("B4:%ld",B4);
	B7 = ((unsigned long)UP - B3) * (50000 / 8); // printf("B7:%ld",B7);
	if (B7 < 0x80000000)
		{
		P = (B7 * 2) / B4;
		}
	else
		{
		P = (B7 / B4) * 2;
		} // printf("P:%ld",P);
	X1 = (P / 256) * (P / 256); // printf("X1:%ld",X1);
	X1 = (X1 * 3038) / 65536; // printf("X1:%ld",X1);
	X2 = (-7357 * P) / 65536; // printf("X2:%ld\n",X2);
	P = P + (X1 + X2 + 3791) / 16;

// ende:
	f = (double)T / 10.0;
	// printf("BMP085 komp.Temperatur: %4.2f Grad (T=%ld)\n",f,T);
	f = (double)P / 100.0;
	// printf("BMP085 komp.Druck: %4.2f mBar (P=%ld)\n",f,P);
	rw = P;
	// sprintf(ep,"bmp085_read (RW=%ld)",rw);
	// log_append(3,ep);
endeende:
	return(rw);

fehlerende:
	i2c_send_stop(busnummer);															// I2C Stop senden
	rw = -204800000;
	goto endeende;
	}



// *****   all_i2c_chip_sht71_read_temp - Temperaturwert eines SHT71 auslesen   *****

signed long int all_i2c_chip_sht71_read_temp(unsigned char busnummer)
	{
	signed long int rw;
	unsigned char t;
	unsigned char low;
	unsigned char high;
	unsigned char crc;
	char ep[80];


	i2c_send_start_sht(busnummer);												// I2C SHT Start senden

	t = i2c_send_byte(busnummer,3,2);											// Byte senden, ACK ermitteln: 3=00000011 = Write to ID #0 (SHT71)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"sht71_readt ERROR:no ACK from SHT71");
			log_append(3,ep);
      goto fehlerende;
      }


	// usleep(242000);																				// 242 ms auf Abschluss der Messung warten
	usleep(285000);																				// 285 ms auf Abschluss der Messung warten

	high = i2c_byte_receive(busnummer);										// Byte holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// Byte holen, ACK senden
	i2c_send_ack(busnummer);
	crc = i2c_byte_receive(busnummer);										// Byte holen, NACK senden
	i2c_send_nak(busnummer);
	// beim Lesen von Bytes knnen Fehler nicht erkannt werden,
	// darum hier auch keine berprfung. Falls der angesprochene
	// Chip nicht antwortet, bekommen wir 255 als Wert zurck.

	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		// printf ("L=%d, H=%d, CRC=%d\n",low,high,crc);
		rw = ((signed long int)high << 8) + (signed long int)low - 3960;
		// rw = ((signed long int)high * 256) + (signed long int)low - 3960;
		}
	else
		{
		sprintf(ep,"sht71_readt ERROR:no data from SHT71");
		log_append(3,ep);
		goto fehlerende;
		}

ende:
	// sprintf(ep,"sht71_readt (RW=%ld)",rw);
	// log_append(3,ep);
	return(rw);

fehlerende:
	rw = -204800000;
	goto ende;
	}



// *****   all_i2c_chip_sht71_read_humidity - Luftfeuchtewert eines SHT71 auslesen   *****

signed long int all_i2c_chip_sht71_read_humidity(unsigned char busnummer)
	{
	signed long int rw;
	signed long int sw;
	unsigned char t;
	unsigned char low;
	unsigned char high;
	unsigned char crc;
	char ep[80];


	i2c_send_start_sht(busnummer);												// I2C SHT Start senden

	t = i2c_send_byte(busnummer,5,2);											// Byte senden, ACK ermitteln: 5=00000101 = read from SHT71 RH
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"sht71_readh ERROR:no ACK from SHT71");
			log_append(3,ep);
      goto fehlerende;
      }

	// usleep(64000);																				// 64 ms auf Abschluss der Messung warten
	usleep(85000);																				// 85 ms auf Abschluss der Messung warten

	high = i2c_byte_receive(busnummer);										// Byte holen, ACK senden
	i2c_send_ack(busnummer);
	low = i2c_byte_receive(busnummer);										// Byte holen, ACK senden
	i2c_send_ack(busnummer);
	crc = i2c_byte_receive(busnummer);										// Byte holen, NACK senden
	i2c_send_nak(busnummer);
	// beim Lesen von Bytes knnen Fehler nicht erkannt werden,
	// darum hier auch keine berprfung. Falls der angesprochene
	// Chip nicht antwortet, bekommen wir 255 als Wert zurck.

	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		sw = ((signed long int)high << 8) + (signed long int)low;

		if (sw <= 1712 )
			{
			rw = 143 * sw;																		// result = a * sensor_out
			if (rw < 8192) rw = 8192;													// check for underflow
			rw -= 8192;																				// result = result + b
			}
		else
			{
			rw = 111 * sw + 46288;														// result = a * sensor_out + b
			if (rw > 409600) rw = 409600;											// check for overflow (optional)
			}

		rw = rw * 20 / 819;																	// result = result /40.96
		// printf ("L=%d, H=%d, CRC=%d sw=%ld, rw=%ld\n",low,high,crc,sw,rw);
		}
	else
		{
		sprintf(ep,"sht71_readh ERROR:no data from SHT71");
		log_append(3,ep);
		goto fehlerende;
		}

	return(rw);
fehlerende:
	rw = -204800000;
	return(rw);
	}



// *****   all_i2c_chip_sht71_read_temp_nw - Temperaturwert eines SHT71 auslesen (noWait Version)   *****

signed long int all_i2c_chip_sht71_read_temp_nw(unsigned char busnummer)
	{
	signed long int rw;
	unsigned char t;
	unsigned char low;
	unsigned char high;
	unsigned char crc;
	char ep[80];
	static signed char m_status = 0;
	static signed long int m_value = -204800000;


	if(physical_measurementtype_rotator != 0)
		{
		rw = m_value;
		m_status = 0;
		goto ende;
		}

	if (m_status == 0)
		{
		// Status 0: Start muss gegeben werden, letzter Mewert als Rckwert
		i2c_send_start_sht(busnummer);											// I2C SHT Start senden

		t = i2c_send_byte(busnummer,3,2);										// Byte senden, ACK ermitteln: 3=00000011 = Write to ID #0 (SHT71)
		if (t != 0)																					// 0 -ACK, 1 -NACK
			{
			sprintf(ep,"sht71_readt_nw ERROR:no ACK from SHT71");
			log_append(3,ep);
     	goto fehlerende;
			m_status = 0;
     	}
		m_status = 1;
		rw = m_value;																				// letzten Mewert zurckliefern
		goto ende;		
		}
	else
		{
		// Status nicht 0: Mewert abholen und zurckliefern
		// usleep(285000);																		// 285 ms auf Abschluss der Messung warten
		high = i2c_byte_receive(busnummer);										// Byte holen, ACK senden
		i2c_send_ack(busnummer);
		low = i2c_byte_receive(busnummer);										// Byte holen, ACK senden
		i2c_send_ack(busnummer);
		crc = i2c_byte_receive(busnummer);										// Byte holen, NACK senden
		i2c_send_nak(busnummer);
		// beim Lesen von Bytes knnen Fehler nicht erkannt werden,
		// darum hier auch keine berprfung. Falls der angesprochene
		// Chip nicht antwortet, bekommen wir 255 als Wert zurck.

		i2c_send_stop(busnummer);															// I2C Stop senden

		if ((low != 255) || (high != 255))
			{
			// printf ("L=%d, H=%d, CRC=%d\n",low,high,crc);
			rw = ((signed long int)high << 8) + (signed long int)low - 3960;
			m_status = 0;
			goto ende;
			}
		else
			{
			sprintf(ep,"sht71_readt_nw ERROR:no data from SHT71");
			log_append(3,ep);
			m_status = 0;
			goto fehlerende;
			}
		}

ende:
	// sprintf(ep,"sht71_readt_nw (mrt=%d,m_status=%d, RW=%ld)",physical_measurementtype_rotator,m_status,rw);
	// log_append(3,ep);
	m_value = rw;
	return(rw);

fehlerende:
	rw = -204800000;
	goto ende;
	}



// *****   all_i2c_chip_sht71_read_humidity_nw - Luftfeuchtewert eines SHT71 auslesen (noWait Version)   *****

signed long int all_i2c_chip_sht71_read_humidity_nw(unsigned char busnummer)
	{
	signed long int rw;
	signed long int sw;
	unsigned char t;
	unsigned char low;
	unsigned char high;
	unsigned char crc;
	char ep[80];
	static signed char m_status = 0;
	static signed long int m_value = -204800000;


	if(physical_measurementtype_rotator != 1)
		{
		rw = m_value;
		m_status = 0;
		goto ende;
		}

	if (m_status == 0)
		{
		// Status 0: Start muss gegeben werden, letzter Mewert als Rckwert
		i2c_send_start_sht(busnummer);											// I2C SHT Start senden

		t = i2c_send_byte(busnummer,5,2);										// Byte senden, ACK ermitteln: 5=00000101 = read from SHT71 RH
    	if (t != 0)																				// 0 -ACK, 1 -NACK
    		{
				sprintf(ep,"sht71_readh_nw ERROR:no ACK from SHT71");
				log_append(3,ep);
				m_status = 0;
      	goto fehlerende;
      	}
		m_status = 1;
		rw = m_value;
		goto ende;
		}
	else
		{
		// Status nicht 0: Mewert abholen und zurckliefern
		// usleep(85000);																		// 85 ms auf Abschluss der Messung warten

		high = i2c_byte_receive(busnummer);									// Byte holen, ACK senden
		i2c_send_ack(busnummer);
		low = i2c_byte_receive(busnummer);									// Byte holen, ACK senden
		i2c_send_ack(busnummer);
		crc = i2c_byte_receive(busnummer);									// Byte holen, NACK senden
		i2c_send_nak(busnummer);
		// beim Lesen von Bytes knnen Fehler nicht erkannt werden,
		// darum hier auch keine berprfung. Falls der angesprochene
		// Chip nicht antwortet, bekommen wir 255 als Wert zurck.

		i2c_send_stop(busnummer);														// I2C Stop senden

		if ((low != 255) || (high != 255))
			{
			sw = ((signed long int)high << 8) + (signed long int)low;

			if (sw <= 1712 )
				{
				rw = 143 * sw;																	// result = a * sensor_out
				if (rw < 8192) rw = 8192;												// check for underflow
				rw -= 8192;																			// result = result + b
				}
			else
				{
				rw = 111 * sw + 46288;													// result = a * sensor_out + b
				if (rw > 409600) rw = 409600;										// check for overflow (optional)
				}

			rw = rw * 20 / 819;																// result = result /40.96
			// printf ("L=%d, H=%d, CRC=%d sw=%ld, rw=%ld\n",low,high,crc,sw,rw);
			m_status = 0;
			goto ende;
			}
		else
			{
			sprintf(ep,"sht71_readh_nw ERROR:no data from SHT71");
			log_append(3,ep);
			m_status = 0;
			goto fehlerende;
			}

		}

ende:
	// sprintf(ep,"sht71_readh_nw (mrt=%d, m_status=%d, RW=%ld)",physical_measurementtype_rotator,m_status,rw);
	// log_append(3,ep);
	m_value = rw;
	return(rw);

fehlerende:
	rw = -204800000;
	goto ende;
	}



// *****   all_i2c_chip_all3032_read_brightness - Helligkeit eines ALL3032 auslesen   *****

signed long int all_i2c_chip_all3032_read_brightness(unsigned char busnummer)
	{
	signed long int rw;
	unsigned char t;
	unsigned char low;
	unsigned char high;
	char ep[80];


	i2c_send_start(busnummer);														// I2C Start senden

	t = i2c_send_byte(busnummer,55,2);										// 0011 1001 = Read from ID #54 (Tiny85)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"all3032_readb ERROR:no ack1 from ALL3032");
			log_append(3,ep);
      goto fehlerende;
      }

	high = i2c_byte_receive(busnummer);										// Byte holen
	i2c_send_ack(busnummer);															// ACK senden
	low = i2c_byte_receive(busnummer);										// Byte holen
	i2c_send_nak(busnummer);															// NACK senden
	// beim Lesen von Bytes knnen Fehler nicht erkannt werden,
	// darum hier auch keine berprfung. Falls der angesprochene
	// Chip nicht antwortet, bekommen wir 255 als Wert zurck.

	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		// printf ("L=%d, H=%d\n",low,high);
		rw = ((long)high * 256L + (long)low);
		// printf("ALL3032: %ld\n",rw);
		}
	else
		{
		sprintf(ep,"all3032_readb ERROR:no data from ALL3032");
		log_append(3,ep);
 		goto fehlerende;
		}

	return(rw);
fehlerende:
	rw = -204800000;
	return(rw);
	}



// *****   all_i2c_chip_all3025_read_counter - Zherstand eines ALL3025 auslesen   *****

signed long int all_i2c_chip_all3025_read_counter(unsigned char busnummer)
	{
	signed long int rw;
	unsigned char t;
	unsigned char low;
	unsigned char mid;
	unsigned char high;
	char ep[80];


	i2c_send_start(busnummer);														// I2C Start senden

	t = i2c_send_byte(busnummer,162,2);										// 1010 0010 = Write to RTC #1
	if (t != 0)																						// 0 -ACK, 1 -NACK
		{
		sprintf(ep,"all3025_readc ERROR:no ack1 from PCF8583:1");
		log_append(3,ep);
		goto fehlerende;
		}
	t = i2c_send_byte(busnummer,0,2);											// 0000 0000 = Register 0: Control/Status
	if (t != 0)																						// 0 -ACK, 1 -NACK
		{
		sprintf(ep,"all3025_readc ERROR:no ack2 from PCF8583:1");
		log_append(3,ep);
		goto fehlerende;
		}
	t = i2c_send_byte(busnummer,32,2);										// 0010 0000 = Event Counter Mode
	if (t != 0)																						// 0 -ACK, 1 -NACK
		{
		sprintf(ep,"all3025_readc ERROR:no ack3 from PCF8583:1");
		log_append(3,ep);
		goto fehlerende;
		}

	i2c_clock_high(busnummer);														// Clock auf High setzen und etwas warten,
	i2c_delay(3);																					// damit wir dann Start senden knnen...
	i2c_send_start(busnummer);														// Start senden (Restart)
	t = i2c_send_byte(busnummer,163,2);										// 1010 0011 = Read from RTC #1
	if (t != 0)																						// 0 -ACK, 1 -NACK
		{
		sprintf(ep,"all3025_readc ERROR:no ack4 from PCF8583:1");
		log_append(3,ep);
		goto fehlerende;
		}

	low = i2c_byte_receive(busnummer);										// Byte holen
	i2c_send_ack(busnummer);															// NACK senden
	mid = i2c_byte_receive(busnummer);										// Byte holen
	i2c_send_ack(busnummer);															// ACK senden
	high = i2c_byte_receive(busnummer);										// Byte holen
	i2c_send_nak(busnummer);															// ACK senden
	// beim Lesen von Bytes knnen Fehler nicht erkannt werden,
	// darum hier auch keine berprfung. Falls der angesprochene
	// Chip nicht antwortet, bekommen wir 255 als Wert zurck.

	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (mid != 255) || (high != 255))
		{
		rw = (long)bcd2dez(high) * 10000L + (long)bcd2dez(mid) * 100L + (long)bcd2dez(low);
		// printf ("ALL3025: L=%d, M:%d, H=%d, RW:%ld\n",low,mid,high,rw);
		}
	else
		{
		sprintf(ep,"all3025_readc ERROR:no data from PCF8583:1");
		log_append(3,ep);
 		goto fehlerende;
		}

	return(rw);
fehlerende:
	rw = -204800000;
	return(rw);
	}



// *****   all_i2c_chip_all4035_read_counter - Zherstand eines ALL4035 auslesen   *****

signed long int all_i2c_chip_all4035_read_counter(unsigned char busnummer)
	{
	signed long int rw;
	unsigned char t;
	unsigned char low;
	unsigned char mid;
	unsigned char high;
	char ep[80];


	i2c_send_start(busnummer);														// I2C Start senden

	t = i2c_send_byte(busnummer,160,2);										// 1010 0000 = Write to RTC #0
	if (t != 0)																						// 0 -ACK, 1 -NACK
		{
		sprintf(ep,"all4035_readc ERROR:no ack1 from PCF8583:0");
		log_append(3,ep);
		goto fehlerende;
		}
	t = i2c_send_byte(busnummer,0,2);											// 0000 0000 = Register 0: Control/Status
	if (t != 0)																						// 0 -ACK, 1 -NACK
		{
		sprintf(ep,"all4035_readc ERROR:no ack2 from PCF8583:0");
		log_append(3,ep);
		goto fehlerende;
		}
	t = i2c_send_byte(busnummer,32,2);										// 0010 0000 = Event Counter Mode
	if (t != 0)																						// 0 -ACK, 1 -NACK
		{
		sprintf(ep,"all4035_readc ERROR:no ack3 from PCF8583:0");
		log_append(3,ep);
		goto fehlerende;
		}

	i2c_clock_high(busnummer);														// Clock auf High setzen und etwas warten,
	i2c_delay(3);																					// damit wir dann Start senden knnen...
	i2c_send_start(busnummer);														// Start senden (Restart)
	t = i2c_send_byte(busnummer,161,2);										// 1010 0001 = Read from RTC #0
	if (t != 0)																						// 0 -ACK, 1 -NACK
		{
		sprintf(ep,"all4035_readc ERROR:no ack4 from PCF8583:0");
		log_append(3,ep);
		goto fehlerende;
		}

	low = i2c_byte_receive(busnummer);										// Byte holen
	i2c_send_ack(busnummer);															// NACK senden
	mid = i2c_byte_receive(busnummer);										// Byte holen
	i2c_send_ack(busnummer);															// ACK senden
	high = i2c_byte_receive(busnummer);										// Byte holen
	i2c_send_nak(busnummer);															// ACK senden
	// beim Lesen von Bytes knnen Fehler nicht erkannt werden,
	// darum hier auch keine berprfung. Falls der angesprochene
	// Chip nicht antwortet, bekommen wir 255 als Wert zurck.

	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (mid != 255) || (high != 255))
		{
		rw = (long)bcd2dez(high) * 10000L + (long)bcd2dez(mid) * 100L + (long)bcd2dez(low);
		// printf ("ALL4035: L=%d, M:%d, H=%d, RW:%ld\n",low,mid,high,rw);
		}
	else
		{
		sprintf(ep,"all4035_readc ERROR:no data from PCF8583:0");
		log_append(3,ep);
 		goto fehlerende;
		}

	return(rw);
fehlerende:
	rw = -204800000;
	return(rw);
	}



// *****   all_i2c_chip_all3022_read_voltage - Spannung eines ALL3022 auslesen   *****

signed long int all_i2c_chip_all3022_read_voltage(unsigned char busnummer,unsigned char channel)
	{
	signed long int rw;
	unsigned char t;
	unsigned char low;
	unsigned char high;
	char ep[80];


	if (channel == 0)
		{
		// printf("ALL3022 test0\n");

		// Low-Voltage Eingang prfen
		i2c_send_start(busnummer);													// I2C Start senden
		t = i2c_send_byte(busnummer,104,2);									// 01101000 = Write to MAX1237EUA
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"all3022_readvl ERROR:no ack1 from ALL3022");
			log_append(3,ep);
      goto fehlerende;
      }
		t = i2c_send_byte(busnummer,242,2);									// 11110010 = Setup Byte unipolar
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"all3022_readvl ERROR:no ack2 from ALL3022");
			log_append(3,ep);
      goto fehlerende;
      }
		t = i2c_send_byte(busnummer,97,2);									// 01100001 = Configuration Byte Kanal 0 se
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"all3022_readvl ERROR:no ack3 from ALL3022");
			log_append(3,ep);
      goto fehlerende;
      }

		i2c_clock_high(busnummer);													// Clock auf High setzen und etwas warten,
		usleep(30000);																			// 30 ms warten, damit wir dann Start senden knnen...

		i2c_send_start(busnummer);													// Start senden (Restart)

		t = i2c_send_byte(busnummer,105,2);									// 01101001 = Read from MAX1237EUA
		if (t != 0)																					// 0 -ACK, 1 -NACK
			{
			sprintf(ep,"all3022_readvl ERROR:no ack4 from ALL3022");
			log_append(3,ep);
			goto fehlerende;
			}

		usleep(30000);																			// 30 ms warten, damit die Umwandlung abgeschlossen ist

		high = i2c_byte_receive(busnummer);									// Byte holen
		i2c_send_ack(busnummer);														// ACK senden
		low = i2c_byte_receive(busnummer);									// Byte holen
		i2c_send_ack(busnummer);														// ACK senden
		i2c_send_stop(busnummer);														// I2C Stop senden

		if ((low != 255) || (high != 255))
			{
			// printf ("L0=%d, H0=%d\n",low,high);
			rw = (signed long int)(high & 15) * 256;
			rw += low;																				// Widerstandsteiler 190K/10K dann stimmts
			rw -= 2047;																				// Nulloffset subtrahieren
			rw *= 2;																					// LV-Eingang 2
			rw -= 50;																					// 0.50 VDC Offset Abziehen
			// printf("ALL3022L: %ld\n",rw);
			}
		else
			{
			sprintf(ep,"all3022_readvl ERROR:no data from ALL3022");
			log_append(3,ep);
	 		goto fehlerende;
			}
		goto endeende;
		}


	if (channel == 1)
		{
		// printf("ALL3022 test1\n");

		// High-Voltage Eingang prfen
		i2c_send_start(busnummer);													// I2C Start senden
		t = i2c_send_byte(busnummer,104,2);									// 01101000 = Write to MAX1237EUA
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"all3022_readvh ERROR:no ack1 from ALL3022");
			log_append(3,ep);
      goto fehlerende;
      }
		t = i2c_send_byte(busnummer,242,2);									// 11110010 = Setup Byte unipolar
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"all3022_readvh ERROR:no ack2 from ALL3022");
			log_append(3,ep);
      goto fehlerende;
      }
		t = i2c_send_byte(busnummer,99,2);									// 01100011 = Configuration Byte Kanal 1 se
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"all3022_readvh ERROR:no ack3 from ALL3022");
			log_append(3,ep);
      goto fehlerende;
      }

		i2c_clock_high(busnummer);													// Clock auf High setzen und etwas warten,
		usleep(30000);																			// 30 ms warten, damit wir dann Start senden knnen...

		i2c_send_start(busnummer);													// Start senden (Restart)

		t = i2c_send_byte(busnummer,105,2);									// 01101001 = Read from MAX1237EUA
		if (t != 0)																					// 0 -ACK, 1 -NACK
			{
			sprintf(ep,"all3022_readvh ERROR:no ack4 from ALL3022");
			log_append(3,ep);
			goto fehlerende;
			}

		usleep(30000);																			// 30 ms warten, damit die Umwandlung abgeschlossen ist

		high = i2c_byte_receive(busnummer);									// Byte holen
		i2c_send_ack(busnummer);														// ACK senden
		low = i2c_byte_receive(busnummer);									// Byte holen
		i2c_send_ack(busnummer);														// ACK senden
		i2c_send_stop(busnummer);														// I2C Stop senden

		if ((low != 255) || (high != 255))
			{
			// printf ("L0=%d, H0=%d\n",low,high);
			rw = (signed long int)(high & 15) * 256;
			rw += low;																				// Widerstandsteiler 1990K/10K dann stimmts
			rw -= 2047;																				// Nulloffset subtrahieren
			rw *= 20;																					// HV-Eingang 20
			// printf("ALL3022H: %ld\n",rw);
			if (rw <= -40299)
				{
				sprintf(ep,"all3022_readvh ERROR:invalid HV data from ALL3022 (maybe its an ALL3021 ?)");
				log_append(3,ep);
	 			goto fehlerende;
				}
			}
		else
			{
			sprintf(ep,"all3022_readvh ERROR:no data from ALL3022");
			log_append(3,ep);
	 		goto fehlerende;
			}
		goto endeende;
		}


	return(-204800000);
fehlerende:
	rw = -204800000;
endeende:
	return(rw);
	}



// *****   all_i2c_chip_all4023_read_voltage - Spannung eines ALL4023 auslesen   *****

signed long int all_i2c_chip_all4023_read_voltage(unsigned char busnummer,unsigned char channel)
	{
	signed long int rw;
	unsigned char t;
	unsigned char low[3];
	unsigned char high[3];
	char ep[80];
	static long int spannung[3] = {-204800000,-204800000,-204800000};

	if (channel == 1)
		{
		rw = spannung[1];
		goto endeende;
		}
	if (channel == 2)
		{
		rw = spannung[2];
		goto endeende;
		}

	spannung[0] = -204800000;
	spannung[1] = -204800000;
	spannung[2] = -204800000;

	i2c_send_start(busnummer);														// I2C Start senden

	t = i2c_send_byte(busnummer,53,2);										// 0011 1001 = Read from ID #52 (Tiny85)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"all4023_readv ERROR:no ack1 from ALL4023");
			log_append(3,ep);
      goto fehlerende;
      }

	high[0] = i2c_byte_receive(busnummer);								// Byte holen
	i2c_send_ack(busnummer);															// ACK senden
	low[0] = i2c_byte_receive(busnummer);									// Byte holen
	i2c_send_nak(busnummer);															// NACK senden
	high[1] = i2c_byte_receive(busnummer);								// Byte holen
	i2c_send_ack(busnummer);															// ACK senden
	low[1] = i2c_byte_receive(busnummer);									// Byte holen
	i2c_send_nak(busnummer);															// NACK senden
	high[2] = i2c_byte_receive(busnummer);								// Byte holen
	i2c_send_ack(busnummer);															// ACK senden
	low[2] = i2c_byte_receive(busnummer);									// Byte holen
	i2c_send_nak(busnummer);															// NACK senden

	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low[0] != 255) || (high[0] != 255))
		{
		// printf ("L0=%d, H0=%d\n",low[0],high[0]);
		rw = ((long)high[0] * 256L + (long)low[0]);
		spannung[0] = rw;
		// printf("ALL4023 V0: %ld\n",rw);
		spannung[1] = ((long)high[1] * 256L + (long)low[1]);
		spannung[2] = ((long)high[2] * 256L + (long)low[2]);
		}
	else
		{
		sprintf(ep,"all4023_readv ERROR:no data from ALL4023");
		log_append(3,ep);
 		goto fehlerende;
		}

	return(rw);
fehlerende:
	rw = -204800000;
endeende:
	return(rw);
	}



// *****   all_i2c_chip_all4033_read_voltage - Spannung eines ALL4033 auslesen   *****

signed long int all_i2c_chip_all4033_read_voltage(unsigned char busnummer)
	{
	signed long int rw;
	unsigned char t;
	unsigned char low;
	unsigned char high;
	char ep[80];


	i2c_send_start(busnummer);														// I2C Start senden

	t = i2c_send_byte(busnummer,57,2);										// 0011 1001 = Read from ID #4 (Tiny85)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"all4033_readv ERROR:no ack1 from ALL4033");
			log_append(3,ep);
      goto fehlerende;
      }

	high = i2c_byte_receive(busnummer);										// Byte holen
	i2c_send_ack(busnummer);															// ACK senden
	low = i2c_byte_receive(busnummer);										// Byte holen
	i2c_send_nak(busnummer);															// NACK senden
	// beim Lesen von Bytes knnen Fehler nicht erkannt werden,
	// darum hier auch keine berprfung. Falls der angesprochene
	// Chip nicht antwortet, bekommen wir 255 als Wert zurck.

	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		// printf ("L=%d, H=%d\n",low,high);
		rw = ((long)high * 256L + (long)low) * 10;
		// printf("ALL4033: %ld\n",rw);
		}
	else
		{
		sprintf(ep,"all4033_readv ERROR:no data from ALL4033");
		log_append(3,ep);
 		goto fehlerende;
		}

	return(rw);
fehlerende:
	rw = -204800000;
	return(rw);
	}



// *****   all_i2c_chip_lm75_read_temp - Temperaturwert eines LM75 auslesen   *****

signed long int all_i2c_chip_lm75_read_temp(unsigned char busnummer,unsigned char address)
	{
	signed long int rw;
	unsigned char t;
	unsigned char low;
	unsigned char high;
	char ep[80];


	i2c_send_start(busnummer);														// I2C Start senden

	t = i2c_send_byte(busnummer,145 + address * 2,2);			// Byte senden, ACK ermitteln: 145=10010001 = Read from ID #address (LM75)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"lm75_readt ERROR:no ack1 from LM75");
			log_append(3,ep);
      goto fehlerende;
      }

	high = i2c_byte_receive(busnummer);										// Byte holen
	i2c_send_ack(busnummer);															// ACK senden
	low = i2c_byte_receive(busnummer);										// Byte holen
	i2c_send_nak(busnummer);															// NACK senden
	// beim Lesen von Bytes knnen Fehler nicht erkannt werden,
	// darum hier auch keine berprfung. Falls der angesprochene
	// Chip nicht antwortet, bekommen wir 255 als Wert zurck.

	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		// printf ("L=%d, H=%d\n",byte_l,byte_h);
		rw = (signed long int)(high & 127) * 2;
		if ((low & 128) != 0) rw++;
		if (high >= 128)
			{
			rw -= 256;	// negative Temperatur
			}
		rw = rw * 50;
		}
	else
		{
		sprintf(ep,"lm75_readt ERROR:no data from LM75");
		log_append(3,ep);
 		goto fehlerende;
		}

	return(rw);
fehlerende:
	rw = -204800000;
	return(rw);
	}



// *****   all_i2c_chip_lm92_read_temp - Temperaturwert eines LM92 auslesen   *****

signed long int all_i2c_chip_lm92_read_temp(unsigned char busnummer,unsigned char address)
	{
	signed long int rw;
	unsigned char t;
	unsigned char low;
	unsigned char high;
	char ep[80];


	i2c_send_start(busnummer);														// I2C Start senden

	t = i2c_send_byte(busnummer,144 + address * 2,2);			// Byte senden, ACK ermitteln: 144=10010000 = Write to ID #address (LM92)
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"lm92_readt ERROR:no ack1 from LM92");
			log_append(3,ep);
      goto fehlerende;
      }

 	i2c_delay(25);
	t = i2c_send_byte(busnummer,0,2);											// Byte senden, ACK ermitteln: 0=0000000 = Register 0: Temperatur
    if (t != 0)																					// 0 -ACK, 1 -NACK
    	{
			sprintf(ep,"lm92_readt ERROR:no ack2 from LM92");
			log_append(3,ep);
      goto fehlerende;
      }

	i2c_send_stop(busnummer);															// I2C Stop senden
 	i2c_delay(25);

	i2c_send_start(busnummer);

	t = i2c_send_byte(busnummer,145 + address * 2,2);			// Byte senden, ACK ermitteln: 145=10010001 = Read from ID #address (LM92)
  if (t != 0)																						// 1 -NACK, 0 -ACK
		{
		sprintf(ep,"lm92_readt ERROR:no ack3 from LM92");
		log_append(3,ep);
    goto fehlerende;
    }

	high = i2c_byte_receive(busnummer);										// Byte holen
	i2c_send_ack(busnummer);															// ACK senden
	low = i2c_byte_receive(busnummer);										// Byte holen
	i2c_send_nak(busnummer);															// NACK senden
	// beim Lesen von Bytes knnen Fehler nicht erkannt werden,
	// darum hier auch keine berprfung. Falls der angesprochene
	// Chip nicht antwortet, bekommen wir 255 als Wert zurck.

	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((low != 255) || (high != 255))
		{
		// printf ("L=%d, H=%d\n",low,high);
		rw = (signed long int)(high & 127) * 32;
		rw += low >> 3;
		if (high >= 128)
			{
			rw -= 4096; // negative Temperatur
			}
		rw = rw * 6 + rw / 4; // * 6.25
		}
	else
		{
		sprintf(ep,"lm92_readt_np ERROR:no data from LM92");
		log_append(3,ep);
		goto fehlerende;
		}

	return(rw);
fehlerende:
	i2c_send_stop(busnummer);															// I2C Stop senden
	rw = -204800000;
	return(rw);
	}




// *****   all_i2c_chip_ads1100_read - Mewert eines ADS1100 auslesen   *****

signed long int all_i2c_chip_ads1100_read(unsigned char busnummer,unsigned char address,unsigned char bits)
	{
	signed long int rw;
	unsigned char t;
	unsigned char low;
	unsigned char high;
	unsigned char status;
	char ep[80];
	signed short int x;


	i2c_send_start(busnummer);

	t = i2c_send_byte(busnummer,144 + address * 2,2);			// Byte senden, ACK ermitteln: 145=10010xx0 = Write to ID #address (ADS1100_ADx)

  if (t != 0)																						// 1 -NACK, 0 -ACK
		{
		sprintf(ep,"ads1100_read ERROR:no ack0 from ADS1100_AD%d",address);
		log_append(3,ep);
    goto fehlerende;
    }

	if(bits == 16)
		{
		// 16(15) bit 8 sps
		t = i2c_send_byte(busnummer,156,2);									// Byte senden, ACK ermitteln: 1001 1100 DR1:1,DR0:1
		}
	if(bits == 14)
		{
		// 14(13) bit 32 sps
		t = i2c_send_byte(busnummer,148,2);									// Byte senden, ACK ermitteln: 1001 0100 DR1:0,DR0:1
		}
  if (t != 0)																						// 1 -NACK, 0 -ACK
		{
		sprintf(ep,"ads1100_read ERROR:no ack1 from ADS1100_AD%d",address);
		log_append(3,ep);
    goto fehlerende;
    }
	i2c_send_stop(busnummer);


	for(x = 0;x < 15;x++)
		{
		usleep(10000);																				// 10 ms pro Loop warten
		i2c_send_start(busnummer);
		t = i2c_send_byte(busnummer,145 + address * 2,2);			// Byte senden, ACK ermitteln: 145=10010xx1 = Read from ID #address (ADS1100_ADx)
  	if (t != 0)																						// 1 -NACK, 0 -ACK
			{
			sprintf(ep,"ads1100_read ERROR:no ack from ADS1100_AD%d",address);
			log_append(3,ep);
    	goto fehlerende;
    	}

		high = i2c_byte_receive(busnummer);										// High-Byte holen
		i2c_send_ack(busnummer);															// ACK senden
		low = i2c_byte_receive(busnummer);										// Low-Byte holen
		i2c_send_ack(busnummer);															// NACK senden
		status = i2c_byte_receive(busnummer);									// Status-Byte holen
		i2c_send_nak(busnummer);															// NACK senden
		i2c_send_stop(busnummer);															// I2C Stop senden
		if(status < 128) goto fertig;													// wenn Bit 7 L ist, ist der Messwert gltig
		}
	goto fehlerende;

fertig:
	if ((low != 255) || (high != 255))
		{
		rw = (signed long int)high * 255 + (signed long int)low;
		// printf ("ADS1100_%d: H=%d, L=%d, 16bit:%ld, Status:%d\n",address,high,low,rw,status);
		}
	else
		{
		sprintf(ep,"ads1100_read ERROR:no data from ADS1100_AD%d",address);
		log_append(3,ep);
		goto fehlerende;
		}

	return(rw);
fehlerende:
	i2c_send_stop(busnummer);															// I2C Stop senden
	rw = -204800000;
	return(rw);
	}




// *****   all_i2c_chip_pcf8574_read -8-bit Eingang lesen   *****

signed long int all_i2c_chip_pcf8574_read(unsigned char busnummer,unsigned char address)
	{
	signed long int rw = 0;
	unsigned char i2c_id;
	char ep[80];
	unsigned char t;


	i2c_id = 64 + address * 2;														// Adresse kann 0...7 sein PCF8574

	i2c_send_start(busnummer);														// I2C Start senden

	t = i2c_send_byte(busnummer,i2c_id + 1,2);						// Byte senden, ACK ermitteln: 64=0100 ttt1 = Read from ID #t (PCF8574)
  if (t != 0)																						// 1 -NACK, 0 -ACK
	 	{
		sprintf(ep,"pcf8574_read ERROR:no ack from PCF8574_%d",address);
		log_append(3,ep);
    goto fehlerende;
    }

	rw = i2c_byte_receive(busnummer);											// Byte holen, ACK senden
	if (rw < 0) goto fehlerende;													// Werte knnen nur zwischen 0...255 liegen

	i2c_send_stop(busnummer);															// I2C Stop senden

	// printf("read8574(bus%d,id%d)=%d\n",busnummer,address,rw);
	return(rw);																						// >0: alles OK

fehlerende:
	i2c_send_stop(busnummer);															// I2C Stop senden
	rw = -1;																							// -1 zeigt an, da etwas nicht stimmt
	return(rw);
	}




// *****   all_i2c_chip_pcf8574_write -8-bit Ausgang setzen   *****

unsigned char all_i2c_chip_pcf8574_write(unsigned char busnummer,unsigned char address,unsigned char wert)
	{
	unsigned char rw = 0;
	unsigned char i2c_id;
	unsigned char t;
	char ep[80];


	i2c_id = 64 + address * 2;														// Adresse kann 0...7 sein PCF8574


	i2c_send_start(busnummer);														// I2C Start senden

	t = i2c_send_byte(busnummer,i2c_id,2);								// Byte senden, ACK ermitteln: 64=0100 ttt0 = Write to ID #t (PCF8574)
    if (t != 0)																					// 1 -NACK, 0 -ACK
    	{
			sprintf(ep,"pcf8574_write ERROR:no ack1 from PCF8574_%d",address);
			log_append(3,ep);
       goto fehlerende;
       }

	t = i2c_send_byte(busnummer,wert,2);									// Byte senden, ACK ermitteln
    if (t != 0)																					// 1 -NACK, 0 -ACK
    	{
			sprintf(ep,"pcf8574_write ERROR:no ack2 from PCF8574_%d",address);
			log_append(3,ep);
      }

	i2c_send_stop(busnummer);															// I2C Stop senden

	return(0);																						// 0: alles OK

fehlerende:
	i2c_send_stop(busnummer);															// I2C Stop senden
	rw = 1;																								// 1 zeigt an, da etwas nicht stimmt
	return(rw);
	}



// *****   all_i2c_chip_pcf8574a_read -8-bit Eingang lesen   *****

signed long int all_i2c_chip_pcf8574a_read(unsigned char busnummer,unsigned char address)
	{
	signed long int rw = 0;
	unsigned char i2c_id;
	char ep[80];
	unsigned char t;


	i2c_id = 112 + address * 2;														// Adresse kann 0...7 sein PCF8574A

	i2c_send_start(busnummer);														// I2C Start senden

	t = i2c_send_byte(busnummer,i2c_id + 1,2);						// Byte senden, ACK ermitteln: 64=0100 ttt1 = Read from ID #t (PCF8574)
  if (t != 0)																						// 1 -NACK, 0 -ACK
	 	{
		sprintf(ep,"pcf8574a_read ERROR:no ack from PCF8574A_%d",address);
		log_append(3,ep);
    goto fehlerende;
    }

	rw = i2c_byte_receive(busnummer);											// Byte holen, ACK senden
	if (rw < 0) goto fehlerende;													// Werte knnen nur zwischen 0...255 liegen

	i2c_send_stop(busnummer);															// I2C Stop senden

	// printf("read8574a(bus%d,id%d)=%d\n",busnummer,address,rw);
	return(rw);																						// >0: alles OK

fehlerende:
	i2c_send_stop(busnummer);															// I2C Stop senden
	rw = -1;																							// -1 zeigt an, da etwas nicht stimmt
	return(rw);
	}




// *****   all_i2c_chip_pcf8574a_write -8-bit Ausgang setzen   *****

unsigned char all_i2c_chip_pcf8574a_write(unsigned char busnummer,unsigned char address,unsigned char wert)
	{
	unsigned char rw = 0;
	unsigned char i2c_id;
	unsigned char t;
	char ep[80];

	i2c_id = 112 + address * 2;														// Adresse kann 0...7 sein PCF8574A


	i2c_send_start(busnummer);														// I2C Start senden

	t = i2c_send_byte(busnummer,i2c_id,2);								// Byte senden, ACK ermitteln: 64=0100 ttt0 = Write to ID #t (PCF8574)
    if (t != 0)																					// 1 -NACK, 0 -ACK
    	{
			sprintf(ep,"pcf8574a_write ERROR:no ack1 from PCF8574A_%d",address);
			log_append(3,ep);
       goto fehlerende;
       }

	t = i2c_send_byte(busnummer,wert,2);									// Byte senden, ACK ermitteln
    if (t != 0)																					// 1 -NACK, 0 -ACK
    	{
			sprintf(ep,"pcf8574a_write ERROR:no ack2 from PCF8574A_%d",address);
			log_append(3,ep);
      }

	i2c_send_stop(busnummer);															// I2C Stop senden

	return(0);																						// 0: alles OK

fehlerende:
	i2c_send_stop(busnummer);															// I2C Stop senden
	rw = 1;																								// 1 zeigt an, da etwas nicht stimmt
	return(rw);
	}



// *****   Berechnen der Prfsumme   *****

unsigned char berechne_pruefsumme(unsigned char startwert,unsigned char prueffeld[],unsigned char bytes)
        {
        unsigned char ps;
        unsigned char t;
        
        ps = startwert;
        
        for(t = 0;t < bytes;t++)
                {
                ps += prueffeld[t] - t;
                }

        return(ps);
        }



// *****   all_i2c_chip_dualrgb_read(Filename) - Daten vom ATMEGA8 zurcklesen   *****

unsigned char all_i2c_chip_dualrgb_read(unsigned char busnummer,unsigned long int bus_delay)
	{
	unsigned char rw = 0;
	unsigned char t;
	// char ep[80];
	unsigned char testarray[50];
	unsigned char testarraypointer;
	unsigned char t_checksumme;
	unsigned char berechnet;


	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(0,0);																// Gruppe auswhlen
	selektiere_port(0,0);																	// Port auswhlen

	// RGB-Dimmer mit ATMEGA8 programmieren
	i2c_send_start(busnummer);														// Start senden 

	// printf ("+RGB0: %d %d %d, RGB1: %d %d %d, Dim 0/1: %d %d\r\n",r0,g0,b0,r1,g1,b1,dim0,dim1);
	t = i2c_send_byte(busnummer,53,2);										// Byte senden, ACK ermitteln: 52 + 1 = Read from RGB-Atmel
	if (t != 0)																						// 1 -NACK, 0 -ACK
   	{
		// sprintf(ep,"dualrgb_read ERROR:no ack1 from Atmel");
		// log_append(3,ep);
		goto fehlerende;
		}

	testarraypointer = 0;
	t_poti0 = i2c_receive_byte(busnummer,1);						// Byte holen, ACK senden
	testarray[testarraypointer++] = t_poti0;
	t_poti1 = i2c_receive_byte(busnummer,1);						// Byte holen, ACK senden
	testarray[testarraypointer++] = t_poti1;
	t_poti2 = i2c_receive_byte(busnummer,1);						// Byte holen, ACK senden
	testarray[testarraypointer++] = t_poti2;
	t_poti3 = i2c_receive_byte(busnummer,1);						// Byte holen, ACK senden
	testarray[testarraypointer++] = t_poti3;
	t_poti4 = i2c_receive_byte(busnummer,1);						// Byte holen, ACK senden
	testarray[testarraypointer++] = t_poti4;
	t_poti5 = i2c_receive_byte(busnummer,1);						// Byte holen, ACK senden
	testarray[testarraypointer++] = t_poti5;
	t_r0 = i2c_receive_byte(busnummer,1);								// r0 Byte holen, ACK senden
	testarray[testarraypointer++] = t_r0;
	t_g0 = i2c_receive_byte(busnummer,1);								// g0 Byte holen, ACK senden
	testarray[testarraypointer++] = t_g0;
	t_b0 = i2c_receive_byte(busnummer,1);								// b0 Byte holen, ACK senden
	testarray[testarraypointer++] = t_b0;
	t_w0 = i2c_receive_byte(busnummer,1);								// w0 Byte holen, ACK senden
	testarray[testarraypointer++] = t_w0;
	t_r1 = i2c_receive_byte(busnummer,1);								// r1 Byte holen, ACK senden
	testarray[testarraypointer++] = t_r1;
	t_g1 = i2c_receive_byte(busnummer,1);								// g1 Byte holen, ACK senden
	testarray[testarraypointer++] = t_g1;
	t_b1 = i2c_receive_byte(busnummer,1);								// b1 Byte holen, ACK senden
	testarray[testarraypointer++] = t_b1;
	t_w1 = i2c_receive_byte(busnummer,1);								// w1 Byte holen, ACK senden
	testarray[testarraypointer++] = t_w1;
	t_ir = i2c_receive_byte(busnummer,1);								// Byte holen, ACK senden
	testarray[testarraypointer++] = t_ir;
	t_ircounter = i2c_receive_byte(busnummer,1);				// Byte holen, ACK senden
	testarray[testarraypointer++] = t_ircounter;
	t_fw_major = i2c_receive_byte(busnummer,1);					// Byte holen, ACK senden
	testarray[testarraypointer++] = t_fw_major;
	t_fw_minor = i2c_receive_byte(busnummer,1);					// Byte holen, ACK senden
	testarray[testarraypointer++] = t_fw_minor;
	t_checksumme = i2c_receive_byte(busnummer,0);				// Byte holen, NACK senden


	// Oben und unten Totbereiche definieren
	if (t_poti0 <   2) t_poti0 =   0;
	if (t_poti1 <   2) t_poti1 =   0;
	if (t_poti2 <   2) t_poti2 =   0;
	if (t_poti3 <   2) t_poti3 =   0;
	if (t_poti4 <   2) t_poti4 =   0;
	if (t_poti5 <   2) t_poti5 =   0;
	if (t_poti0 > 250) t_poti0 = 255;
	if (t_poti1 > 250) t_poti1 = 255;
	if (t_poti2 > 250) t_poti2 = 255;
	if (t_poti3 > 250) t_poti3 = 255;
	if (t_poti4 > 250) t_poti4 = 255;
	if (t_poti5 > 250) t_poti5 = 255;


	berechnet = berechne_pruefsumme(42,testarray,testarraypointer);
	if (berechnet != t_checksumme)
		{
		// printf("Prfsumme:%d, Berechnet:%d  ",t_checksumme,berechnet);
		goto fehlerende;
		}

	i2c_send_stop(busnummer);														// I2C Stop senden
	rw = 0;																							// 0: alles OK
	goto endeende;

fehlerende:
	i2c_send_stop(busnummer);														// I2C Stop senden
	// fprintf(f,"\n<error>1</error>\n");
	rw = 1;																							// 1 zeigt an, da etwas nicht stimmt

endeende:
	return(rw);
	}



// *****   all_i2c_chip_dualrgb_write -8-fach Dimmer setzen   *****

unsigned char all_i2c_chip_dualrgb_write(unsigned char busnummer,unsigned long int bus_delay,unsigned char r0,unsigned char g0,unsigned char b0,unsigned char w0,unsigned char r1,unsigned char g1,unsigned char b1,unsigned char w1,unsigned char dim0,unsigned char dim1)
	{
	unsigned char rw = 0;
	unsigned char t;
	char ep[80];
	unsigned char testarray[50];
	unsigned char testarraypointer;
	unsigned char berechnet;


	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(0,0);																// Gruppe auswhlen
	selektiere_port(0,0);																	// Port auswhlen

	// RGB-Dimmer mit ATMEGA8 programmieren
	i2c_send_start(busnummer);														// Start senden 

	// printf ("+RGBW0: %d %d %d %d, RGBW1: %d %d %d %d, Dim 0/1: %d %d\r\n",r0,g0,b0,w0,r1,g1,b1,w1,dim0,dim1);
	t = i2c_send_byte(busnummer,52,2);										// Byte senden, ACK ermitteln: 52 = Write to RGB-Atmel
  if (t != 0)																						// 1 -NACK, 0 -ACK
   	{
		sprintf(ep,"dualrgb_write ERROR:no ack1 from Atmel");
		log_append(4,ep);
		goto fehlerende;
		}

	testarraypointer = 0;
	testarray[testarraypointer++] = r0;
	t = i2c_send_byte(busnummer,r0,2);										// Byte senden, ACK ermitteln
  if (t != 0)																						// 1 -NACK, 0 -ACK
   	{
		sprintf(ep,"dualrgb_write ERROR:no ack_r0 from Atmel");
		log_append(4,ep);
    goto fehlerende;
    }

	testarray[testarraypointer++] = g0;
	t = i2c_send_byte(busnummer,g0,2);										// Byte senden, ACK ermitteln
  if (t != 0)																						// 1 -NACK, 0 -ACK
   	{
		sprintf(ep,"dualrgb_write ERROR:no ack_g0 from Atmel");
		log_append(4,ep);
    goto fehlerende;
    }

	testarray[testarraypointer++] = b0;
	t = i2c_send_byte(busnummer,b0,2);										// Byte senden, ACK ermitteln
  if (t != 0)																						// 1 -NACK, 0 -ACK
   	{
		sprintf(ep,"dualrgb_write ERROR:no ack_b0 from Atmel");
		log_append(4,ep);
    goto fehlerende;
    }

	testarray[testarraypointer++] = w0;
	t = i2c_send_byte(busnummer,w0,2);										// Byte senden, ACK ermitteln
  if (t != 0)																						// 1 -NACK, 0 -ACK
   	{
		sprintf(ep,"dualrgb_write ERROR:no ack_w0 from Atmel");
		log_append(4,ep);
    goto fehlerende;
    }

	testarray[testarraypointer++] = dim0;
	t = i2c_send_byte(busnummer,dim0,2);									// Byte senden, ACK ermitteln
  if (t != 0)																						// 1 -NACK, 0 -ACK
   	{
		sprintf(ep,"dualrgb_write ERROR:no ack_dim0 from Atmel");
		log_append(4,ep);
    goto fehlerende;
    }


	testarray[testarraypointer++] = r1;
	t = i2c_send_byte(busnummer,r1,2);										// Byte senden, ACK ermitteln
  if (t != 0)																						// 1 -NACK, 0 -ACK
   	{
		sprintf(ep,"dualrgb_write ERROR:no ack_r1 from Atmel");
		log_append(4,ep);
    goto fehlerende;
    }

	testarray[testarraypointer++] = g1;
	t = i2c_send_byte(busnummer,g1,2);										// Byte senden, ACK ermitteln
  if (t != 0)																						// 1 -NACK, 0 -ACK
   	{
		sprintf(ep,"dualrgb_write ERROR:no ack_g1 from Atmel");
		log_append(4,ep);
    goto fehlerende;
    }

	testarray[testarraypointer++] = b1;
	t = i2c_send_byte(busnummer,b1,2);										// Byte senden, ACK ermitteln
  if (t != 0)																						// 1 -NACK, 0 -ACK
  	{
		sprintf(ep,"dualrgb_write ERROR:no ack_b1 from Atmel");
		log_append(4,ep);
    goto fehlerende;
    }

	testarray[testarraypointer++] = w1;
	t = i2c_send_byte(busnummer,w1,2);										// Byte senden, ACK ermitteln
  if (t != 0)																						// 1 -NACK, 0 -ACK
   	{
		sprintf(ep,"dualrgb_write ERROR:no ack_w1 from Atmel");
		log_append(4,ep);
    goto fehlerende;
    }

	testarray[testarraypointer++] = dim1;
	t = i2c_send_byte(busnummer,dim1,2);									// Byte senden, ACK ermitteln
  if (t != 0)																						// 1 -NACK, 0 -ACK
   	{
		sprintf(ep,"dualrgb_write ERROR:no ack_dim1 from Atmel");
		log_append(4,ep);
    goto fehlerende;
    }

	berechnet = berechne_pruefsumme(42,testarray,testarraypointer);
	t = i2c_send_byte(busnummer,berechnet,2);							// Byte senden, ACK ermitteln
  if (t != 0)																						// 1 -NACK, 0 -ACK
   	{
		sprintf(ep,"dualrgb_write ERROR:no ack_checksum from Atmel");
		log_append(4,ep);
    goto fehlerende;
    }

	
	i2c_send_stop(busnummer);															// I2C Stop senden
	return(0);																						// 0: alles OK

fehlerende:
	i2c_send_stop(busnummer);															// I2C Stop senden
	rw = 1;																								// 1 zeigt an, da etwas nicht stimmt
	return(rw);
	}



// ***** all_i2c_chip_ltc4258_writepoe   -   PoE-Ports ein- und ausschalten   *****

unsigned char all_i2c_chip_ltc4258_writepoe(unsigned char busnummer,unsigned char poe_chipid,unsigned char poe_portnummer,unsigned char poe_schaltwunsch)
	{
	unsigned char bitmaske = 0;
	unsigned char t;


	for (t = 0;t < 3;t++)
	{
	i2c_send_start(busnummer);														// Start senden 
	i2c_send_byte(busnummer,64 + poe_chipid * 2,1);				// Byte senden, ACK einfach vom Master senden "010a aaaw"
	i2c_send_byte(busnummer,18,1);												// Register $12 Operating Mode
	i2c_send_byte(busnummer,85,1);												// Datenbyte 0101 0101 = 85 alle 4 Ports Manuell
	i2c_send_stop(busnummer);															// I2C Stop senden

	if ((poe_portnummer == 0) && (poe_schaltwunsch == 1)) bitmaske =   1; // 0000 0001
	if ((poe_portnummer == 1) && (poe_schaltwunsch == 1)) bitmaske =   2; // 0000 0010
	if ((poe_portnummer == 2) && (poe_schaltwunsch == 1)) bitmaske =   4; // 0000 0100
	if ((poe_portnummer == 3) && (poe_schaltwunsch == 1)) bitmaske =   8; // 0000 1000
	if ((poe_portnummer == 0) && (poe_schaltwunsch == 0)) bitmaske =  16; // 0001 0000
	if ((poe_portnummer == 1) && (poe_schaltwunsch == 0)) bitmaske =  32; // 0010 0000
	if ((poe_portnummer == 2) && (poe_schaltwunsch == 0)) bitmaske =  64; // 0100 0000
	if ((poe_portnummer == 3) && (poe_schaltwunsch == 0)) bitmaske = 128; // 1000 0000

	i2c_send_start(busnummer);														// Start senden 
	i2c_send_byte(busnummer,64 + poe_chipid * 2,1);				// Byte senden, ACK einfach vom Master senden "010a aaaw"
	i2c_send_byte(busnummer,25,1);												// Register $19  Power enable Pushbutton
	i2c_send_byte(busnummer,bitmaske,1);									// Datenbyte
	i2c_send_stop(busnummer);															// I2C Stop senden
	}

	return(0);																						// 0: alles OK
	}




// *****   all_i2c_chip_rfid001_read_cardid - Kartennummer auslesen   *****

signed long int all_i2c_chip_rfid001_read_cardid(unsigned char busnummer,char *string_value)
	{
	signed long int rw;
	unsigned char t;
	unsigned char karten;
	unsigned char i;
	char ep[80];
	unsigned char rwarray[9];															// Array fr Rckwerte vom Cardreader


	// Anzahl vorhandener Karten-IDs holen
	//   0      : Keine Karte
	//   1      : mind. 1 Karte
	//   2...254: Anzahl Karten (derzet noch nicht implementiert)
	// 255      : keine Karte oder Fehler
	//
	// Kommando $AA $F0: Hole Anzahl
	//          $B0 $00...$07: Hole Datenbytes
	//          $B0 $08: hole Prfsumme ber die 8 Datenbytes
	//          $AB $00: Lsche aktuelle Karten-ID

	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,80,2);										// Byte senden, ACK ermitteln: $50=01010000 = Write from ID #80 (RFID001)
  if (t != 0)																						// 0 -ACK, 1 -NACK
   	{
		// printf("**NO RFID01** \n");
		sprintf(ep,"rfid_read ERROR:no ack01 from RFID001");
		log_append(3,ep);
    goto fehlerende;
    }
	t = i2c_send_byte(busnummer,170,2);										// $AA: Lesekommando
  if (t != 0)																						// 0 -ACK, 1 -NACK
  	{
		// printf("**NO RFID02** \n");
		sprintf(ep,"rfid_read ERROR:no ack02 from RFID001");
		log_append(3,ep);
    goto fehlerende;
    }
	t = i2c_send_byte(busnummer,240,2);										// $F0: Anzahl 
  if (t != 0)																						// 0 -ACK, 1 -NACK
   	{
		// printf("**NO RFID03** \n");
		sprintf(ep,"rfid_read ERROR:no ack03 from RFID001");
		log_append(3,ep);
  	goto fehlerende;
    }
	i2c_send_stop(busnummer);															// I2C Stop senden


	i2c_send_start(busnummer);														// I2C Start senden (Restart)
	t = i2c_send_byte(busnummer,81,2);										// Byte senden, ACK ermitteln: $50=01010001 = Read from ID #80 (RFID001)
  if (t != 0)																						// 0 -ACK, 1 -NACK
  	{
		// printf("**NO RFID04** \n");
		sprintf(ep,"rfid_read ERROR:no ack04 from RFID001");
		log_append(3,ep);
		goto fehlerende;
		}

	karten = i2c_byte_receive(busnummer);									// Byte holen
	i2c_send_nak(busnummer);															// NAK senden
	i2c_send_stop(busnummer);															// I2C Stop senden

	// printf("**%d Karte(n) im FIFO**\n",karten);
	usleep(100000);

	if(karten == 0)
		{
		// 0: RFID Reader da, aber keine Karten im FIFO
		strcpy(string_value,"0000000000000000");							// Karten-ID ist $0000 0000
		rw = 0;
		goto ende;
		}
	if(karten == 255)
		{
		// $FF: RFID Reader da, aber keine Karten oder Fehler
		strcpy(string_value,"0000000000000000");							// Karten-ID ist $0000 0000
		rw = -1;
		goto ende;
		}

	// 1...254: mind. 1 Karte im FIFO; diese auslesen
	rw = karten;
	for(i = 0;i < 9;i++)
		{
		i2c_send_start(busnummer);													// I2C Start senden
		t = i2c_send_byte(busnummer,80,2);									// Byte senden, ACK ermitteln: $50=01010000 = Write from ID #80 (RFID001)
  	if (t != 0)																					// 0 -ACK, 1 -NACK
   		{
			// printf("**NO RFID11** \n");
			sprintf(ep,"rfid_read ERROR:no ack11 from RFID001");
			log_append(3,ep);
			goto crcfehler;
    	}
		t = i2c_send_byte(busnummer,176,2);									// $B0: Lesekommando
  	if (t != 0)																					// 0 -ACK, 1 -NACK
  		{
			// printf("**NO RFID12** \n");
			sprintf(ep,"rfid_read ERROR:no ack12 from RFID001");
			log_append(3,ep);
			goto crcfehler;
    	}
		t = i2c_send_byte(busnummer,i,2);										// i: Anzahl 
  	if (t != 0)																					// 0 -ACK, 1 -NACK
   		{
			// printf("**NO RFID13** \n");
			sprintf(ep,"rfid_read ERROR:no ack13 from RFID001");
			log_append(3,ep);
			goto crcfehler;
    	}
		i2c_send_stop(busnummer);														// I2C Stop senden
		i2c_send_start(busnummer);													// I2C Start senden (Restart)
		t = i2c_send_byte(busnummer,81,2);									// Byte senden, ACK ermitteln: $50=01010001 = Read from ID #80 (RFID001)
  	if (t != 0)																					// 0 -ACK, 1 -NACK
  		{
			// printf("**NO RFID14** \n");
			sprintf(ep,"rfid_read ERROR:no ack14 from RFID001");
			log_append(3,ep);
			goto crcfehler;
			}

		rwarray[i] = i2c_byte_receive(busnummer);						// Byte holen
		i2c_send_nak(busnummer);														// NAK senden
		i2c_send_stop(busnummer);														// I2C Stop senden
		// printf("--Byte%d:%d\n",i,rwarray[i]);
		usleep(1000);
		}

	// Lschkmmando
	i2c_send_start(busnummer);														// I2C Start senden
	t = i2c_send_byte(busnummer,80,2);										// Byte senden, ACK ermitteln: $50=01010000 = Write from ID #80 (RFID001)
  if (t != 0)																						// 0 -ACK, 1 -NACK
   	{
		// printf("**NO RFID21** \n");
		sprintf(ep,"rfid_read ERROR:no ack21 from RFID001");
		log_append(3,ep);
    goto fehlerende;
    }
	t = i2c_send_byte(busnummer,171,2);										// $AB 00: Lschkommando
  if (t != 0)																						// 0 -ACK, 1 -NACK
  	{
		// printf("**NO RFID22** \n");
		sprintf(ep,"rfid_read ERROR:no ack22 from RFID001");
		log_append(3,ep);
    goto fehlerende;
    }
	t = i2c_send_byte(busnummer,0,2);										// $00: Anzahl 
  if (t != 0)																						// 0 -ACK, 1 -NACK
   	{
		// printf("**NO RFID33** \n");
		sprintf(ep,"rfid_read ERROR:no ack33 from RFID001");
		log_append(3,ep);
  	goto fehlerende;
    }
	i2c_send_stop(busnummer);															// I2C Stop senden
	i2c_send_start(busnummer);														// I2C Start senden (Restart)
	t = i2c_send_byte(busnummer,81,2);										// Byte senden, ACK ermitteln: $50=01010001 = Read from ID #80 (RFID001)
  if (t != 0)																						// 0 -ACK, 1 -NACK
  	{
		/// printf("**NO RFID34** \n");
		sprintf(ep,"rfid_read ERROR:no ack34 from RFID001");
		log_append(3,ep);
		goto fehlerende;
		}

	t = i2c_byte_receive(busnummer);											// Byte holen
	i2c_send_nak(busnummer);															// NAK senden
	i2c_send_stop(busnummer);															// I2C Stop senden
	// printf("*Lsch-RW:%d**\n",t);

	sprintf(string_value,"%02x%02x%02x%02x%02x%02x%02x%02x",rwarray[7],rwarray[6],rwarray[5],rwarray[4],rwarray[3],rwarray[2],rwarray[1],rwarray[0]);
	goto ende;

crcfehler:
	strcpy(string_value,"0000000000000000");
	rw = -2;
	goto ende;
fehlerende:
	strcpy(string_value,"0000000000000000");
	rw = -204800000;
ende:
	return(rw);
	}



// *****   simplified_rfid001_read_cardid - RFID001 Karten-ID direkt auslesen   *****

double simplified_rfid001_read_cardid(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay,char *string_value)
	{
	unsigned char retries = 0; // SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[180];
	char zwischenspeicher[80] = "Nix drinnen !";

beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_rfid001_read_cardid(busnummer,zwischenspeicher);	// Chip abfragen
	if (rw <= -204800000) goto fehlerende;								// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point
	strcpy (string_value,zwischenspeicher);
	// printf("RFID Karte: %s\n",string_value);
ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_rfid001_read(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d, string:\"%s\")",bus_group,bus_port,bus_delay,ff,try_counter,zwischenspeicher);
	log_append(4,ep);
	return(ff);

fehlerende:
	strcpy(string_value,"(not found)");										// String mit Fehlermeldung besetzen
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_all3032_read_brightness - ATTINY85 direkt auslesen   *****

double simplified_all3032_read_brightness(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay)
	{
	unsigned char retries = SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_all3032_read_brightness(busnummer);	// Chip abfragen
	if (rw <= -204800000) goto fehlerende;								// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_all3032_readb(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_all3025_read_counter - PCF8583 direkt auslesen   *****

double simplified_all3025_read_counter(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay)
	{
	unsigned char retries = SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_all3025_read_counter(busnummer);		// Chip abfragen
	if (rw <= -204800000) goto fehlerende;								// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_all3025_readc(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_all4035_read_counter - PCF8583 direkt auslesen   *****

double simplified_all4035_read_counter(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay)
	{
	unsigned char retries = SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_all4035_read_counter(busnummer);		// Chip abfragen
	if (rw <= -204800000) goto fehlerende;								// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_all4035_readc(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_all4023_read_voltage - ATTINY85 direkt auslesen   *****

double simplified_all4023_read_voltage(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay,unsigned char channel)
	{
	unsigned char retries = SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_all4023_read_voltage(busnummer,channel); // Chip abfragen
	if ((channel == 0) && (rw <= -204800000)) goto fehlerende; // wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_all4023_readv(grp=%d, port=%d, speed=%ld, channel=%d) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,channel,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_all3021_read_voltage - MAX1237 direkt auslesen   *****

double simplified_all3021_read_voltage(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay)
	{
	unsigned char retries = SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_all3022_read_voltage(busnummer,0);	// Chip abfragen
	if (rw <= -204800000) goto fehlerende;								// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_all3021_readv(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_all3022_read_voltage - MAX1237 direkt auslesen   *****

double simplified_all3022_read_voltage(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay,unsigned char channel)
	{
	unsigned char retries = SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_all3022_read_voltage(busnummer,channel); // Chip abfragen
	if ((channel == 0) && (rw <= -204800000)) goto fehlerende; // wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_all3022_readv(grp=%d, port=%d, speed=%ld, channel=%d) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,channel,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_all4033_read_voltage - ATTINY85 direkt auslesen   *****

double simplified_all4033_read_voltage(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay)
	{
	unsigned char retries = SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_all4033_read_voltage(busnummer);		// Chip abfragen
	if (rw <= -204800000) goto fehlerende;								// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_all4033_readw(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_lm75_read_temp - LM75 direkt auslesen   *****

double simplified_lm75_read_temp(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay,unsigned char address)
	{
	unsigned char retries = SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_lm75_read_temp(busnummer,address);	// Chip abfragen
	if (rw <= -204800000) goto fehlerende;								// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_lm75_readt(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_lm92_read_temp - LM92 direkt auslesen   *****

double simplified_lm92_read_temp(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay,unsigned char address)
	{
	unsigned char retries = SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_lm92_read_temp(busnummer,address);	// Chip abfragen
	if (rw <= -2048000) goto fehlerende;									// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_lm92_readt(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_ads1100_read - ADS1100 direkt auslesen   *****

double simplified_ads1100_read(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay,unsigned char address,unsigned char bits)
	{
	unsigned char retries = SENSOR_RETRIES;
	double ff = -2048000.0;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_ads1100_read(busnummer,address,bits); // Chip abfragen
	if (rw <= -2048000) goto fehlerende;									// wenn nix zurckkam, nochmal probieren
	if(bits == 14)
		{
		ff = (double)rw / 100 * 4;													// umrechnen in floating-Point (*4: for 13->15 bits skalierung)
		}
	if(bits == 16)
		{
		ff = (double)rw / 100;															// umrechnen in floating-Point
		}

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_ads1100_read(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}






// *****   simplified_sht71_read_temp - SHT71 Temperatur direkt auslesen   *****

double simplified_sht71_read_temp(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay)
	{
	unsigned char retries = 0; 														// SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_sht71_read_temp(busnummer);					// Chip abfragen
	if (rw <= -2048000) goto fehlerende;									// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_sht71_readt(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_sht71_read_humidity - SHT71 Feuchtigkeit direkt auslesen   *****

double simplified_sht71_read_humidity(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay)
	{
	unsigned char retries = 0; 														// SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_sht71_read_humidity(busnummer);			// Chip abfragen
	if (rw <= -2048000) goto fehlerende;									// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_sht71_readh(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}




// *****   simplified_sht71_read_temp_nw - SHT71 Temperatur direkt auslesen (noWait)   *****

double simplified_sht71_read_temp_nw(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay)
	{
	unsigned char retries = 0; 														// SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_sht71_read_temp_nw(busnummer);			// Chip abfragen
	if (rw <= -2048000) goto fehlerende;									// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_sht71_readt_nw(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_sht71_read_humidity_nw - SHT71 Feuchtigkeit direkt auslesen (noWait)   *****

double simplified_sht71_read_humidity_nw(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay)
	{
	unsigned char retries = 0; 														// SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_sht71_read_humidity_nw(busnummer);	// Chip abfragen
	if (rw <= -2048000) goto fehlerende;									// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_sht71_readh_nw(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_dsdx015a2r_read_pressure - DSDX015A2R Luftdruck direkt auslesen   *****

double simplified_dsdx015a2r_read_pressure(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay)
	{
	unsigned char retries = 0; 														// SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_dsdx015a2r_read_pressure(busnummer);// Chip abfragen
	if (rw <= -2048000) goto fehlerende;									// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_dsdx015a2r_read(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_hdi0811_read_pressure - HDI0811ARY8P3 Luftdruck direkt auslesen   *****

double simplified_hdi0811_read_pressure(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay)
	{
	unsigned char retries = 0; 														// SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_hdi0811_read_pressure(busnummer);		// Chip abfragen
	if (rw <= -2048000) goto fehlerende;									// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_hdi0811_read(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_bmp085_read_pressure - BMP085 Luftdruck direkt auslesen   *****

double simplified_bmp085_read_pressure(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay)
	{
	unsigned char retries = 0; 														// SENSOR_RETRIES;
	double ff;
	signed long int rw;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_bmp085_read_pressure(busnummer);		// Chip abfragen
	if (rw <= -2048000) goto fehlerende;									// wenn nix zurckkam, nochmal probieren
	ff = (double)rw / 100;																// umrechnen in floating-Point

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_bmp085_read(grp=%d, port=%d, speed=%ld) (RW=%5.2f, tries=%d)",bus_group,bus_port,bus_delay,ff,try_counter);
	log_append(4,ep);
	return(ff);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	ff = -2048000;
	goto ende;
	}



// *****   simplified_pcf8574_read - Lesen eines 8-bit I/Os   *****

signed long int simplified_pcf8574_read(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay,unsigned char address)
	{
	signed long int rw = -1;
	unsigned char retries = SENSOR_RETRIES;
	unsigned char try_counter = 0;
	char ep[80];
	unsigned char wert = 0;


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_pcf8574_read(busnummer,address);		// Chip lesen
	//printf("pcf8574_read(bus%d,address%d): %d\n",busnummer,address,rw);
	if (rw < 0) goto fehlerende;													// wenn nix zurckkam, nochmal probieren
	wert = rw;

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_pcf8574_read(grp=%d, port=%d, speed=%ld addr=%d value=%d) (RW=%ld, tries=%d)",bus_group,bus_port,bus_delay,address,wert,rw,try_counter);
	log_append(4,ep);
	return(rw);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	rw = -2048000;
	goto ende;
	}




// *****   simplified_pcf8574_write - Schreiben eines 8-bit I/Os   *****


unsigned char simplified_pcf8574_write(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay,unsigned char address,unsigned char wert)
	{
	unsigned char rw = 0;
	unsigned char retries = SENSOR_RETRIES;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_pcf8574_write(busnummer,address,wert); // Chip beschreiben
	if (rw != 0) goto fehlerende;													// wenn nix zurckkam, nochmal probieren

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_pcf8574_write(grp=%d, port=%d, speed=%ld addr=%d value=%d) (RW=%d, tries=%d)",bus_group,bus_port,bus_delay,address,wert,rw,try_counter);
	log_append(4,ep);
	return(rw);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	rw = 1;
	goto ende;
	}



// *****   simplified_pcf8574a_read - Lesen eines 8-bit I/Os   *****

signed long int simplified_pcf8574a_read(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay,unsigned char address)
	{
	signed long int rw = -1;
	unsigned char retries = SENSOR_RETRIES;
	unsigned char try_counter = 0;
	char ep[80];
	unsigned char wert = 0;


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_pcf8574a_read(busnummer,address);		// Chip lesen
	if (rw < 0) goto fehlerende;													// wenn nix zurckkam, nochmal probieren
	wert = rw;

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_pcf8574a_read(grp=%d, port=%d, speed=%ld addr=%d value=%d) (RW=%ld, tries=%d)",bus_group,bus_port,bus_delay,address,wert,rw,try_counter);
	log_append(4,ep);
	return(rw);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	rw = 1;
	goto ende;
	}




// *****   simplified_pcf8574a_write - Schreiben eines 8-bit I/Os   *****


unsigned char simplified_pcf8574a_write(unsigned char busnummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay,unsigned char address,unsigned char wert)
	{
	unsigned char rw = 0;
	unsigned char retries = SENSOR_RETRIES;
	unsigned char try_counter = 0;
	char ep[80];


beginn:
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(busnummer,bus_group);								// Gruppe auswhlen
	selektiere_port(busnummer,bus_port);									// Port auswhlen

	rw = all_i2c_chip_pcf8574a_write(busnummer,address,wert); // Chip beschreiben
	if (rw != 0) goto fehlerende;													// wenn nix zurckkam, nochmal probieren

ende:
	i2c_send_stop(busnummer);															// Datentransfer sauber beenden
	sprintf(ep,"s_pcf8574a_write(grp=%d, port=%d, speed=%ld addr=%d value=%d) (RW=%d, tries=%d)",bus_group,bus_port,bus_delay,address,wert,rw,try_counter);
	log_append(4,ep);
	return(rw);

fehlerende:
	try_counter++;
	if (try_counter < retries)
		{
		i2c_send_stop(busnummer);														// Datentransfer sauber beenden
		usleep(50000);																			// 50 milli-Sekunden warten
		goto beginn;																				// nochmal probieren
		}

	rw = 1;
	goto ende;
	}



// **********  simplified_all5000_led_set() - LEDs ein- und ausschalten auf ALL4500/ALL5000 mit PCF8574A auf ID0

void simplified_all5000_led_set(unsigned short int led_nummer,unsigned char schaltwunsch)
	{
	unsigned char bitmaske = 0;
	unsigned char led_sollstatus = 0;
	const unsigned char zp[8] = {1,2,4,8,16,32,64,128};
	unsigned char i;
	unsigned char busspeed = 1; // 40


	// printf("led_set(%d,%d)\n",led_nummer,schaltwunsch);

  led_sollstatus = 0;
	if (schaltwunsch != 0) led_sollstatus = 1;
	
	bitmaske = simplified_pcf8574a_read(0,0,0,busspeed,0);

	i = zp[led_nummer];
	if (led_sollstatus == 0)
		{
		// ausschalten (auf "1" setzen)
		bitmaske |= i;
		}
	else
		{
		// einschalten (auf "0" setzen)
		bitmaske &= (255 - i);
		}

	simplified_pcf8574a_write(0,0,0,busspeed,0,bitmaske);
	}
