// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   iopin_read.c   -   Abfragen eines GPIO-Pins von der Kommandozeile aus                                         **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2010 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************


#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "at91gpio.h"
#include "all_i2c.h"
#include "i2c_chips.h"



// **********   Hauptprogramm   **********

int main(int argc,char **argv)
	{
	signed short int rw = 0;
	signed short int port_nummer = -1;
	signed short int bit_nummer = -1;
	char parameter1[5];
	char parameter2[3];


 	if (argc != 2)
		{
		printf("Usage: iopin_read [Port][Bit]\n");
		printf("e.g.: iopin_read PB23 or iopin_read PA0\n");
		rw = -2;
		goto ende;
		}

	if (strlen(argv[1]) > 4)
		{
		printf("iopin_read: argument \"%s\" is longer than 4 characters\n",argv[1]);
		rw = -3;
		goto ende;
		}

	if (strlen(argv[1]) <3)
		{
		printf("iopin_read: argument \"%s\" is shorter than 3 characters\n",argv[1]);
		rw = -3;
		goto ende;
		}

	strcpy(parameter1,argv[1]);

	if (parameter1[0] != 'P')
		{
		printf("iopin_read: argument \"%s\" does not start with 'P'\n",argv[1]);
		rw = -4;
		goto ende;
		}

	if ((parameter1[1] < 'A') || (parameter1[1] > 'E'))
		{
		printf("iopin_read: argument \"%s\": 'Px..' is not in the Range of A...E\n",argv[1]);
		rw = -5;
		goto ende;
		}
	if (parameter1[1] == 'A') port_nummer = PORT_A;
	if (parameter1[1] == 'B') port_nummer = PORT_B;
	if (parameter1[1] == 'C') port_nummer = PORT_C;
	if (parameter1[1] == 'D') port_nummer = PORT_D;
	if (parameter1[1] == 'E') port_nummer = PORT_E;

	if(strlen(parameter1) == 3)
		{
		parameter2[0] = parameter1[2];
		parameter2[1] = 0;
		if ((parameter2[0] < '0') || (parameter2[0] > '9'))
			{
			printf("iopin_read: argument \"%s\": 'Px..' bit_number is not in the Range of 0...9\n",argv[1]);
			rw = -6;
			goto ende;
			}
		}	

	if(strlen(parameter1) == 4)
		{
		parameter2[0] = parameter1[2];
		parameter2[1] = parameter1[3];
		parameter2[2] = 0;
		if ((parameter2[0] < '0') || (parameter2[0] > '9'))
			{
			printf("iopin_read: argument \"%s\": 'Px..' bit_number is not in the Range of 0x...9x\n",argv[1]);
			rw = -7;
			goto ende;
			}
		if ((parameter2[1] < '0') || (parameter2[1] > '9'))
			{
			printf("iopin_read: argument \"%s\": 'Px..' bit_number is not in the Range of x0...x9\n",argv[1]);
			rw = -8;
			goto ende;
			}
		}	

 	bit_nummer = atoi(parameter2);

	if (bit_nummer > 31)
		{
		printf("iopin_read: argument \"%s\" bit_number %d larger than allowed 0...31\n",argv[1],bit_nummer);
		rw = -9;
		goto ende;
		}


	gpio_devmem_open();																		// GPIOs oeffnen
	// printf("P%c%d read: ",port_nummer + 'A',bit_nummer);
	rw = gpio_port_read(port_nummer,bit_nummer);					// Bitwert holen
	if(rw > 0) rw = 1;																		// und normalisieren
	printf("%d\n",rw);																		// und ausgeben
ende:
	return(rw);																						// als Funktionswert zurckgeben
	}

