// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   knopf_demon.c   -   Demon zum Abfragen der Taster einer PL LED21 Lampe                                        **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   I/O Registerbelegung   **********
//
// PE21		Taster "+" (Pullup nach 3.3V, Kontakt schaltet nach Masse)
// PE20		Taster "-" (Pullup nach 3.3V, Kontakt schaltet nach Masse)
// PE19		Taster "S" (Pullup nach 3.3V, Kontakt schaltet nach Masse)
// PE16		Taster "C" (Pullup nach 3.3V, Kontakt schaltet nach Masse)
// auf dem alten Dieterboard gehen PE17 und PE18 nicht (sind Dauer-H)


#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <fcntl.h>
#include "at91gpio.h"
#include "all_i2c.h"
#include "i2c_chips.h"
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "sqldb.h"
#include "logging.h"



// *****   udp_send() - Daten senden   *****

void udp_send(char zielip[],unsigned short int portnummer,unsigned char *daten,unsigned short int laenge)
	{
	int							udp_socket;
	struct					sockaddr_in steuerstruct;

	// printf("udp_send an %s:%d, %d Bytes.\n",zielip,portnummer,laenge);

	if ((udp_socket = socket(PF_INET,SOCK_DGRAM,IPPROTO_UDP)) >= 0) // UDP-Socket anlegen
		{
		// Construct the server sockaddr_in structure
		memset(&steuerstruct,0,sizeof(steuerstruct));				// Struct auf 0 setzen
		steuerstruct.sin_family = AF_INET;									// Internet/IP
		steuerstruct.sin_addr.s_addr = inet_addr(zielip);		// IP-Addresse
		steuerstruct.sin_port = htons(portnummer);					// UDP-Portnummer

		sendto(udp_socket,daten,laenge,0,(struct sockaddr *) &steuerstruct,sizeof(steuerstruct));
		close(udp_socket);
		// printf("OK.\n");
		}
	}



// **********   Globale Variablen   **********

signed long int ti = 0;																	// System-Timer
unsigned char uhrzeit_sekunde;													// Uhr Sekunde
unsigned char uhrzeit_minute;														// Uhr Minute
unsigned char uhrzeit_stunde;														// Uhr Stunde
unsigned char last_sekunde = 80;												// letzte Sekunde
unsigned short int udp_rx_port = 54321;									// TCP Socket fr Client/Server Verbindung
char udp_rx_username[1024] = "";												// Username fr UDP-Verbindung
char udp_rx_password[1024] = "";												// Passwort fr UDP-Verbindung



// *****   demon_readconfig   -   Konfigurationsdaten lesen   *****

void demon_readconfig(void)
	{
	udp_rx_port = sqldb_read_long("/sys/network/udp_rx_port");
	sqldb_read_string("/sys/network/udp_rx_username",udp_rx_username);
	sqldb_read_string("/sys/network/udp_rx_password",udp_rx_password);
	}



// **********   Demon   **********

int main(int argc,char **argv)
	{
	char ep[80];
	unsigned char lastpe;																	// zuletzt geschlossener Kontakt von PE
	unsigned char k[52];																	// Puffer fr UDP-Paket
	unsigned char taste;																	// Merker fr gedrckte Taste
	unsigned char j;
	unsigned char c;
	unsigned char l;


	gpio_devmem_open();																		// GPIOs oeffnen
	gpio_port_init_output(PORT_E,21);											// PE21
	gpio_port_init_output(PORT_E,20);											// PE20
	gpio_port_init_output(PORT_E,19);											// PE19
	gpio_port_init_output(PORT_E,16);											// PE18

	gpio_port_write(PORT_E,21,0);													// L
	gpio_port_write(PORT_E,20,0);													// L
	gpio_port_write(PORT_E,19,0);													// L
	gpio_port_write(PORT_E,16,0);													// L

	gpio_port_write(PORT_E,21,1);													// H
	gpio_port_write(PORT_E,20,1);													// H
	gpio_port_write(PORT_E,19,1);													// H
	gpio_port_write(PORT_E,16,1);													// H

	gpio_port_init_input(PORT_E,21);											// auf Eingang setzen
	gpio_port_init_input(PORT_E,20);											// auf Eingang setzen
	gpio_port_init_input(PORT_E,19);											// auf Eingang setzen
	gpio_port_init_input(PORT_E,16);											// auf Eingang setzen


	strcpy (log_filename,"/www/reports/knopf_demon.log");	// Log-Filename setzen
	log_level = 3;																				// Debuglevel setzen
	sprintf(ep,"----------   knopf_demon started (debuglevel=%d)   ----------",log_level);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben

	demon_readconfig();																		// Einstellungen fuer den Kontakt zum RGB-Demon lesen

	printf("knopf_demon started.\n");


hauptschleife:
	// printf("Hauptschleife\n");
	lastpe = 0;
hauptschleife2:
	ti++;																									// Sys-Clock hochzhlen
	sleep(0);																							// ein bichen Zeit vertrdeln
	if(gpio_port_read(PORT_E,21) == 0)
		{
		// printf("PE21:L\n");
		if (lastpe == 21) goto lauern2;
		goto hauptschleife; // Solange L ist, erstmal abwarten
		}
	if(gpio_port_read(PORT_E,20) == 0)
		{
		// printf("PE20:L\n");
		if (lastpe == 20) goto lauern2;
		goto hauptschleife; // Solange L ist, erstmal abwarten
		}
	if(gpio_port_read(PORT_E,19) == 0)
		{
		// printf("PE19:L\n");
		if (lastpe == 19) goto lauern2;
		goto hauptschleife; // Solange L ist, erstmal abwarten
		}
	if(gpio_port_read(PORT_E,16) == 0)
		{
		// printf("PE16:L\n");
		if (lastpe == 16) goto lauern2;
		goto hauptschleife; // Solange L ist, erstmal abwarten
		}
	usleep(75000);																				// danach 75 ms warten, um evtl. Kontaktprellen nicht mitzuzhlen

lauern:
	sleep(0);																							// Kontrolle ans System bergeben
lauern2:
	usleep(100000);																				// danach 100 ms warten, um evtl. Kontaktprellen nicht mitzuzhlen
	if(gpio_port_read(PORT_E,21) == 0) goto found_plus;		// darauf lauern, da der Port auf L gezogen wird
	if(gpio_port_read(PORT_E,20) == 0) goto found_minus;	// darauf lauern, da der Port auf L gezogen wird
	if(gpio_port_read(PORT_E,19) == 0) goto found_s;			// darauf lauern, da der Port auf L gezogen wird
	if(gpio_port_read(PORT_E,16) == 0) goto found_c;			// darauf lauern, da der Port auf L gezogen wird
	// printf("Quatsch erkannt !\n");
	goto lauern;


found_plus:
	lastpe = 21;
	taste = '+';
	// printf("Plus !\n");
	goto send;

found_minus:
	lastpe = 20;
	taste = '-';
	// printf("Minus !\n");
	goto send;

found_s:
	lastpe = 19;
	taste = 'S';
	// printf("\"S\" (innerer Ring) !\n");
	goto send;

found_c:
	lastpe = 16;
	taste = 'C';
	// printf("\"C\2 (aeusserer Ring) !\n");
	goto send;


send:
	memset(k,0,51);
	l = 0;
	for(j = 0;j < 20;j++)
		{
		c = udp_rx_username[j];
		if (c == 0) goto skip1;
		k[l++] = c;
		}
skip1:
	l = 20;
	for(j = 0;j < 20;j++)
		{
		c = udp_rx_password[j];
		if (c == 0) goto skip2;
		k[l++] = c;
		}
skip2:
	k[40] = 1;
	k[41] = 2;
	k[42] = 3;
	k[43] = 4;
	k[44] = 5;
	k[45] = 6;
	k[46] = 7;
	k[47] = 8;
	k[48] = 'K';
	k[49] = taste;
	k[50] = 0;

	udp_send("127.0.0.1",udp_rx_port,k,50);
	goto hauptschleife2;


// ende:
	sprintf(ep,"----------   knopf_demon terminated   ----------");
	log_append(1,ep);																			// Endmeldung ins Logfile schreiben
	return(0);
	}

