// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   logging.c   -   Logfile-Funktionen                                                                            **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// ----------   log-levels   ----------
// 0 - unconditional output
// 1 - Spezielle Ausgaben, wichtige Meldungen
// 2 - Fehlermeldungen
// 3 - Warnungen
// 4 - Pipifax-Meldungen und Hinweise jeglicher Art
//
//-1 - Bedingungslose Ausgabe nach /dev/console



#include <stdio.h>
#include <time.h>
#include "logging.h"



// *****   Globale Variablen   *****

/*
char							file_log[] = "generic.log";						// Logfilename
char							path_data[] = "./";										// in diesem Verzeichnis liegen die Austauschdaten
*/
char							log_filename[255];										// kompletter Filename mit Pfad
unsigned char			log_level = 4;												// 0:quiet, 1-special BS, 2-Errors, 3-Warnings/Infos, 4-Everything




/*
// *****   log_init - Logfile initialisieren   *****

void log_init(void)
	{
	strcpy(log_filename,path_data);						// Pfad zu den Datendateien
	strcat(log_filename,file_log);						// Name des Logfiles
	}
*/



// *****   log_append - Daten in Logfile schreiben   *****

void log_append(signed char level,char meldung[])
	{
	FILE *f;
	struct tm *ptr;
	time_t lt;

	if (level == -1)
		{
		// -1: Ausgabe auf Konsole
		f = fopen("/dev/console","wb");											// Konsole ffnen
		if (f != NULL)
			{
			fprintf(f,"%s\n",meldung);
			fflush(f);
			fclose(f);
			}
		}
	else
		{
		// Ausgabe in Logfile
		if (level <= log_level)															// nur protokollieren, wenn Level hoch genug (= ber Schwelle)
			{
			f = fopen(log_filename,"ab");											// File fr append ffnen
			if (f != NULL)
				{
				lt = time(NULL);
				ptr = localtime(&lt);
				// fprintf(f,"%02d:%02d:%02d %02d.%02d.%04d %s\n",ptr->tm_hour,ptr->tm_min,ptr->tm_sec,ptr->tm_mday,ptr->tm_mon,ptr->tm_year + 1900,meldung);
				fprintf(f,"%04d-%02d-%02d,%02d:%02d:%02d[%d]%s\n",ptr->tm_year + 1900,ptr->tm_mon + 1,ptr->tm_mday,ptr->tm_hour,ptr->tm_min,ptr->tm_sec,level,meldung);
				fflush(f);
				fclose(f);
				}
			}
		}
	}



// *****   log_kill - Logfile-Lnge auf 0 Bytes krzen   *****

void log_kill(void)
	{
	FILE *f;

	f = fopen(log_filename,"wb");					// File fr berschreiben ffnen
	if (f != NULL)
		{
		fflush(f);
		fclose(f);
		}
	}
