// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   plugwise_demon.c   -   Demon zum Steuern von Plugwise-Steckdosen                                              **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   Eintrge in der Konfigurations-Datenbank, die vom Demon ausgewertet werden   **********
//
// /control/plugwise/max_retries												Anzahl Versuche, einen Slave zu erreichen
// /control/plugwise/max_slaves"												Anzahl zu verwaltender Slaves
// /control/plugwise/reload_counter"										Zhler, um festzustellen ob ein Neuladen der Config ntig ist
// /control/actors/actorXX/value"												gibt den zu schaltenden Wert; wird mit gepolltem Wert upgedated



#include <errno.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <termios.h>
#include <time.h>
#include <sys/time.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <math.h>
#include <float.h>
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "sqldb.h"
#include "sensor_demon.h"
#include "shm_use.h"
#include "plugwise_demon.h"
#include "logging.h"




// *****   Globale Variablen   *****

unsigned short int		cache_max_slaves;
signed char						cache_slave_enabled[PLUGWISE_MAX_SLAVES];
char									cache_slave_mac_address[PLUGWISE_MAX_SLAVES][17];
signed long int				cache_slave_mapfrom_actor[PLUGWISE_MAX_SLAVES];
signed long int				cache_slave_mapto_sensor[PLUGWISE_MAX_SLAVES];
signed long int				cache_slave_value[PLUGWISE_MAX_SLAVES];
double								cache_slave_watt[PLUGWISE_MAX_SLAVES];
signed long int				plugwise_reload_counter = -1;			// Merker fr Reload-Counter
char									plugwise_comport[20];							// Schnittstelle des Master-Moduls
signed long int				read_delay;												// Verzgerung zwischen selbstttigen Reads
signed long int				plugwise_max_retries = 5;
int										shID;
struct s_shm_actordata*	shm_actor_r_data;								// Aktordaten R im Shared-RAM
struct s_shm_actordata*	shm_actor_w_data;								// Aktordaten W im Shared-RAM
signed long int				shadow_w_data[MAX_LOGICAL_ACTORS];// Kopie der Aktordaten-Integers
unsigned long int			last_read_timestamp = 0;					// Zeit des letzten Reads
signed long int				zyklischer_sklave = 0;						// Nummer fr Zyklische Abfrage



// *****   rs232_open("/dev/...")   -   seriellen Port ffnen   *****
// the baudrate parameter is ignored, it is always set to 4800

int rs232_open(char device[],unsigned long int baudrate)
	{
	int fd;
	struct termios options;


	memset((void *)&options,0,sizeof(options));

	// fd = open("/dev/ttyS4",O_RDWR | O_NOCTTY |O_SYNC);
	fd = open(device,O_RDWR | O_NOCTTY |O_SYNC);
	if( fd < 0) goto ende;


	tcdrain(fd);

	// Comport-Parameter einstellen
  options.c_cflag = B4800 | CS8 | CLOCAL | CREAD;
  options.c_cflag &= ~(PARENB | CSTOPB);
  options.c_iflag |= (IGNPAR | ECHOCTL | ECHOKE);
  options.c_lflag &= ~(ICANON | OPOST| ECHO | ECHOE | ISIG | IEXTEN | ECHOK | ONLCR |IXON);
  options.c_cc[VMIN]  = 0;
  options.c_cc[VTIME]  = 0;
  tcsetattr(fd,TCSAFLUSH,&options);

ende:
	return(fd);
	}



// *****   get_ms_now()   -   aktuelle Zeit in ms ermitteln   *****

long get_ms_now(void)
	{
	long rw;
	const long msdivider = CLOCKS_PER_SEC / 1000;					// Teiler um von Clockticks aus ms zu kommen
  struct timeval tv;
	time_t curtime;
	struct tm *myTm;


	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln 
  curtime = tv.tv_sec;																	// Lores-Zeit in s reusziehen
	myTm = localtime(&curtime);														// ...und berfhren in tm-Struktur
	rw = (myTm->tm_hour * 3600 + myTm->tm_min * 60 + myTm->tm_sec) * 1000 + tv.tv_usec / msdivider;
	return(rw);
	}



// *****   plugwise_schalte2()   -   Schalten von Slaves mit Wiederholung im Fehlerfall   *****

signed long int plugwise_schalte2(unsigned short int dosennummer,unsigned short int schaltwunsch,unsigned short int tries)
	{
	signed long int rw = -1;
	signed short int t;
	char ep[333];

	for(t = 0;t < tries;t++)
		{
		rw = plugwise_schalte(dosennummer,schaltwunsch);
		if(rw >= 0) goto ende;
		sprintf(ep,"plugwise_schalte2: Retry for plugwise slave %d:%d (RW:%ld)",dosennummer,schaltwunsch,rw);
		log_append(4,ep);																		// Meldung ins Logfile schreiben
		usleep(150000);																			// 150 ms warten
		}

ende:
	return(rw);
	}



// *****   plugwise_query2()   -   Abfragen von Slaves mit Wiederholung im Fehlerfall   *****

signed long int plugwise_query2(unsigned short int dosennummer,unsigned short int tries)
	{
	signed long int rw = -1;
	signed short int t;
	char ep[333];

	for(t = 0;t < tries;t++)
		{
		rw = plugwise_query(dosennummer);
		if(rw >= 0) goto ende;
		sprintf(ep,"plugwise_query2: Retry for plugwise slave %d (RW:%ld)",dosennummer,rw);
		log_append(4,ep);																		// Meldung ins Logfile schreiben
		usleep(150000);																			// 150 ms warten
		}

ende:
	return(rw);
	}



// *****   plugwise_query_power2()   -   Abfragen von Slaves mit Wiederholung im Fehlerfall   *****

double plugwise_query_power2(unsigned short int dosennummer,unsigned short int tries)
	{
	double f = -1;
	signed short int t;
	char ep[333];

	for(t = 0;t < tries;t++)
		{
		f = plugwise_query_power(dosennummer);
		if(f >= 0) goto ende;
		sprintf(ep,"plugwise_query_power2: Retry for plugwise slave %d (RW:%4.2f)",dosennummer,f);
		log_append(4,ep);																		// Meldung ins Logfile schreiben
		usleep(150000);																			// 150 ms warten
		}

ende:
	// printf("RW2=%4.2f\n",f);
	return(f);
	}



// *****   plugwise_schalte() - Schalten von Slaves   *****

signed long int plugwise_schalte(unsigned short int dosennummer,unsigned short int schaltwunsch)
	{
	signed long int rw = -2;
	char ep[300];

	rw = plugwise_ll_send(plugwise_comport,cache_slave_mac_address[dosennummer],schaltwunsch);

	sprintf(ep,"plugwise_schalte(%d:%d): %s, Port:%s  RW=%ld",dosennummer,schaltwunsch,cache_slave_mac_address[dosennummer],plugwise_comport,rw);
	log_append(4,ep);																			// Meldung ins Logfile schreiben

	return(rw);
	}



// *****   plugwise_query() - Ermitteln des Schaltzustands von Slaves   *****

signed long int plugwise_query(unsigned short int dosennummer)
	{
	signed long int rw = -2;
	char ep[300];

	rw = plugwise_ll_send(plugwise_comport,cache_slave_mac_address[dosennummer],2);

	sprintf(ep,"plugwise_query(%d): %s, Port:%s  RW=%ld",dosennummer,cache_slave_mac_address[dosennummer],plugwise_comport,rw);
	log_append(4,ep);																			// Meldung ins Logfile schreiben

	return(rw);
	}



// *****   plugwise_query_power() - Ermitteln der Leistungsaufnahme von Slaves   *****

double plugwise_query_power(unsigned short int dosennummer)
	{
	signed long int rw = -2;
	double f;
	char ep[300];

	rw = plugwise_ll_send(plugwise_comport,cache_slave_mac_address[dosennummer],3);

	if (rw < 0)
		{
		f = rw;
		}
	else
		{
		f = rw / 100;
		}

	sprintf(ep,"plugwise_query_power(%d): %s, Port:%s  RW=%ld (%4.2fW)",dosennummer,cache_slave_mac_address[dosennummer],plugwise_comport,rw,f);
	log_append(4,ep);																			// Meldung ins Logfile schreiben

	return(f);
	}



// *****   cache_setzedefault()   -   Mappingtabelle lschen   *****

void cache_setzedefault(void)
	{
	unsigned short int t;

	for(t = 0;t < PLUGWISE_MAX_SLAVES;t++)
		{
		cache_slave_enabled[t] = 0;
		strcpy(cache_slave_mac_address[t],"0000000000000000");
		cache_slave_mapfrom_actor[t] = -1;
		cache_slave_mapto_sensor[t] = -1;
		cache_slave_value[t] = -204800000;
		cache_slave_watt[t] = -2048000.0;
		shm_actor_r_data[t].value_long = -204800000;
		shm_actor_r_data[t].value_float = -2048000.0;
		strcpy(shm_actor_r_data[t].value_string,"(unknown)");
		}
	}


// **********   Demon-Configfile einlesen   **********

void pwdemon_readconfig(void)
	{
	char ep[1024];
	unsigned short int t;
	unsigned short int sklaven = 0;

	plugwise_reload_counter = sqldb_read_long("/control/plugwise/server/reload_counter");

	log_level = sqldb_read_long("/control/plugwise/server/loglevel");
	cache_max_slaves = sqldb_read_long("/control/plugwise/server/max_slaves");
	read_delay = sqldb_read_long("/control/plugwise/server/readdelay");
	if (read_delay < 1) read_delay = 1;
	sqldb_read_string("/control/plugwise/server/comport",plugwise_comport);

	for(t = 0;t < cache_max_slaves;t++)
		{
		sprintf(ep,"/control/plugwise/slave%d/enabled",t);
		cache_slave_enabled[t] = sqldb_read_long(ep);
		if (cache_slave_enabled[t] > 0) sklaven++;
		sprintf(ep,"/control/plugwise/slave%d/mapfrom_actor",t);
		cache_slave_mapfrom_actor[t] = sqldb_read_long(ep);
		sprintf(ep,"/control/plugwise/slave%d/mapto_sensor",t);
		cache_slave_mapto_sensor[t] = sqldb_read_long(ep);
		sprintf(ep,"/control/plugwise/slave%d/mac_address",t);
		sqldb_read_string(ep,cache_slave_mac_address[t]);
		}

	sprintf(ep,"Global Plugwise Configuration read. Loglevel:%d, Readdelay:%ld, Slaves:%d",log_level,read_delay,sklaven);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	}




// *****   mid$() Funktion von Basic   *****

void midstring(char langer_string[],unsigned short int anfang, unsigned short int laenge,char *ergebnis)
	{
	unsigned short int t;
	unsigned short int i = 0;

	for(t = 0;t < laenge;t++)
		{
		ergebnis[i++] = langer_string[anfang + t];
		}
	ergebnis[i] = 0;
	} 


// ***** find_data() - Ermitteln des Datensrings aus dem Nll, den Plugwise zurckliefert   *****
// Nimmt die letzten end_length Zeichen von input_string[] und sucht darin nach
// dem Vorkommen des search_pattern[]. Ab dieser Stelle wird der String in result_string[]
// zurckgeliefert.
// Sollte der input_string zu kurz sein, oder das Pattern nicht gefunden werden, ist der
// result_string[] leer und es wird -1 zurckgeliefert. Ansonsten 1.

signed char find_data(char input_string[],unsigned short int end_length,char search_pattern[], char *result_string)
	{
	signed char rw = -2;
	char tmp_string[255];
	char *x;

	if(strlen(input_string) < end_length)
		{
		result_string[0] = 0;
		rw = -1;
		goto ende;
		}

	midstring(input_string,strlen(input_string) - end_length,end_length,tmp_string);
	x = strstr(tmp_string,search_pattern);
	if(x != NULL)
		{
		strcpy(result_string,x);
		rw = 1;
		goto ende;
		}
	else
		{
		result_string[0] = 0;
		rw = -1;
		goto ende;
		}

ende: return(rw);
	}



// *****   reflect()   *****

unsigned long reflect (unsigned long crc, int bitnum)
	{
	// reflects the lower 'bitnum' bits of 'crc'
	unsigned long i;
	unsigned long j = 1;
	unsigned long crcout = 0;

	for (i = (unsigned long)1 << (bitnum - 1); i; i >>= 1)
		{
		if (crc & i) crcout |= j;
		j <<= 1;
		}
	return(crcout);
	}



// ***** generate_crc_table()   *****

void generate_crc_table(void)
	{
	// make CRC lookup table used by table algorithms
	int i;
	int j;
	unsigned long bit;
	unsigned long crc;

	for (i = 0;i < 256;i++)
		{
		crc = (unsigned long)i;
		if (refin) crc = reflect(crc,8);
		crc <<= order - 8;

		for (j = 0;j < 8;j++)
			{
			bit = crc & crchighbit;
			crc <<= 1;
			if (bit) crc ^= polynom;
			}			

		if (refin) crc = reflect(crc,order);
		crc &= crcmask;
		crctab[i] = crc;
		}
	}



// *****   crcbitbybit()   *****

unsigned long crcbitbybit(char* p, unsigned long len)
	{
	//precalculate_crc();
	int k;
	int l;
	int z;
	unsigned long bit;
	unsigned long crc;
	unsigned long i;
	unsigned long j;
	unsigned long c;

	// at first, compute constant bit masks for whole CRC and CRC high bit
	crcmask = ((((unsigned long)1 << (order - 1)) -1) << 1) | 1;
	crchighbit = (unsigned long)1 << (order - 1);

	//generate_crc_table();
	for (k = 0;k < 256;k++)
		{
		crc = (unsigned long)k;
		if (refin) crc = reflect(crc,8);
		crc <<= order - 8;

		for (l = 0;l < 8;l++)
			{
			bit = crc & crchighbit;
			crc <<= 1;
			if (bit) crc ^= polynom;
			}

		if (refin) crc = reflect(crc,order);
		crc &= crcmask;
		//crctab[k]= crc;
		}

	// compute missing initial CRC value
	if (!direct)
		{
		crcinit_nondirect = crcinit;
		crc = crcinit;
		for (z = 0;z < order;z++)
			{
			bit = crc & crchighbit;
			crc <<= 1;
			if (bit) crc ^= polynom;
			}
		crc &= crcmask;
		crcinit_direct = crc;
		}
	else
		{
		crcinit_direct = crcinit;
		crc = crcinit;
		for (z = 0;z < order;z++)
			{
			bit = crc & 1;
			if (bit) crc ^= polynom;
			crc >>= 1;
			if (bit) crc |= crchighbit;
			}
		crcinit_nondirect = crc;
		}

	// bit by bit algorithm with augmented zero bytes.
	// does not use lookup table, suited for polynom orders between 1...32.
	for (i = 0;i < len;i++)
		{
		c = (unsigned long)*p++;
		if (refin) c = reflect(c,8);

		for (j = 0x80;j;j >>= 1)
			{
			bit = crc & crchighbit;
			crc <<= 1;
			if (c & j) crc |= 1;
			if (bit) crc ^= polynom;
			}
		}

	for (i = 0;i < order;i++)
		{
		bit = crc & crchighbit;
		crc <<= 1;
		if (bit) crc ^= polynom;
		}

	if (refout) crc = reflect(crc,order);
	crc ^= crcxor;
	crc &= crcmask;

	return(crc);
	}






// *****   plgwsd_initport()   *****

int plgwsd_initport(int fd)
	{
	struct termios options;
	// Get the current options for the port...
	tcgetattr(fd, &options);
	// Set the baud rates to 19200...
	cfsetispeed(&options, B115200);
	cfsetospeed(&options, B115200);
	// Enable the receiver and set local mode...
	options.c_cflag |= (CLOCAL | CREAD);
	// Set data bits
	options.c_cflag &= ~CSIZE;														// Mask the character size bits
	options.c_cflag |= CS8;																// Select 8 data bits

	// Set RAW input
	options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);

	// Set Raw output
	options.c_oflag &= ~OPOST;

	// Set timeout to 1 sec
	options.c_cc[VMIN] = 0;
	options.c_cc[VTIME] = 10;
	// Set the new options for the port...
	tcsetattr(fd, TCSANOW, &options);
	return(1);
	}



// *****   readport()   *****

int readport(int fd,char *result)
	{
	int iIn = read(fd, result, 255);

	result[iIn-1] = 0x00;
	if (iIn < 0)
		{
		if (errno == EAGAIN)
			{
			printf("SERIAL EAGAIN ERROR\n");
			//continue;
			return(1);
			}
		else
			{
			printf("SERIAL read error %d %s\n", errno, strerror(errno));
			return(0);
			}
		}                    
	return(1);
	}



// *****   read_serialport()   *****

int read_serialport (int fd, char *buf, int bytes, int timeout)
	{
	fcntl(fd, F_SETFL, 0);
  char *dest = buf;
  int remaining = bytes;
	int got;
	int n;
  struct timeval tv;

  fd_set fdr;

	// reinit timeval structure each time through loop
	tv.tv_sec = 0;
	tv.tv_usec = 0;

	// wait for data to come in using select
	FD_ZERO(&fdr);
	FD_SET(fd, &fdr);
	n = select(fd,&fdr,NULL,NULL,&tv);
	printf("%i\n",n);
	if (n < 0)
		perror("select failed");
	else if (n == 0)
		puts("TIMEOUT");
	else
		{
		// We have input
		if (FD_ISSET(fd,&fdr))
		got = read(fd,dest,remaining);
		}

	// return number of bytes read, might be 0 if 0 requested
	return(bytes);
	}



// *****   HexToFloat()   *****

float HexToFloat(char *hex)
	{
	union
		{
		int i;
		float f;
		} switcheroo;
	char buf[9];
	memset(buf,0,9);
	strncpy(buf,hex,8);

	sscanf(buf,"%x",&switcheroo.i);
	//printf("0x%X %f\n", switcheroo.i, switcheroo.f);
	return(switcheroo.f);
	}



// *****   htoi()   *****

unsigned int htoi(char s[])
	{
	unsigned int val = 0;
	int x = 0;

	if(s[x] == '0' && (s[x+1] == 'x' || s[x+1] == 'X')) x += 2;

	while(s[x] != '\0')
		{
		if(val > UINT_MAX) return(0);
		else if(s[x] >= '0' && s[x] <='9')
			{
			val = val * 16 + s[x] - '0';
			}
		else if(s[x]>='A' && s[x] <='F')
			{
			val = val * 16 + s[x] - 'A' + 10;
			}
		else if(s[x]>='a' && s[x] <='f')
			{
			val = val * 16 + s[x] - 'a' + 10;
			}
		else return(0);
		x++;
		}
	//printf("%i\n",val);
	return val;
	}



// *****   plgwsd_send_powercode   *****

void plgwsd_send_powercode(int fd,char circlemac[],char action[])
	{
	char header[4] = { 0x05, 0x05, 0x03, 0x03} ;
	char powercode[5] ="0017";
	char endline[3] = { 0x0D, 0x0A,'\0'} ;
	unsigned long  crc;
	char zeroes[5] = "0000";
	char string[32];
	char crcsum[10];
	char buf[41];


	// printf("powercode(fd=%d, mac=\"%s\",cmd=\"%s\")\n",fd,circlemac,action);
	memset(string,0,32);
	strncpy(string,powercode,4);
	strncat(string,circlemac,16);
	strncat(string,action,2);

	crc = crcbitbybit(string,strlen(string));
	//printf("%i\n",crc);
	sprintf(crcsum,"%lX",crc);
	//printf("%s,%i\n",crcsum,strlen(crcsum));
	if (strlen(crcsum) == 3)
		{
		strncat(string,zeroes,1);
		strncat(string,crcsum,3);
		}
	else
		{
		strncat(string,crcsum,4);
		}

	memset(buf,0,40);
	strncpy(buf,header,4);
	strncat(buf,string,26);
	strncat(buf,endline,2);
	usleep(500);
	write(fd, buf, 40 ) ;
	usleep(500);
	// printf("The string sent to the Plugwise device was: :%s \n", buf);
	}



// *****   plgwsd_get_status - On/Off ermitteln*****

signed long int plgwsd_get_status(int fd,char circlemac[])
	{
	signed long int rw = -1;
	// FILE *f;
	char header[5] = { 0x05, 0x05, 0x03, 0x03,'\0'};
	char zeroes[5] ="0000";
	char statusinforequest[5] ="0023";
	// char statusinforesponse[5] ="0024";
	char endline[3] = { 0x0D, 0x0A,'\0'} ;
	unsigned long  crcstat;
	char strstat[27];
	char crcsumstat[10];
	// char *x;
	// char endstring[200];
  char substr[80];
  char substr2[10];
  char substr3[10];
  char substr4[10];
  char substr5[10];
  char substr6[10];
  char substr7[10];
  char substr8[10];
  char substr9[10];
  char substr10[10];
  char substr11[10];
	char bufstat[41];
	char sResult[255];


	memset(strstat,'\0',27);
	strncpy(strstat,statusinforequest,4);
	strncat(strstat,circlemac,16);
	crcstat=crcbitbybit(strstat, strlen(strstat));
	sprintf( crcsumstat, "%lX", crcstat);
	if (strlen(crcsumstat)==3){
	strncat(strstat,zeroes,1);
	strncat(strstat,crcsumstat,3);
	}else{
	strncat(strstat,crcsumstat,4);
	}
  // printf("%s\n",strstat);

	memset(bufstat,'\0',41);
	strncpy(bufstat,header,4);
	strncat(bufstat,strstat,24);
	strncat(bufstat,endline,2);
//	printf("%s\n",bufstat);	
	
	usleep(500);
	write(fd, bufstat, 40 );
 //       printf("%s\n",bufstat);
	usleep(500000);

	fcntl(fd, F_SETFL, FNDELAY); // don't block serial read
	//read_serialport(fd,sResult,255,0); 
		
	if (!readport(fd,sResult)) {
		printf("read failed\n");
		close(fd);
		return 1;
	}
  // printf("%s\n", sResult);
	// printf("len: %d\n",strlen(sResult));

	if(strlen(sResult) < 100)
		{
		rw = -1;
		goto ende;
		}



	if(find_data(sResult,80,"0024",substr) < 0)
		{
		rw = -1;
		goto ende;
		}

	midstring(substr, 24, 2, substr2);
	midstring(substr, 26, 2, substr3);
	midstring(substr, 28, 4, substr4);
	midstring(substr, 32, 8, substr5);
	midstring(substr, 40, 2, substr6);
	midstring(substr, 42, 2, substr7);
	midstring(substr, 44, 4, substr8);
	midstring(substr, 48, 4, substr9);
	midstring(substr, 52, 4, substr10);
	midstring(substr, 56, 8,substr11);

	// printf("%s\n", substr);
	// printf("%s\n", substr1);
	// printf("Jahr:		%d\n", htoi(substr2)+2000);
	// printf("Monat:		%d\n", htoi(substr3));
	// printf("Minuten:	%d\n", htoi(substr4));
	// printf("Logadresse:	%d\n", (htoi(substr5)-278528)/32);
	if(strcmp(substr6,"01")==0)
		{
		// printf("Schaltstatus:	on\n");
		rw = 1;
		}
	if(strcmp(substr6,"00")==0)
		{
		// printf("Schaltstatus:	off\n");
		rw = 0;
		}

	/*
	if(strcmp(substr7,"85")==0)
		{
		printf("Stromfrequenz:	50 Hz\n");
		}
	if(strcmp(substr7,"C5")==0)
		{
		printf("Stromfrequenz:	60 Hz\n");
		}
	printf("HW Version:	%s-%s-%s\n", substr8,substr9,substr10);
	*/

	// printf("FW Date:	%s\n", asctime(gmtime(substr11)));

ende:	return(rw);
	}



// *****   plgwsd_get_calibration - Leitungsaufnahme in W ermitteln   *****

float plgwsd_get_calibration(int fd, char circlemac[])
	{
	float rw = -1;
	char header[4] = { 0x05, 0x05, 0x03, 0x03} ;
	char zeroes[5] ="0000";
	char calibinforequest[5] ="0026";
	// char calibinforesponse[5] ="0027";
	char powerinforequest[5] ="0012";
	// char powerinforesponse[5] ="0013";
	char endline[3] = { 0x0D, 0x0A,'\0'} ;
	unsigned long  crc,crc2;
	float GainA,GainB,OffTot,OffRuis,value,corrected,power;
	int pulse1,pulse8;
	char string[27];
	char crcsum[10];
	char buf[41];
	char sResult[255];
 	char substr[80];
 	char substr1[10];
 	char substr2[10];
 	char substr3[10];
 	char substr4[10];
	char string2[27];
	char crcsum2[10];
	char buf2[41];
  char pwrinfo_substr[80];
  char pwrinfo_substr1[10];
  char pwrinfo_substr2[10];


	memset(string,0,27);
	strncpy(string,calibinforequest,4);
	strncat(string,circlemac,16);
	crc=crcbitbybit(string,strlen(string));
	sprintf(crcsum,"%lX",crc);
	if (strlen(crcsum)==3)
		{
		strncat(string,zeroes,1);
		strncat(string,crcsum,3);
		}
	else
		{
		strncat(string,crcsum,4);
		}

	memset(buf,0,40);
	strncpy(buf,header,4);
	strncat(buf,string,26);
	strncat(buf,endline,2);
	usleep(500);
	write(fd, buf, 40 );
	// printf("%s\n",string);
	usleep(500000);

	fcntl(fd, F_SETFL, FNDELAY); // don't block serial read

		
	if (!readport(fd,sResult)) {
		printf("read failed\n");
		close(fd);
		return 1;
	}
	
	// printf("The string %d received from Plugwise was:\n %s\n",  strlen(sResult),sResult);
	if(strlen(sResult) < 100)
		{
		rw = -1;
		goto ende;
		}

	// midstring(sResult, begin, end,substr);
	if(find_data(sResult,80,"0027",substr) < 0)
		{
		// printf("** No 0027 found\n");
		rw = -1;
		goto ende;
		}
	// printf("substr=\"%s\"\n",substr);

	midstring(substr, 24, 8, substr1);
	midstring(substr, 32, 8, substr2); 
	midstring(substr, 40, 8, substr3);
	midstring(substr, 48, 8, substr4);

	GainA=HexToFloat(substr1);
	GainB=HexToFloat(substr2);
	OffTot=HexToFloat(substr3);
	OffRuis=HexToFloat(substr4);

	// sending the request	 
	memset(string2,0,27);
	strncpy(string2,powerinforequest,4);
	strncat(string2,circlemac,16);
	crc2=crcbitbybit(string2,strlen(string2));
	sprintf(crcsum2,"%lX",crc2);
	strncat(string2,crcsum2,4);

	memset(buf2,0,40);
	strncpy(buf2,header,4);
	strncat(buf2,string2,26);
	strncat(buf2,endline,2);
	
	usleep(500);
	write(fd, buf2, 40 ) ;
	// printf("%s\n",string2);
	usleep(500000);
	char sResult2[255];
	fcntl(fd,F_SETFL,FNDELAY); // don't block serial read

	if (!readport(fd,sResult2))
		{
		printf("read failed\n");
		close(fd);
		return 1;
		}

	if(find_data(sResult2,80,"0013",pwrinfo_substr) < 0)
		{
		// printf("** No 0013 found\n");
		rw = -1;
		goto ende;
		}
	// printf("pwrinfo_substr=\"%s\"\n",pwrinfo_substr);

	midstring(pwrinfo_substr, 24, 4,pwrinfo_substr1);
	midstring(pwrinfo_substr, 28, 4,pwrinfo_substr2);

	pulse1=htoi(pwrinfo_substr1);
	pulse8=htoi(pwrinfo_substr2);

  value = pulse8 / 8.0;
  corrected = 1.0 * (((pow(value + OffRuis, 2.0) * GainB) + ((value + OffRuis) * GainA)) + OffTot);
  power=((corrected / 1.0) / 468.9385193)*1000;
	// printf("%f Watt\n",power);
	rw = power;
	if(rw < 0.125) rw = 0; // Grundrauschen unterdrcken
ende: return(rw);
	}




// ***** plugwise_ll_send   -   Lowlevel-Sendefunktion fr die Befehle   *****

signed long int plugwise_ll_send(char port[],char circlemac[],signed short int befehl)
	{
	signed long int rw = -9;
	int fd;
	char kommando[10];
	

	fd = open(port, O_RDWR | O_NONBLOCK);
	if (fd == -1)
		{
		printf("plugwise_ll_send: open_port: Unable to open serial port \"%s\"\n",port);
		rw = -2;
		goto ende;
		}
	else
		{
		usleep(500);
		fcntl(fd, F_SETFL,0);
		}
 	plgwsd_initport(fd);


	if((befehl == 0) || (befehl == 1))
		{
		sprintf(kommando,"%02d",befehl);
		plgwsd_send_powercode(fd,circlemac,kommando);
		rw = befehl;
		goto fertig;
		}

	if(befehl == 2)
		{
		rw = plgwsd_get_status(fd,circlemac);
		goto fertig;
		}

	if(befehl == 3)
		{
		rw = 100 * plgwsd_get_calibration(fd,circlemac);
		goto fertig;
		}


	// unbekannter Befehl
	rw = -99;
fertig:
	close(fd);
ende: return(rw);
	}



// *****   Hauptschleife   *****

int main(int argc,char **argv)
	{
	// unsigned char		retries;
	unsigned long					ti = 0;													// Zhler, geht bei jedem Durchlauf um 1 hoch
	char									lstring[255];										// Puffer fr Logging
	char									ep[1024];												// Puffer fr Logfile-Meldungen
	signed long int				t;
	signed long int				i;
	double								f = -1;


	// auf existierendes Shared Memory fr Demonstatus zugreifen
	if (use_my_demonstatus_shm(6) != 1) goto errorende;		// MY_DEMON_ID = 6: plugwise_demon

	// Status von Demon in SHM eintragen
	my_demonstatus_data[my_demon_id].demon_id = my_demon_id;														// 0 ist Default, hier wird die eigene ID eingetragen
	sprintf(my_demonstatus_data[my_demon_id].name_string,"plugwise_demon");							// Name
	sprintf(my_demonstatus_data[my_demon_id].version_string,"%s",PLUGWISE_DEMON_VERSION);// Versionsnummer
	my_demonstatus_data[my_demon_id].last_boot = time(0);																// Timestamp(UTC) des Starts
	my_demonstatus_data[my_demon_id].last_activity = time(0);														// Timestamp(UTC) der letzten Aktion
	my_demonstatus_data[my_demon_id].ti = ti;																						// Counter
	sprintf(my_demonstatus_data[my_demon_id].debug_message,"Start");										// letzte Debug-Meldung
	my_demonstatus_data[my_demon_id].error_message[0] = 0;															// letzte Fehlermeldung
	my_demonstatus_data[my_demon_id].status_long = 0;																		// numerischer Status
	sprintf(my_demonstatus_data[my_demon_id].status_string,"startup");									// String-Status


	// Logging-System initialisieren
	strcpy (log_filename,"/www/reports/plugwise_demon.log"); // Logfilenamen setzen
	log_level = 3;																				// Debuglevel setzen
	// log_kill();																						// Logfile loeschen
	sprintf(ep,"----------   plugwise_demon V %s started (debuglevel=%d)   ----------",PLUGWISE_DEMON_VERSION,log_level);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben



	// auf existierendes Shared Memory fr Aktordaten Read zugreifen
	shID = shmget(SHM_ID_ACTOR_R_DATA,SHM_ACTOR_R_DATA_SIZE,0666);
	if (shID >= 0)
		{
		shm_actor_r_data = shmat(shID,0,0);
		if (shm_actor_r_data == (struct s_shm_actordata *)-1)
			{
			printf("Problem beim Zugriff auf ACTOR_R_DATA shared memory !\n");
			goto ende;
			}
		else
			{
			// printf("SHM_Actor_r_data OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_Actor_r_data war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		goto ende;
    }

	// auf existierendes Shared Memory fr Aktordaten Write zugreifen
	shID = shmget(SHM_ID_ACTOR_W_DATA,SHM_ACTOR_W_DATA_SIZE,0666);
	if (shID >= 0)
		{
		shm_actor_w_data = shmat(shID,0,0);
		if (shm_actor_w_data == (struct s_shm_actordata *)-1)
			{
			printf("Problem beim Zugriff auf ACTOR_W_DATA shared memory !\n");
			goto ende;
			}
		else
			{
			// printf("SHM_Actor_w_data OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_Actor_w_data war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		goto ende;
    }


	// Shadow-Write auf ngltige Werte setzen
	for (t = 0;t < MAX_LOGICAL_ACTORS;t++)
		{
		shadow_w_data[t] = -99;
		}


	// Einstellungen fr lokalen Plugwise-UDP-Demon
	cache_setzedefault();
	pwdemon_readconfig();


	// Start loggen
	sprintf(lstring,"Comport:\"%s\"",plugwise_comport);
	log_append(1,lstring);


	last_read_timestamp = get_ms_now();										// now() merken
	update_demondebug("running");													// letzte Debug-Meldung
hauptschleife:
	update_demonstatus(ti,1,"main loop");									// Status von Demon in SHM aktualisieren
	ti++;																									// Schleifenzhler erhhen
	usleep(25000);																				// 25 ms Rechenzeit freigeben

	i = sqldb_read_long("/control/plugwise/server/reload_counter");
	// printf("reloadmerker: %ld, DB: %ld\n",plugwise_reload_counter,i);
	if (plugwise_reload_counter != i)
		{
		update_demonstatus(ti,2,"reload");									// Status von Demon in SHM aktualisieren
		pwdemon_readconfig();
		sprintf(ep,"Global Plugwise Configuration re-read.");
		log_append(1,ep);																			// Meldung ins Logfile schreiben
		goto hauptschleife;
		}



	// -----   Aktor-Werte auf Vernderungen prfen   -----
	for(t = 0;t < PLUGWISE_MAX_SLAVES;t++)
		{
		if (cache_slave_enabled[t] > 0)
			{
			if (cache_slave_mapfrom_actor[t] >= 0)
				{
				if ((shm_actor_w_data[cache_slave_mapfrom_actor[t]].value_long != shadow_w_data[t])
				||  (shm_actor_w_data[cache_slave_mapfrom_actor[t]].value_long != shm_actor_r_data[cache_slave_mapfrom_actor[t]].value_long))
					{
					// Slave aktiv, korrekt gemappt und Aktor-W-Wert hat sich gendert: wir mssen Schalten !
					update_demonstatus(ti,10,"switch");						// Status von Demon in SHM aktualisieren
					sprintf(ep,"W %ld:%ld",t,shm_actor_w_data[cache_slave_mapfrom_actor[t]].value_long);
					update_demondebug(ep);
					// printf("Aktor %ld will slave %ld auf %ld setzen. (%s)\n",cache_slave_mapfrom_actor[t],t,shm_actor_w_data[cache_slave_mapfrom_actor[t]].value_long,cache_slave_lon_id[t]);
					i = plugwise_schalte2(t,shm_actor_w_data[cache_slave_mapfrom_actor[t]].value_long,plugwise_max_retries);
					if (i >= 0)
						{
						shadow_w_data[t] = i;
						shm_actor_w_data[cache_slave_mapfrom_actor[t]].value_long = shadow_w_data[t];
						shm_actor_r_data[cache_slave_mapfrom_actor[t]].value_long = shadow_w_data[t];
						}
					else
						{
						shm_actor_r_data[cache_slave_mapfrom_actor[t]].value_long = -204800000;
						}
					}
				}
			}
		}



	// -----   Zyklisches Lesen der Aktoren   -----
	for(t = 0;t < PLUGWISE_MAX_SLAVES;t++)
		{
		if(cache_slave_enabled[t] > 0) goto beginn_zyklus;
		}
	update_demonstatus(ti,20,"no slaves");								// Status von Demon in SHM aktualisieren
	update_demondebug("no slaves");												// Status von Demon in SHM aktualisieren
	sleep(3);																							// dann knnen wir auch lnger schlafen
	goto ende_zyklus;
beginn_zyklus:
	update_demonstatus(ti,30,"verify");										// Status von Demon in SHM aktualisieren
	if (last_read_timestamp > get_ms_now()) last_read_timestamp = 0; // berlauf um Mitternacht kompensieren
	if (get_ms_now() > last_read_timestamp + read_delay * 1000)
		{
		// printf("**PW_READ TI=%ld: %ld** delay=%ld\n",ti,last_read_timestamp,read_delay);
suche_zyklus:
		last_read_timestamp = get_ms_now();
		zyklischer_sklave++;
		if(zyklischer_sklave >= PLUGWISE_MAX_SLAVES) zyklischer_sklave = 0;
		if (cache_slave_enabled[zyklischer_sklave] <= 0) goto suche_zyklus;
		i = plugwise_query2(zyklischer_sklave,plugwise_max_retries);
		if (i >= 0)
			{
			shm_actor_r_data[cache_slave_mapfrom_actor[zyklischer_sklave]].value_long = i;
			if (i > 0)
				{
				f = plugwise_query_power2(zyklischer_sklave,plugwise_max_retries);
				if (f > 0)
					{
					shm_actor_r_data[cache_slave_mapfrom_actor[zyklischer_sklave]].value_float = f;
					sprintf(ep,"R %ld:%ld, %1.1fW",zyklischer_sklave,i,f);
					update_demondebug(ep);
					}
				else
					{
					shm_actor_r_data[cache_slave_mapfrom_actor[zyklischer_sklave]].value_float = 0.0;
					sprintf(ep,"R %ld:%ld, -W",zyklischer_sklave,i);
					update_demondebug(ep);
					}
				}
			else
				{
				shm_actor_r_data[cache_slave_mapfrom_actor[zyklischer_sklave]].value_float = 0.0;
				sprintf(ep,"R %ld:%ld",zyklischer_sklave,i);
				update_demondebug(ep);
				}
			}
		else
			{
			shm_actor_r_data[cache_slave_mapfrom_actor[zyklischer_sklave]].value_long = -204800000;
			shm_actor_r_data[cache_slave_mapfrom_actor[zyklischer_sklave]].value_float = -2048000.00;
			sprintf(ep,"R %ld:-",zyklischer_sklave);
			update_demondebug(ep);
			}

		}
ende_zyklus: ;

	goto hauptschleife;

ende:
	return(0);

errorende:
	return(-1);
	}
