// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   powermeter_demon.c   -   Demon zum Polling angeschlossener Stromsenensoren fr den ALL3690                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************


// ----------   Anschlsse am ARM-CPU-Board (fr ALL3690)   ----------
//
// Port	Verwendung
// PA20	I2C Data Input/Output
// PA21	I2C Clock Output
// PD15 Firmware-Reset Input (H:normal, L: totale Lobotomie beim Booten)
// PB27 S0Bus Eingang mit Pullup nach H
// PB20 Analog-Multiplexer Adresse 0
// PB21 Analog-Multiplexer Adresse 1
// PB22 Analog-Multiplexer Adresse 2
//
// ADC 0 MUX Port	Verwendung
// 0              L1
// 1							L2
// 2							L3
// 3							L1 Offset
// 4							L2 Offset
// 5							L3 Offset
// 6							+5V USB
// 7							+12V Spannugnsversorgung vom Netzteil
//
// ADC 1 MUX Port	Verwendung
// 0							L4
// 1							L5
// 2							L6
// 3							L4 Offset
// 4							L5 Offset
// 5							L6 Offset
// 6							frei
// 7							frei


#include <errno.h> /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <unistd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "at91gpio.h"
#include "all_i2c.h"
#include "i2c_chips.h"
#include "sqldb.h"
#include "powermeter_demon.h"
#include "logging.h"



// *****   globale Variablen   *****

#define HISTORY_MAX 3600
signed long int history_watt[10][HISTORY_MAX];
double history_ampere[10][HISTORY_MAX];

#define CONFIG_DATABASE_FILE "/etc/allnetenv/config.s3db"
#define VAR_DATABASE_FILE "/tmp/var.s3db"
#define LOG_DATABASE_FILE "/data/powermeter.s3db"



// *****   sql_lese_d0zaehler   -   Auslesen der D0-Zhlerstnde aus der tmp-DB   *****

void sql_lese_d0zaehler(sqlite3 *ptr)
	{
	char sql[80];
	char bla[80];

	sprintf(sql,"SELECT [dkwh0] as [value] from [measurements] WHERE [id]='0';");
	sqldb_read_string3(ptr,sql,bla);
	d0_kwh[0] = atof(bla);

	sprintf(sql,"SELECT [dkwh1] as [value] from [measurements] WHERE [id]='0';");
	sqldb_read_string3(ptr,sql,bla);
	d0_kwh[1] = atof(bla);

	sprintf(sql,"SELECT [a8] as [value] from [measurements] WHERE [id]='0';");
	sqldb_read_string3(ptr,sql,bla);
	d0_a[0] = atof(bla);

	sprintf(sql,"SELECT [a9] as [value] from [measurements] WHERE [id]='0';");
	sqldb_read_string3(ptr,sql,bla);
	d0_a[1] = atof(bla);

	sprintf(sql,"SELECT [w8] as [value] from [measurements] WHERE [id]='0';");
	sqldb_read_string3(ptr,sql,bla);
	d0_w[0] = atof(bla);

	sprintf(sql,"SELECT [w9] as [value] from [measurements] WHERE [id]='0';");
	sqldb_read_string3(ptr,sql,bla);
	d0_w[1] = atof(bla);

	// printf("D0_KWH0: %1.4f, D0_KWH1: %1.4f\n",d0_kwh[0],d0_kwh[1]);
	}



// *****   sql_lese_s0ticks   -   Auslesen der S0-Zhlerstnde aus der tmp-DB   *****

void sql_lese_s0ticks(sqlite3 *ptr)
	{
	char string[80];


	sqldb_read_string3(ptr,"SELECT [value] from [s0] where [tag]='S0_0';",string);
	s0ticks[0] = atol(string);

	sqldb_read_string3(ptr,"SELECT [value] from [s0] where [tag]='S0_1';",string);
	s0ticks[1] = atol(string);
	}



// *****   sql_schreibe_s0ticks   -   Schreiben der S0-Zhlerstnde in die tmp-DB   *****

void sql_schreibe_s0ticks(sqlite3 *ptr)
	{
	char sql[255];


	// sqldb_write_long2(ptr,"S0_0",s0ticks[0]);
	sprintf(sql,"UPDATE [s0] set [value]='%ld' where [tag]='S0_0';",s0ticks[0]);
	do_sqlyy(ptr,sql,0);

	// sqldb_write_long2(ptr,"S0_1",s0ticks[1]);
	sprintf(sql,"UPDATE [s0] set [value]='%ld' where [tag]='S0_1';",s0ticks[1]);
	do_sqlyy(ptr,sql,0);

	sprintf(sql,"UPDATE [measurements] set [stick0]='%ld' where [id]='0';",s0ticks[0]);
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"UPDATE [measurements] set [stick1]='%ld' where [id]='0';",s0ticks[1]);
	do_sqlyy(ptr,sql,0);
	}



// *****   sql_lese_s0ticks_permanent   -   Auslesen der permanent gespeicherten S0-Zhlerstnde   *****

void sql_lese_s0ticks_permanent(void)
	{
	s0ticks[0] = sqldb_read_long("/powermeter/s0/counter");
	s0ticks[1] = sqldb_read_long("/powermeter/s1/counter");
	}



// *****   sql_schreibe_s0ticks_permanent   -   Permanentes Speichern der S0-Zhlerstnde   *****

void sql_schreibe_s0ticks_permanent(void)
	{
	sqldb_write_long("/powermeter/s0/counter",s0ticks[0]);
	sqldb_write_long("/powermeter/s1/counter",s0ticks[1]);
	}



// *****   sql_lese_spannungen   -   Auslesen der Spannungen aus der tmp-DB   *****

void sql_lese_spannungen(sqlite3 *ptr)
	{
	char string[80];


	sqldb_read_string3(ptr,"SELECT [value] from [voltages] where [tag]='voltage_0';",string);
	powermeter_voltage[0] = atof(string);
	sqldb_read_string3(ptr,"SELECT [value] from [voltages] where [tag]='voltage_1';",string);
	powermeter_voltage[1] = atof(string);
	sqldb_read_string3(ptr,"SELECT [value] from [voltages] where [tag]='voltage_2';",string);
	powermeter_voltage[2] = atof(string);
	sqldb_read_string3(ptr,"SELECT [value] from [voltages] where [tag]='voltage_3';",string);
	powermeter_voltage[3] = atof(string);
	sqldb_read_string3(ptr,"SELECT [value] from [voltages] where [tag]='voltage_4';",string);
	powermeter_voltage[4] = atof(string);
	sqldb_read_string3(ptr,"SELECT [value] from [voltages] where [tag]='voltage_5';",string);
	powermeter_voltage[5] = atof(string);
	sqldb_read_string3(ptr,"SELECT [value] from [voltages] where [tag]='voltage_6';",string);
	powermeter_voltage[6] = atof(string);
	sqldb_read_string3(ptr,"SELECT [value] from [voltages] where [tag]='voltage_7';",string);
	powermeter_voltage[7] = atof(string);
	}



// *****   sql_schreibe_spannungen   -   Schreiben der Spannungen in die tmp-DB   *****

void sql_schreibe_spannungen(sqlite3 *ptr)
	{
	char sql[255];


	sprintf(sql,"UPDATE [voltages] set [value]='%1.2f' where [tag]='voltage_0';",powermeter_voltage[0]);
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"UPDATE [voltages] set [value]='%1.2f' where [tag]='voltage_1';",powermeter_voltage[1]);
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"UPDATE [voltages] set [value]='%1.2f' where [tag]='voltage_2';",powermeter_voltage[2]);
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"UPDATE [voltages] set [value]='%1.2f' where [tag]='voltage_3';",powermeter_voltage[3]);
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"UPDATE [voltages] set [value]='%1.2f' where [tag]='voltage_4';",powermeter_voltage[4]);
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"UPDATE [voltages] set [value]='%1.2f' where [tag]='voltage_5';",powermeter_voltage[5]);
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"UPDATE [voltages] set [value]='%1.2f' where [tag]='voltage_6';",powermeter_voltage[6]);
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"UPDATE [voltages] set [value]='%1.2f' where [tag]='voltage_7';",powermeter_voltage[7]);
	do_sqlyy(ptr,sql,0);
	}



// *****   sql_lese_spannungen_permanent   -   Auslesen der permanent gespeicherten Spannungen   *****

void sql_lese_spannungen_permanent(void)
	{
	powermeter_voltage[0] = sqldb_read_long("/powermeter/voltage0");
	powermeter_voltage[1] = sqldb_read_long("/powermeter/voltage1");
	powermeter_voltage[2] = sqldb_read_long("/powermeter/voltage2");
	powermeter_voltage[3] = sqldb_read_long("/powermeter/voltage3");
	powermeter_voltage[4] = sqldb_read_long("/powermeter/voltage4");
	powermeter_voltage[5] = sqldb_read_long("/powermeter/voltage5");
	powermeter_voltage[6] = sqldb_read_long("/powermeter/voltage6");
	powermeter_voltage[7] = sqldb_read_long("/powermeter/voltage7");
	}



// *****   sql_erase_sensorvalues   -   Lschen und Neuanlegen der Sensorwert-Tabelle   *****

void sql_erase_sensorvalues(sqlite3 *ptr)
	{
	char sql[512];

	// unlink(VAR_DATABASE_FILE);														// evtl. vorhandene alte DB lschen

	// Table Measurements
	do_sqlyy(ptr,"DROP TABLE [measurements];",0);
	do_sqlyy(ptr,"CREATE TABLE [measurements] ([id] INTEGER  PRIMARY KEY AUTOINCREMENT NOT NULL,[timestamp] TIMESTAMP DEFAULT CURRENT_TIMESTAMP NULL,\
		[a0],[a1],[a2],[a3],[a4],[a5],[a8],[a9],[w0],[w1],[w2],[w3],[w4],[w5],[w6],[w7],[w8],[w9],[stick0],[stick1],\
		[dkwh0],[dkwh1],[d0watt0],[d0watt1],[d0watt2],[d1watt0],[d1watt1],[d1watt2],\
		[reload_pm],[reload_scounter],[reload_dcounter],[sleep_demon],[TI],\
		[adc0],[adc1],[adc2],[adc3],[adc4],[adc5]);",0);
	sprintf(sql,"INSERT INTO measurements (id,a0,a1,a2,a3,a4,a5,a8,a9,w0,w1,w2,w3,w4,w5,w6,w7,w8,w9,stick0,stick1,dkwh0,dkwh1,d0watt0,d0watt1,d0watt2,d1watt0,d1watt1,d1watt2,reload_pm,reload_scounter,reload_dcounter,sleep_demon,TI,adc0,adc1,adc2,adc3,adc4,adc5)\
		VALUES(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);");
	do_sqlyy(ptr,sql,0);

	// Table S0
	do_sqlyy(ptr,"DROP TABLE [s0];",0);
	do_sqlyy(ptr,"CREATE TABLE [s0] ([id] INTEGER  PRIMARY KEY AUTOINCREMENT NOT NULL,[tag],[value]);",0);
	sprintf(sql,"INSERT INTO s0 ([id],[tag],[value]) VALUES('0','S0_0','0');");
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"INSERT INTO s0 ([id],[tag],[value]) VALUES('1','S0_1','0');");
	do_sqlyy(ptr,sql,0);

	// Table Voltages
	do_sqlyy(ptr,"DROP TABLE [voltages];",0);
	do_sqlyy(ptr,"CREATE TABLE [voltages] ([id] INTEGER  PRIMARY KEY AUTOINCREMENT NOT NULL,[tag],[value]);",0);
	sprintf(sql,"INSERT INTO voltages ([id],[tag],[value]) VALUES('0','voltage_0','230');");
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"INSERT INTO voltages ([id],[tag],[value]) VALUES('1','voltage_1','230');");
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"INSERT INTO voltages ([id],[tag],[value]) VALUES('2','voltage_2','230');");
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"INSERT INTO voltages ([id],[tag],[value]) VALUES('3','voltage_3','230');");
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"INSERT INTO voltages ([id],[tag],[value]) VALUES('4','voltage_4','230');");
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"INSERT INTO voltages ([id],[tag],[value]) VALUES('5','voltage_5','230');");
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"INSERT INTO voltages ([id],[tag],[value]) VALUES('6','voltage_6','230');");
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"INSERT INTO voltages ([id],[tag],[value]) VALUES('7','voltage_7','230');");
	do_sqlyy(ptr,sql,0);

	}



// *****   sql_update_sensorvalues   -   Aktualisieren der Sensorwerte   *****

void sql_update_sensorvalues(sqlite3 *ptr)
	{
	char sql[1024];
	// static unsigned long int z = 0;

	// z++;
	sprintf(sql,"UPDATE measurements SET [TI]='%ld',[a0]='%1.2f',[a1]='%1.2f',[a2]='%1.2f',[a3]='%1.2f',[a4]='%1.2f',[a5]='%1.2f',[adc0]='%ld',[adc1]='%ld',[adc2]='%ld',[adc3]='%ld',[adc4]='%ld',[adc5]='%ld',[w0]='%ld',[w1]='%ld',[w2]='%ld',[w3]='%ld',[w4]='%ld',[w5]='%ld',[w6]='%ld',[w7]='%ld' WHERE [id]='0';",
		ti,powermeter_current[0],powermeter_current[1],powermeter_current[2],powermeter_current[3],powermeter_current[4],powermeter_current[5],
		powermeter_adc[0],powermeter_adc[1],powermeter_adc[2],powermeter_adc[3],powermeter_adc[4],powermeter_adc[5],
		powermeter_watt[0],powermeter_watt[1],powermeter_watt[2],powermeter_watt[3],powermeter_watt[4],powermeter_watt[5],powermeter_watt[6],powermeter_watt[7]);

	// printf("SQL=%s\n",sql);
	do_sqlyy(ptr,sql,0);
	// printf("OK.\n");
	}



// *****   Alarme abarbeiten und ggfs. Emails versenden   *****

void process_alerts(void)
	{
	signed long int t;
	char kommando[888];
	FILE *f;
	double wert = 0;
	signed char now_minute;
	signed char now_stunde;
	signed char now_tag;
	time_t timer;
	struct tm *tblock;
	signed short int calc_from_time;
	signed short int calc_to_time;
	signed short int calc_now_time;


	timer = time(NULL);
	tblock = localtime(&timer);
	now_minute = tblock->tm_min;
	now_stunde = tblock->tm_hour;
	now_tag = tblock->tm_wday; // 0:Sonntag, 1:Montag, ..., 6:Samstag
	calc_now_time = now_stunde * 100 + now_minute;

	// printf("now_tag=%d\n",now_tag);
	for(t = 0;t < max_alerts;t++)
		{
		if ( ((alert_time_day_montag[t]			> 0) && (now_tag == 1))
			|| ((alert_time_day_dienstag[t]		> 0) && (now_tag == 2))
			|| ((alert_time_day_mittwoch[t]		> 0) && (now_tag == 3))
			|| ((alert_time_day_donnerstag[t]	> 0) && (now_tag == 4))
			|| ((alert_time_day_freitag[t]		> 0) && (now_tag == 5))
			|| ((alert_time_day_samstag[t]		> 0) && (now_tag == 6))
			|| ((alert_time_day_sonntag[t]		> 0) && (now_tag == 0)) )
			{
			// printf("Wochentag fr Alarm %ld OK; time: %04d\n",t,calc_now_time);
			calc_from_time = alert_time_from_hour[t] * 100 + alert_time_from_minute[t];
			calc_to_time = alert_time_to_hour[t] * 100 + alert_time_to_minute[t];
			if ((calc_from_time <= calc_now_time) && (calc_to_time >= calc_now_time))
				{
				// printf("Uhrzeit fr Alarm %ld OK\n",t);
				if(alert_phase[t] >= 0)
					{
					if(alert_phase[t] == 0) wert = powermeter_watt[0];
					if(alert_phase[t] == 1) wert = powermeter_watt[1];
					if(alert_phase[t] == 2) wert = powermeter_watt[2];
					if(alert_phase[t] == 3) wert = powermeter_watt[0] + powermeter_watt[1] + powermeter_watt[2];
					// printf("Alarm %ld muss verarbeitet werden... ",t);
					if((wert > alert_maximum[t]) || (wert < alert_minimum[t]))
						{
						// printf("ALARM %ld !\n",t);
						if (alert_seccount[t] < 2048000)
							{
							alert_seccount[t]++;
							if(alert_seccount[t] >= alert_duration[t])
								{
								f = fopen("/tmp/mail.txt","wb");
								fprintf(f,"%s\n",alert_body[t]);
								fprintf(f,"%s: %ld W\n",phase0_name,powermeter_watt[0]);
								fprintf(f,"%s: %ld W\n",phase1_name,powermeter_watt[1]);
								fprintf(f,"%s: %ld W\n",phase2_name,powermeter_watt[2]);
								fprintf(f,"%s: %ld W\n",phaseall_name,powermeter_watt[0] + powermeter_watt[1] + powermeter_watt[2]);
								fprintf(f,"\n-ALL3690 Powermeter\n");
								fclose(f);
								sprintf(kommando,"makemime -a \"Subject: %s\" /tmp/mail.txt | sendmail -w3 -au%s -ap`php -r 'echo base64_decode(\"%s\");'` -S %s -f %s -t %s",
												alert_subject[t],smtp_username,smtp_password,smtp_server,smtp_sender,alert_recipient[t]);
								// printf("%s\n",kommando);
								system(kommando);
								alert_seccount[t] = 2048001;
								}
							}
						else
							{
							// Mail wurde bereits verschickt
							}
						}
					else
						{
						// printf("OK\n");
						alert_seccount[t] = -1;
						}
					}
				}
			}
		}
	}



// *****   Logging-Daten in die permanente Logging-DB schreiben   *****

void schreibe_datenfile_sqlite(signed long int w0,signed long int w1,signed long int w2,signed long int w3,signed long int w4,signed long int w5,signed long int w6,signed long int w7,signed long int w8,signed long int w9,signed long int sekunden)
	{
	char sql[1024];
	sqlite3 *handle;																			// Handle fr die DB


	sqlite3_open(LOG_DATABASE_FILE,&handle);							// SQLITE3-Datenbank ffnen
	sprintf(sql,"INSERT INTO measurements([w0],[w1],[w2],[w3],[w4],[w5],[w6],[w7],[w8],[w9],[duration]) VALUES ('%ld','%ld','%ld','%ld','%ld','%ld','%ld','%ld','%ld','%ld','%ld')",w0,w1,w2,w3,w4,w5,w6,w7,w8,w9,(long int)sekunden);
	log_append(1,sql);
	// printf("SQL=%s\n",sql);
	do_sqlyy(handle,sql,0);
	sqlite3_close(handle);																// Datenbank schlieen
	}



// *****   lese_history   -   History-Daten einlesen   *****

void lese_history(void)
	{
	signed long int retry = 0;
	FILE *f;
	signed long int t;
	signed long int i;
	double tmp_f[10];
	signed long int tmp_l[10];
	signed long int tmp_index;
	char puffer[1024];
	signed long int z = 0;


	for(t = 0;t < HISTORY_MAX;t++)
		{
		for (i = 0;i <= 9;i++)
			{
			history_watt[i][t] = 0;
			history_ampere[i][t] = 0;
			}
		}

nochmal:
	if (retry > 3) goto fertig;
	if (retry > 0)
		{
		printf("%ld. Versuch, die History-Datei zu lesen...\n",retry);
		sleep(1);
		}

	f = fopen("/tmp/history.txt","rb");
	if(f != NULL)
		{
schleiferl:
		i = fscanf(f,"%s",&puffer[0]);
		if(i == EOF) goto ende;
		if (strlen(puffer) < 5)
			{
			printf("Zu kurze Zeile: \"%s\"\n",puffer);
			goto schleiferl;
			}

		//             idx  a0  a1  a2  a3  a4  a5  w0  w1  w2  w3  w4  w5  w6  w7  w8  w9
		sscanf(puffer,"%ld,%lf,%lf,%lf,%lf,%lf,%lf,%ld,%ld,%ld,%ld,%ld,%ld,%ld,%ld,%ld,%ld",
			&tmp_index,&tmp_f[0],&tmp_f[1],&tmp_f[2],&tmp_f[3],&tmp_f[4],&tmp_f[5],
			&tmp_l[0],&tmp_l[1],&tmp_l[2],&tmp_l[3],&tmp_l[4],&tmp_l[5],&tmp_l[6],&tmp_l[7],&tmp_l[8],&tmp_l[9]);

		if ((tmp_index >= 0) && (tmp_index <= HISTORY_MAX))
			{
			z++;
			// printf("Zeile %ld: index %ld\n",z,tmp_index);
			for(i = 0;i < 10;i++)
				{
				history_watt[i][tmp_index] = tmp_l[i];
				history_ampere[i][tmp_index] = tmp_f[i];
				}
			}
		else
			{
			printf("falscher Index %ld: \"%s\"\n",tmp_index,puffer);
			goto schleiferl;
			}

		goto schleiferl;
ende:
		fclose(f);
		printf("%ld History-Werte gelesen.\n",z);
		}
	else
		{
		printf("Keine History-Werte gelesen, da /tmp/history.txt nicht geffnet werden konnte.\n");
		retry++;
		goto nochmal;
		}
fertig: ;
	}



// *****   schreibe_history   -   History-Daten schreiben   *****

void schreibe_history(void)
	{
	FILE *f;
	signed short int t;
	signed short int i;

	for(t = HISTORY_MAX - 2;t >= 0;t--)
		{
		for(i = 0;i <= 9;i++)
			{
			history_watt[i][t + 1] = history_watt[i][t];
			history_ampere[i][t + 1] = history_ampere[i][t];
			}
		}

		for(i = 0;i <= 9;i++)
			{
			history_watt[i][0] = powermeter_watt[i];
			history_ampere[i][0] = powermeter_current[i];
			}

	f = fopen("/tmp/history.tmp","wb");
	if(f != NULL)
		{
		for(t = 0;t < HISTORY_MAX;t++)
			{
			fprintf(f,"%d,%1.2f,%1.2f,%1.2f,%1.2f,%1.2f,%1.2f,",t,history_ampere[0][t],history_ampere[1][t],history_ampere[2][t],history_ampere[3][t],history_ampere[4][t],history_ampere[5][t]);
			fprintf(f,"%ld,%ld,%ld,%ld,%ld,%ld,%ld,%ld,%ld,%ld\n",history_watt[0][t],history_watt[1][t],history_watt[2][t],history_watt[3][t],history_watt[4][t],history_watt[5][t],history_watt[6][t],history_watt[7][t],history_watt[8][t],history_watt[9][t]);
			}
		fclose(f);
		system("mv /tmp/history.tmp /tmp/history.txt");
		}
	}



// *****   schreibe_datenfile_xml_ips   -   Sensordaten in XML-File schreiben (alte Version fr IP-Symcon)   *****

void schreibe_datenfile_xml_ips(void)
	{
	FILE *f;
	signed short int t;
	char path_tmpfile[80] = "/tmp/sensordata.tmp";
	char path_xml[80] = "/tmp/sensordata.xml";
	char ep[200];
	struct timeval tv;																		// Uhrzeit-Kram
	time_t curtime;																				// Uhrzeit-Kram
	struct tm *myTm;																			// Uhrzeit-Kram
	unsigned short int jahr;
	unsigned char monat;
	unsigned char tag;
	unsigned char stunde;
	unsigned char minute;
	unsigned char sekunde;



	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln 
  curtime = tv.tv_sec;																	// Lores-Zeit in s rausziehen
	myTm = localtime(&curtime);														// ...und berfhren in tm-Struktur
	stunde = myTm->tm_hour;																// Stnde
	minute = myTm->tm_min;																// Minute
	sekunde = myTm->tm_sec;																// Sekunde
	jahr = myTm->tm_year + 1900;													// Jahr
	monat = myTm->tm_mon + 1;															// Monat
	tag = myTm->tm_mday;																	// Tag

	f = fopen(path_tmpfile,"wb");
	if(f != NULL)
		{
		fprintf(f,"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\" ?>\n");
		fprintf(f,"<data>\n");
		for(t = 0;t < 10;t++)
			{
			fprintf(f," <sensor%d>\n",t);										// Sensornummer
			fprintf(f,"  <name>%d</name>\n",t); // Sensorbenennung
			fprintf(f,"  <value_float>%ld</value_float>\n",powermeter_watt[t]); // Sensorwert float
			fprintf(f,"  <value_int>%ld</value_int>\n",(signed long int)(powermeter_watt[t]) * 100); // Sensorwert int
			// fprintf(f,"  <value_string>%s</value_string>\n",logical_string[t]); // Sensorwert string
			fprintf(f," </sensor%d>\n",t);									// Sensornummer
			}
		for(t = 0;t < 8;t++)
			{
			fprintf(f," <adc%d>\n",t);
			fprintf(f,"  <value_int>%ld</value_int>\n",(signed long int)(adc_value[t] * 100));
			fprintf(f," </adc%d>\n",t);
			}
		for(t = 0;t < 3;t++)
			{
			fprintf(f," <adc_compensated%d>\n",t);
			fprintf(f,"  <value_int>%ld</value_int>\n",(signed long int)(adc_value[t] * 100 - adc_value[t + 3] * 100));
			fprintf(f," </adc_compensated%d>\n",t);
			}
		fprintf(f," <system>\n");
		fprintf(f,"  <date>%02d.%02d.%04d</date>\n",tag,monat,jahr);
		fprintf(f,"  <time>%02d:%02d:%02d</time>\n",stunde,minute,sekunde);
		fprintf(f,"  <devicename>%s</devicename>\n",device_name);
		fprintf(f,"  <devicetype>%s</devicetype>\n",device_type);
		fprintf(f,"  <sys>%ld</sys>\n",ti);
		fprintf(f," </system>\n");
		fprintf(f,"</data>\n");
		fclose(f);
		t = rename(path_tmpfile,path_xml);									// POSIX-Version des Unix "mv" Kommandos
		if (t != 0)
			{
			sprintf(ep,"*** ERROR: Could not move file \"%s\" to \"%s\"",path_tmpfile,path_xml);
			log_append(2,ep);																	// Fehler protokollieren
			}
		}
	else
		{
		sprintf(ep,"*** ERROR: Could not open file \"%s\" for write",path_tmpfile);
		log_append(2,ep);																		// Fehler protokollieren
		}

	}



// *****   schreibe_datenfile_xml   -   Sensordaten in XML-File schreiben   *****

void schreibe_datenfile_xml(void)
	{
	FILE *f;
	signed short int t;
	char path_tmpfile[80] = "/tmp/sensordata_new.tmp";
	char path_xml[80] = "/tmp/sensordata_new.xml";
	char ep[200];
	struct timeval tv;																		// Uhrzeit-Kram
	time_t curtime;																				// Uhrzeit-Kram
	struct tm *myTm;																			// Uhrzeit-Kram
	unsigned short int jahr;
	unsigned char monat;
	unsigned char tag;
	unsigned char stunde;
	unsigned char minute;
	unsigned char sekunde;
	// static double phase_lag[3] = {0,0,0};


	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln 
  curtime = tv.tv_sec;																	// Lores-Zeit in s rausziehen
	myTm = localtime(&curtime);														// ...und berfhren in tm-Struktur
	stunde = myTm->tm_hour;																// Stnde
	minute = myTm->tm_min;																// Minute
	sekunde = myTm->tm_sec;																// Sekunde
	jahr = myTm->tm_year + 1900;													// Jahr
	monat = myTm->tm_mon + 1;															// Monat
	tag = myTm->tm_mday;																	// Tag

	f = fopen(path_tmpfile,"wb");
	if(f != NULL)
		{
		fprintf(f,"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\" ?>\n");
		fprintf(f,"<data>\n");
		for(t = 0;t <= 9;t++)
			{
			fprintf(f," <sensor%d>\n",t);										// Sensornummer
			fprintf(f,"  <name>%d</name>\n",t); // Sensorbenennung
			fprintf(f,"  <watt_float>%1.2f</watt_float>\n",(double)powermeter_watt[t]); // Sensorwert float
			fprintf(f,"  <watt_int>%ld</watt_int>\n",powermeter_watt[t] * 100); // Sensorwert int
			fprintf(f,"  <ampere_float>%1.2f</ampere_float>\n",powermeter_current[t]); // Sensorwert float
			fprintf(f,"  <ampere_int>%ld</ampere_int>\n",(signed long int)(powermeter_current[t] * 100)); // Sensorwert int
			fprintf(f," </sensor%d>\n",t);									// Sensornummer
			}
/*
		for(t = 0;t < 8;t++)
			{
			fprintf(f," <adc%d>\n",t);
			fprintf(f,"  <value_int>%ld</value_int>\n",(signed long int)(adc_value[t] * 100));
			fprintf(f," </adc%d>\n",t);
			}
		for(t = 0;t < 3;t++)
			{
			fprintf(f," <adc_compensated%d>\n",t);
			fprintf(f,"  <value_int>%ld</value_int>\n",(signed long int)(adc_value[t] * 100 - adc_value[t + 3] * 100));
			fprintf(f," </adc_compensated%d>\n",t);
			}
*/
		fprintf(f," <system>\n");
		fprintf(f,"  <date>%02d.%02d.%04d</date>\n",tag,monat,jahr);
		fprintf(f,"  <time>%02d:%02d:%02d</time>\n",stunde,minute,sekunde);
		fprintf(f,"  <devicename>%s</devicename>\n",device_name);
		fprintf(f,"  <devicetype>%s</devicetype>\n",device_type);
		fprintf(f,"  <numericdevice>%d</numericdevice>\n",hwmodel);
		fprintf(f,"  <sys>%ld</sys>\n",ti);
		fprintf(f," </system>\n");
		fprintf(f,"</data>\n");
		fclose(f);
		t = rename(path_tmpfile,path_xml);									// POSIX-Version des Unix "mv" Kommandos
		if (t != 0)
			{
			sprintf(ep,"*** ERROR: Could not move file \"%s\" to \"%s\"",path_tmpfile,path_xml);
			log_append(2,ep);																	// Fehler protokollieren
			}
		}
	else
		{
		sprintf(ep,"*** ERROR: Could not open file \"%s\" for write",path_tmpfile);
		log_append(2,ep);																		// Fehler protokollieren
		}

	schreibe_history();
	}



// **********   Benachrichtigungen einlesen   **********

void notification_readconfig(void)
	{
	signed short int t;
	char ep[200];


	for (t = 0;t < MAX_NOTIFICATIONS;t++)
		{
		notification_enabled[t] = 0;												// Enabled 0/1
		notification_action[t] = -1;												// Art der Benachrichtigung
		notification_sendtime_hour[t] = 0;									// Versandzeit Stunde
		notification_sendtime_minute[t] = 0;								// Versandzeit Minute
		notification_name[t][0] = 0;												// Bezeichnung
		notification_subject[t][0] = 0;											// Email-Subject
		notification_recipient[t][0] = 0;										// Email-Empfnger
		notification_body[t][0] = 0;												// Email-Body
		}

	notifications_max = sqldb_read_long("/powermeter/config/notification/maxnotification");
	notifications_count = 0;

	for (t = 0;t < notifications_max;t++)
		{
		sprintf(ep,"/powermeter/config/notification%d/enabled",t);
		notification_enabled[t] = sqldb_read_long(ep);
		if (notification_enabled[t] > 0)
			{
			notifications_count++;
			sprintf(ep,"/powermeter/config/notification%d/action",t);
			notification_action[t] = sqldb_read_long(ep);
			sprintf(ep,"/powermeter/config/notification%d/sendtime_hour",t);
			notification_sendtime_hour[t] = sqldb_read_long(ep);
			sprintf(ep,"/powermeter/config/notification%d/sendtime_minute",t);
			notification_sendtime_minute[t] = sqldb_read_long(ep);
			sprintf(ep,"/powermeter/config/notification%d/name",t);
			sqldb_read_string(ep,notification_name[t]);
			sprintf(ep,"/powermeter/config/notification%d/subject",t);
			sqldb_read_string(ep,notification_subject[t]);
			sprintf(ep,"/powermeter/config/notification%d/recipient",t);
			sqldb_read_string(ep,notification_recipient[t]);
			sprintf(ep,"/powermeter/config/notification%d/body",t);
			sqldb_read_string(ep,notification_body[t]);
			}
		}

	notification_reloadcounter = sqldb_read_long("/powermeter/config/notification/reloadcounter");

	sprintf(ep,"%d Notifications read from DB",notifications_count);
	log_append(2,ep);																			// Ladevorgang protokollieren
	sprintf(ep,"%d Notifications read\n",notifications_count);
	}



// *****   Benachrichtigungen abarbeiten und ggfs. Emails versenden   *****

void process_notifications(void)
	{
	signed long int t;
	static signed char last_hour = -1;										// letzte Stunde fr Mitternachtsdetektierung
	signed char current_hour = -2;												// aktuelle Stunde
	signed char current_minute = -2;											// aktuelle Minute
	time_t time_utc;
	struct tm *ts;																				// Timestruct fr Localtime


	time_utc = time(NULL);																// Aktuelle Zeit ermitteln
	ts = localtime(&time_utc);														// in Ortszeit konvertieren
	current_hour = ts->tm_hour;														// Stunde ermitteln
	current_minute = ts->tm_min;													// Minute ermitteln
	if ((current_hour == 0) && (last_hour != current_hour))
		{
		for (t = 0;t < MAX_NOTIFICATIONS;t++)								// Um Mitternacht die Sendeflags lschen
			{
			flag_notification[t] = 0;
			}
		}
	if (last_hour != current_hour)
		{
		last_hour = current_hour;
		}


	for (t = 0;t < MAX_NOTIFICATIONS;t++)									// Notifications durchrdeln
		{
		if (flag_notification[t] == 0)											// War das heute schon ? Wenn nein, weitermachen
			{
			if (notification_enabled[t] > 0)									// Notification berhaupt aktiv ?
				{
				printf("Check Notification %ld (%02d:%02d) (now: %02d:%02d)\n",t,notification_sendtime_hour[t],notification_sendtime_minute[t],current_hour,current_minute);
				if (notification_sendtime_hour[t] <= current_hour) // hat die Stunde geschlagen ?
					{
					printf("Hour OK\n");
					if (notification_sendtime_minute[t] <= current_minute) // Minute erreicht ?
						{
						printf("Minute OK\n");
						notification_send(t);												// Benachrichtigung verschicken
						flag_notification[t] = 1;										// und merken, da sie bearbeitet wurde
						}
					}
				}
			}
		}
	}



// **********   Versenden einer Notification   *********

void notification_send(signed short int nummer)
	{
	// char ep[4096];
	time_t time_utc;
	time_t time_jetzt;
	time_t anfang_utc;
	struct tm *time_heute;																// Timestruct fr Localtime aktueller Tag
	struct tm *time_gestern;															// Timestruct fr Localtime Vortag
	struct tm *time_anfang;																// Timestruct fr Localtime Anfang
	signed short int heute_jahr;
	signed short int heute_monat;
	signed short int heute_tag;
	signed short int heute_wochentag;
	signed short int anfang_jahr;
	signed short int anfang_monat;
	signed short int anfang_tag;
	signed short int ende_jahr;
	signed short int ende_monat;
	signed short int ende_tag;
	char startzeit[11] = "2011-01-01";										// Stringausgabe der Startzeit
	char endzeit[11] = "2011-12-31";											// Stringausgabe der Startzeit
	double kwh[10];																				// Kilowattstunden-Summen
	signed short int t;
	char sql[4096];
	signed long int ws[10];
	FILE *f;
	char kommando[4096];


	// Ende des Berichtszeitraums ermitteln (Gestern)
	time_jetzt = time(NULL);															// Zeit in GMT ermitteln
	time_heute = localtime(&time_jetzt);									// Heutiges Datum ermitteln
	heute_tag = time_heute->tm_mday;
	heute_monat = time_heute->tm_mon + 1;
	heute_jahr = time_heute->tm_year + 1900;
	heute_wochentag = time_heute->tm_wday;								// 0:Sonntag, 1:Montag, ..., 6:Samstag
	printf("Heute: %04d-%02d-%02d (wday=%d)\n",heute_jahr,heute_monat,heute_tag,heute_wochentag);

	time_utc = time(NULL);																// Zeit in GMT ermitteln
	time_utc -= 24 * 3600;																// Gestern ermitteln (1 Tag abziehen)
	time_gestern = localtime(&time_utc);
	ende_tag = time_gestern->tm_mday;
	ende_monat = time_gestern->tm_mon + 1;
	ende_jahr = time_gestern->tm_year + 1900;
	sprintf(endzeit,"%04d-%02d-%02d",ende_jahr,ende_monat,ende_tag);
	printf("Endzeit: %s\n",endzeit);

	// Anfang des Berichtszeitraums ermitteln
	if (notification_action[nummer] == 1)									// 1: Vortag
		{
		printf("Type 1 (Vortag)\n");
		strcpy(startzeit,endzeit);
		goto versenden;
		}
	if (notification_action[nummer] == 3)									// 3: letzte Woche
		{
		printf("Type 3 (letzte Woche)\n");
		if (heute_wochentag != 1)														// 0:Sonntag, 1:Montag, ..., 6:Samstag
			{
			printf ("Heute ist nicht Montag, also kein Report der vergangenen Woche ! (wday=%d)\n",heute_wochentag);
			goto nix_versenden;
			}
		printf ("Es ist Montag, also kann die letzte Woche ermittelt werden.\n");
		time_utc = time(NULL);															// Zeit in GMT ermitteln
		time_utc -= 24 * 3600;															// Gestern ermitteln (1 Tag abziehen)
		anfang_utc = time_utc - 24 * 3600 * 6;							// 6 Tage zurckrechnen, um auf den letzten Montag zu kommen
		time_anfang = localtime(&anfang_utc);
		anfang_tag = time_anfang->tm_mday;
		anfang_monat = time_anfang->tm_mon + 1;
		anfang_jahr = time_anfang->tm_year + 1900;
		sprintf(startzeit,"%04d-%02d-%02d",anfang_jahr,anfang_monat,anfang_tag);
		printf("Startzeit: %s\n",startzeit);
		goto versenden;
		}
	if (notification_action[nummer] == 4)									// 4: letzter Monat
		{
		printf("Type 4 (letzter Monat)\n");
		if (heute_tag != 1)																	// Tag berprfen
			{
			printf ("Heute ist nicht der 1. im Monat, also kein Report des Vormonats ! (day=%d)\n",heute_tag);
			goto nix_versenden;
			}
		sprintf(startzeit,"%04d-%02d-01",ende_jahr,ende_monat);
		printf("Startzeit: %s\n",startzeit);
		goto versenden;
		}
	printf("** ERROR ** Unbekannter Reporttyp %d\n",notification_action[nummer]);
	goto nix_versenden;


versenden:
	printf("*** notification_send(%d) ***:\n",nummer);
	printf("Zeitraum vom \"%s\" bis zum \"%s\"\n",startzeit,endzeit);

	sprintf(sql,"SELECT sum(w0*duration) as ws0,sum(w1*duration) as ws1,sum(w2*duration) as ws2,sum(w3*duration) as ws3,sum(w4*duration) as ws4,sum(w5*duration) as ws5,sum(w6*duration) as ws6,sum(w7*duration) as ws7,sum(w8*duration) as ws8,sum(w9*duration) as ws9 FROM measurements WHERE substr(datetime(timestamp,'localtime'),1,10)>='%s' AND substr(datetime(timestamp,'localtime'),1,10)<='%s';",startzeit,endzeit);
	// printf("SQL:\n%s\n--------------------------------------------------\n",sql);
	sqldb_read_measurement_sums(sql,&ws[0],&ws[1],&ws[2],&ws[3],&ws[4],&ws[5],&ws[6],&ws[7],&ws[8],&ws[9]);
	// printf("ws0:%ld, ws1:%ld, ws2:%ld\n",ws[0],ws[1],ws[2]);
	// printf("ws3:%ld, ws4:%ld, ws5:%ld\n",ws[3],ws[4],ws[5]);
	// printf("ws6:%ld, ws7:%ld, ws8:%ld, ws9:%ld\n",ws[6],ws[7],ws[8],ws[9]);
	for (t = 0;t < 10;t++)
		{
		kwh[t] = (double)ws[t] / 3600000;
		}

	printf("KWh0:%1.3f, KWh1:%1.3f, KWh2:%1.3f, ",kwh[0],kwh[1],kwh[2]);
	printf("KWh3:%1.3f, KWh4:%1.3f, KWh5:%1.3f\n",kwh[3],kwh[4],kwh[5]);


	f = fopen("/tmp/mail.txt","wb");
	fprintf(f,"%s\n",notification_body[t]);
	fprintf(f,"%s: %4.2f KWh\n",phase0_name,kwh[0]);
	fprintf(f,"%s: %4.2f KWh\n",phase1_name,kwh[1]);
	fprintf(f,"%s: %4.2f KWh\n",phase2_name,kwh[2]);
	fprintf(f,"%s: %4.2f KWh\n",phase3_name,kwh[3]);
	fprintf(f,"%s: %4.2f KWh\n",phase4_name,kwh[4]);
	fprintf(f,"%s: %4.2f KWh\n",phase5_name,kwh[5]);
	fprintf(f,"%s: %4.2f KWh\n",phase6_name,kwh[6]);
	fprintf(f,"%s: %4.2f KWh\n",phase7_name,kwh[7]);
	fprintf(f,"%s: %4.2f KWh\n",phase8_name,kwh[8]);
	fprintf(f,"%s: %4.2f KWh\n",phase9_name,kwh[9]);
	// fprintf(f,"%s: %4.2f W\n",phaseall_name,powermeter_watt[0] + powermeter_watt[1] + powermeter_watt[2]);
	fprintf(f,"\n\n-ALL3690 Powermeter\n");
	fclose(f);
	sprintf(kommando,"makemime -a \"Subject: %s\" /tmp/mail.txt | sendmail -w3 -au%s -ap`php -r 'echo base64_decode(\"%s\");'` -S %s -f %s -t %s",
					notification_subject[nummer],smtp_username,smtp_password,smtp_server,smtp_sender,notification_recipient[nummer]);
	printf("%s\n",kommando);
	system(kommando);


nix_versenden: ;
	}



// *****   sqldb_read_measurement_sums()   -   Daten vom Server holen   *****

void sqldb_read_measurement_sums(char sql[],signed long int *ws0,signed long int *ws1,signed long int *ws2,signed long int *ws3,signed long int *ws4,signed long int *ws5,signed long int *ws6,signed long int *ws7,signed long int *ws8,signed long int *ws9)
	{
	sqlite3 *sql_config_handle;														// Handle fr die DB
	sqlite3_stmt *vm;
	char *sql_query;
	signed short int i = 0;
	char zs[96];
	signed short int rw;


beginn:
	sqlite3_open("/data/powermeter.s3db",&sql_config_handle);// SQLITE3-Datenbank ffnen

	//printf("SQL:\"%s\"\n",sql);
	sql_query = sqlite3_mprintf("%s",sql);								// Vorbereitung des SQL-Querys
	
	// Umwandlung des SQl Querys in Byte-code
	rw = sqlite3_prepare(sql_config_handle,sql_query,strlen(sql_query),&vm,NULL);
	if (rw != SQLITE_OK)
		{
		// printf("(Error %d bei \"%s\")\n",rw,sql);
		// printf("SQLITE Read lock problem !!!!!\n");
		sqlite3_finalize(vm);																// Loeschen des Statements
		sqlite3_free(sql_query);														// freigeben des Bytecodes
		sqlite3_close(sql_config_handle);										// Datenbank schlieen
		usleep(25000);																			// 25 ms warten
		goto beginn;
		}

	// Ausgabe des Datensatzes
	while(sqlite3_step(vm) != SQLITE_DONE)
		{
		i++;
		if(sqlite3_column_text(vm,0) == NULL)
			{
			printf("NULL Pointer als Result bei \"%s\"\n",sql);
			*ws0 = 0;
			*ws1 = 0;
			*ws2 = 0;
			*ws3 = 0;
			*ws4 = 0;
			*ws5 = 0;
			*ws6 = 0;
			*ws7 = 0;
			*ws8 = 0;
			*ws9 = 0;
			goto final;
			}
		// printf("%s\n",sqlite3_column_text(vm,0));						// Inhalt des Datensatzes ausgeben
		strcpy(zs,(char *)sqlite3_column_text(vm,0));				// String aus dem DB-Recordset kopieren
		// printf("RW0:\"%s\"\n",zs);
		*ws0 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,1));				// String aus dem DB-Recordset kopieren
		*ws1 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,2));				// String aus dem DB-Recordset kopieren
		*ws2 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,3));				// String aus dem DB-Recordset kopieren
		*ws3 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,4));				// String aus dem DB-Recordset kopieren
		*ws4 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,5));				// String aus dem DB-Recordset kopieren
		*ws5 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,6));				// String aus dem DB-Recordset kopieren
		*ws6 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,7));				// String aus dem DB-Recordset kopieren
		*ws7 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,8));				// String aus dem DB-Recordset kopieren
		*ws8 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,9));				// String aus dem DB-Recordset kopieren
		*ws9 = atol(zs);
		}
	if(i < 1)																							// wenn kein Datensatz, -1 zurckgeben
		{
		*ws0 = -1;
		}

final:
	sqlite3_finalize(vm);																	// Loeschen des Statements
	sqlite3_free(sql_query);															// freigeben des Bytecodes
	sqlite3_close(sql_config_handle);											// Datenbank schlieen
	}



// **********   Demon-Configfile einlesen   **********

void demon_readconfig(void)
	{
	signed short int t;
	char ep[200];
	char tmp1[150] = "";
	char tmp3[150];

	sqldb_read_string("/control/devicename",device_name);
	// strcpy(device_type,"ALL3690_V1");
	sqldb_read_string("/sys/hardware/model",device_type);
	sqldb_read_string("/powermeter/config/measurementtitle0",phase0_name);
	sqldb_read_string("/powermeter/config/measurementtitle1",phase1_name);
	sqldb_read_string("/powermeter/config/measurementtitle2",phase2_name);
	sqldb_read_string("/powermeter/config/measurementtitle3",phase3_name);
	sqldb_read_string("/powermeter/config/measurementtitle4",phase4_name);
	sqldb_read_string("/powermeter/config/measurementtitle5",phase5_name);
	sqldb_read_string("/powermeter/config/measurementtitle6",phase6_name);
	sqldb_read_string("/powermeter/config/measurementtitle7",phase7_name);
	sqldb_read_string("/powermeter/config/measurementtitle8",phase8_name);
	sqldb_read_string("/powermeter/config/measurementtitle9",phase9_name);
	sqldb_read_string("/powermeter/config/measurementtitletotal",phaseall_name);

	for(t = 0;t < 8;t++)
		{
		sprintf(ep,"/powermeter/zero_offset_%d",t);
		zero_offset[t] = sqldb_read_long(ep);
		sprintf(ep,"/powermeter/multiply%d",t);
		powermeter_multiply[t] = sqldb_read_float(ep);
		sprintf(ep,"/powermeter/voltage%d",t);
		powermeter_voltage[t] = sqldb_read_float(ep);

		// printf("powermeter_zero_offset[%d]=%ld\n",t,zero_offset[t]);
		// printf("powermeter_multiply[%d]=%f\n",t,powermeter_multiply[t]);
		// printf("powermeter_voltage[%d]=%f\n",t,powermeter_voltage[t]);
		}

	for(t = 0;t < POWERMETER_MAX_ALERTS;t++)
		{
		alert_phase[t] = -1;
		alert_minimum[t] = -2048000.0;
		alert_maximum[t] = 2048000.0;
		alert_duration[t] = 30;
		strcpy(alert_recipient[t],"");
		sprintf(alert_subject[t],"Alert Email %d",t);
		strcpy(alert_body[t],"");
		alert_seccount[t] = -1;
		}

	max_alerts = sqldb_read_long("/powermeter/config/alert/maxalert");
	for(t = 0;t < max_alerts;t++)
		{
		sprintf(ep,"/powermeter/config/alert/alert%d",t);
		alert_phase[t] = sqldb_read_long(ep);
		sprintf(ep,"/powermeter/config/alert/minvalue%d",t);
		alert_minimum[t] = sqldb_read_float(ep);
		sprintf(ep,"/powermeter/config/alert/maxvalue%d",t);
		alert_maximum[t] = sqldb_read_float(ep);
		sprintf(ep,"/powermeter/config/alert/duration%d",t);
		alert_duration[t] = sqldb_read_long(ep);
		sprintf(ep,"/powermeter/config/alert/recipient%d",t);
		sqldb_read_string(ep,alert_recipient[t]);
		sprintf(ep,"/powermeter/config/alert/subject%d",t);
		sqldb_read_string(ep,alert_subject[t]);
		sprintf(ep,"/powermeter/config/alert/body%d",t);
		sqldb_read_string(ep,alert_body[t]);
		sprintf(ep,"/powermeter/config/alert/valid%d/on/mo",t);
		alert_time_day_montag[t] = sqldb_read_float(ep);
		sprintf(ep,"/powermeter/config/alert/valid%d/on/tu",t);
		alert_time_day_dienstag[t] = sqldb_read_float(ep);
		sprintf(ep,"/powermeter/config/alert/valid%d/on/we",t);
		alert_time_day_mittwoch[t] = sqldb_read_float(ep);
		sprintf(ep,"/powermeter/config/alert/valid%d/on/th",t);
		alert_time_day_donnerstag[t] = sqldb_read_float(ep);
		sprintf(ep,"/powermeter/config/alert/valid%d/on/fr",t);
		alert_time_day_freitag[t] = sqldb_read_float(ep);
		sprintf(ep,"/powermeter/config/alert/valid%d/on/sa",t);
		alert_time_day_samstag[t] = sqldb_read_float(ep);
		sprintf(ep,"/powermeter/config/alert/valid%d/on/su",t);
		alert_time_day_sonntag[t] = sqldb_read_float(ep);
		sprintf(ep,"/powermeter/config/alert/valid%d/from/hour",t);
		alert_time_from_hour[t] = sqldb_read_float(ep);
		sprintf(ep,"/powermeter/config/alert/valid%d/from/min",t);
		alert_time_from_minute[t] = sqldb_read_float(ep);
		sprintf(ep,"/powermeter/config/alert/valid%d/to/hour",t);
		alert_time_to_hour[t] = sqldb_read_float(ep);
		sprintf(ep,"/powermeter/config/alert/valid%d/to/min",t);
		alert_time_to_hour[t] = sqldb_read_float(ep);
		}
	printf("%ld Alerts loaded.\n",max_alerts);

	sqldb_read_string("/powermeter/config/dbpath",tmp1);
	sqldb_read_string("/powermeter/config/dbname",tmp3);
	strcpy(sqlite_db_filename,tmp1);
	strcat(sqlite_db_filename,"/");
	strcat(sqlite_db_filename,tmp3);
	sqlite_db_timeinterval = sqldb_read_long("/powermeter/config/dbtimeinterval");
	if(sqlite_db_timeinterval < 5) sqlite_db_timeinterval = 5;
	printf("logging all %ld seconds to DB at \"%s\"\n",sqlite_db_timeinterval,sqlite_db_filename);

	sqldb_read_string("/sys/network/mail/smtp",smtp_server);
	sqldb_read_string("/sys/network/mail/user",smtp_username);
	sqldb_read_string("/sys/network/mail/pass",smtp_password);
	sqldb_read_string("/sys/network/mail/sender",smtp_sender);
	}



// *****   multiplexer_select()   -   ADC multiplexen   *****

void multiplexer_select(unsigned char t)
	{
		if(t == 0)
			{
			gpio_port_write(PORT_B,20,0);
			gpio_port_write(PORT_B,21,0);
			gpio_port_write(PORT_B,22,0);
			}
		if(t == 1)
			{
			gpio_port_write(PORT_B,20,1);
			gpio_port_write(PORT_B,21,0);
			gpio_port_write(PORT_B,22,0);
			}
		if(t == 2)
			{
			gpio_port_write(PORT_B,20,0);
			gpio_port_write(PORT_B,21,1);
			gpio_port_write(PORT_B,22,0);
			}
		if(t == 3)
			{
			gpio_port_write(PORT_B,20,1);
			gpio_port_write(PORT_B,21,1);
			gpio_port_write(PORT_B,22,0);
			}
		if(t == 4)
			{
			gpio_port_write(PORT_B,20,0);
			gpio_port_write(PORT_B,21,0);
			gpio_port_write(PORT_B,22,1);
			}
		if(t == 5)
			{
			gpio_port_write(PORT_B,20,1);
			gpio_port_write(PORT_B,21,0);
			gpio_port_write(PORT_B,22,1);
			}
		if(t == 6)
			{
			gpio_port_write(PORT_B,20,0);
			gpio_port_write(PORT_B,21,1);
			gpio_port_write(PORT_B,22,1);
			}
		if(t == 7)
			{
			gpio_port_write(PORT_B,20,1);
			gpio_port_write(PORT_B,21,1);
			gpio_port_write(PORT_B,22,1);
			}
	usleep(1000);																					// 1 ms warten
	}



// **********   sensor_poll()   -   Sensoren abfragen   **********

void sensor_poll(void)
	{
	unsigned short int t;
	double tmp_float;

	// printf("[TI:%ld]   \n",ti);

	// ADC auf ID0 abfragen
	for(t = 0;t < 8;t++)
		{
		multiplexer_select(t);
		tmp_float = simplified_ads1100_read(0,0,0,10,0,14);
		// if (t < 6) printf("ADC%d:%1.2f(Z:%ld)",t,tmp_float,zero_offset[t]);
		if (tmp_float > 327.67) tmp_float = 0; // bei Overflow (< 0) auf 0 setzen

		adc_value[t] = tmp_float;
		if (tmp_float * 100 < (double)zero_offset[t])
			{
			tmp_float = 0;
			}
		else
			{
			tmp_float -= (double)zero_offset[t] / 100;
			}

		powermeter_tmp[t] = tmp_float;
		// printf("tmp_float:%1.2f, ",tmp_float);
		}
		// printf("\n");

	// Umkopieren der A/D Wandlerwerte
	powermeter_adc[0] = (signed long int)(adc_value[0] * 100.0);
	powermeter_adc[1] = (signed long int)(adc_value[1] * 100.0);
	powermeter_adc[2] = (signed long int)(adc_value[2] * 100.0);

	for(t = 0;t < 3;t++)
		{
		// powermeter_current[t] = (powermeter_tmp[t] - powermeter_tmp[t + 3]) * powermeter_multiply[t];
		powermeter_current[t] = powermeter_tmp[t] * powermeter_multiply[t];
		powermeter_watt[t] = powermeter_current[t] * powermeter_voltage[t];
		// printf("Kanal%d current:%1.2fA, voltage:%1.2fV, power:%ldW\n",t,powermeter_current[t],powermeter_voltage[t],powermeter_watt[t]);
		}
	for(t = 6;t < 8;t++)
		{
		powermeter_watt[t] = powermeter_tmp[t] / 20;				// Spannungswerte
		}


	for(t = 0;t < 8;t++)
		{
		if(powermeter_current[t] < 0) powermeter_current[t] = 0;
		if(powermeter_watt[t] < 0) powermeter_watt[t] = 0;
		}


	if (hwmodel == 3691)
		{
		// ALL3691, 2 Kanle

		// ADC auf ID1 abfragen
		for(t = 0;t < 6;t++)
			{
			multiplexer_select(t);
			tmp_float = simplified_ads1100_read(0,0,0,10,1,14);
			if (tmp_float > 327.67) tmp_float = 0; // bei Overflow (< 0) auf 0 setzen
			// if (t < 6) printf("2DC%d:%1.2f, ",t,tmp_float);

			adc_value[t] = tmp_float;
			if (tmp_float * 100 < (double)zero_offset[t + 3])
				{
				tmp_float = 0;
				}
			else
				{
				tmp_float -= (double)zero_offset[t + 3] / 100;
				}
			powermeter_tmp[t] = tmp_float;

			}
			// printf("\n");

		// Umkopieren der A/D Wandlerwerte
		powermeter_adc[3] = (signed long int)(adc_value[0] * 100.0);
		powermeter_adc[4] = (signed long int)(adc_value[1] * 100.0);
		powermeter_adc[5] = (signed long int)(adc_value[2] * 100.0);

		for(t = 3;t <= 5;t++)
			{
			powermeter_current[t] = powermeter_tmp[t - 3] * powermeter_multiply[t];
			powermeter_watt[t] = powermeter_current[t] * powermeter_voltage[t];
			// printf("Kanal%d current:%1.2fA, voltage:%1.2fV, power:%ldW\n",t,powermeter_current[t],powermeter_voltage[t],powermeter_watt[t]);
			}


		for(t = 0;t < 8;t++)
			{
			// if(powermeter_result[t] < 0) powermeter_result[t] = 0;
			if(powermeter_current[t] < 0) powermeter_current[t] = 0;
			if(powermeter_watt[t] < 0) powermeter_watt[t] = 0;
			}

		// D0-Bus Rckwerte
		sql_lese_d0zaehler(sql_var_handle);										// Zhlerstnde aus der tmp-DB lesen
		powermeter_current[8] = d0_a[0];
		powermeter_watt[8] = d0_w[0];
		powermeter_current[9] = d0_a[1];
		powermeter_watt[9] = d0_w[1];
		}
	else
		{
		// ALL3690 mit nur 1 Kanal
		powermeter_adc[3] = 0;
		powermeter_adc[4] = 0;
		powermeter_adc[5] = 0;

		// Rckwerte des 2. Kanals
		for (t = 3;t <= 5;t++)
			{
			powermeter_current[t] = 0;
			powermeter_watt[t] = 0;
			}

		// D0-Bus Rckwerte
		powermeter_current[8] = 0;
		powermeter_watt[8] = 0;
		powermeter_current[9] = 0;
		powermeter_watt[9] = 0;

		}

	}



// **********   Demon   **********

int main(int argc,char **argv)
	{
	signed short int start_ignore = 10;										// ignoiert die ersten 10 Messungen
	char ep[255];
	signed long int summe_watt[10];												// Summe Phasen
	unsigned long int summe_count;												// Anzahl Messungen
	time_t last_summe_timestamp;													// Zeitpunkt der letzten Messung
	signed long int summe_time_diff;											// Zeitunterschied in Sekunden
	time_t time_lt;
	signed long int t;
	signed long int s0_delta;															// Differenz S0-Ticks zum letzten mal
	double s0_kwh;																				// Energiemenge KWH
	double s0_joule;																			// Energiemenge in Joule (Wattsekunden)
	double s0_watt[2];																		// Leistung in Watt
	double d0_kwhx;																				// Energiemenge KWH
	double d0_joule;																			// Energiemenge in Joule (Wattsekunden)
	double d0_watt[2];																		// Leistung in Watt


	gpio_devmem_open();																		// GPIOs oeffnen

	gpio_port_init_output(PORT_B,20);										  // Multiplexer Adresse 0
	gpio_port_write(PORT_B,20,0);
	gpio_port_init_output(PORT_B,21);										  // Multiplexer Adresse 1
	gpio_port_write(PORT_B,21,0);
	gpio_port_init_output(PORT_B,22);										  // Multiplexer Adresse 2
	gpio_port_write(PORT_B,22,0);



	strcpy(log_filename,powermeter_logfile);							// Pfade fuer Logfile setzen
	// log_kill();																						// Logfile loeschen

	sqlite3_open(VAR_DATABASE_FILE,&sql_var_handle);			// SQLITE3-Datenbank ffnen
	sql_erase_sensorvalues(sql_var_handle);								// tempDB Tables lschen und neu anlegen

	hwmodel = sqldb_read_long("/sys/hardware/numeric_model");		// auf welcher Hardware sind wir ? 3690 oder 3691 ?
	demon_readconfig();																		// Einstellungen fuer den Sensordemon lesen
	notification_readconfig();														// Einstellungen fuer Benachrichtigungen laden

	sql_lese_s0ticks_permanent();													// gespeicherte S0-Zhlerstnde lesen
	sql_schreibe_s0ticks(sql_var_handle);									// und in die tmp-DB als Startwerte bernehmen

	sql_lese_spannungen_permanent();											// gespeicherte Spannungen der Phasen lesen
	sql_schreibe_spannungen(sql_var_handle);							// und in die tmp-DB als Startwerte bernehmen

	lese_history();																				// History lesen

	log_level = powermeter_loglevel;
	sprintf(ep,"----------   powermeter_demon V %s for ALL%d started (debuglevel=%d) S0_0:%ld, S0_1:%ld   ----------",POWERMETER_DEMON_VERSION,hwmodel,log_level,s0ticks[0],s0ticks[1]);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben
	printf("powermeter_demon V %s for ALL%d started (debuglevel=%d) S0_0:%ld, S0_1:%ld\n",POWERMETER_DEMON_VERSION,hwmodel,log_level,s0ticks[0],s0ticks[1]);

	i2c_ruhe(0);																					// Ruhe am Bus (Initialisierung auf Standardwerte)

	for (t = 0;t < MAX_NOTIFICATIONS;t++)
		{
		flag_notification[t] = 0; // TODO: fr normalen Betrieb wieder auf 0 setzen
		}
	for (t = 0;t <= 9;t++)
		{
		summe_watt[t] = 0;
		}
	summe_count = 0;
	last_summe_timestamp = time(NULL);

	s0ticks_old[0] = s0ticks[0];
	s0ticks_old[1] = s0ticks[1];
	s0ticks_per_kwh[0] = sqldb_read_long("/powermeter/s0/ticks_per_kwh_0");
	s0ticks_per_kwh[1] = sqldb_read_long("/powermeter/s0/ticks_per_kwh_1");
	if (s0ticks_per_kwh[0] <= 0) s0ticks_per_kwh[0] = 1000;
	if (s0ticks_per_kwh[1] <= 0) s0ticks_per_kwh[1] = 1000;
	printf("S0 Ticks pro KWH 0:%ld, 1:%ld\n",s0ticks_per_kwh[0],s0ticks_per_kwh[1]);

	sql_lese_d0zaehler(sql_var_handle);										// D0-Zhlerstnde aus der tmp-DB lesen
	d0_kwh_old[0] = d0_kwh[0];														// alte Zhlerstnde merken
	d0_kwh_old[1] = d0_kwh[1];

hauptschleife:
	if (notification_reloadcounter != sqldb_read_long("/powermeter/config/notification/reloadcounter"))
		{
		notification_readconfig();
		goto hauptschleife;
		}

	ti++;																									// Sys-Clock hochzhlen
	sensor_poll();																				// Sensoren abfragen

	if (start_ignore > 0)																	// die ersten paar Messungen ignorieren, damit sich das einpegeln kann
		{
		printf("ignoring Data (%d)...\n",start_ignore);
		start_ignore--;
		goto fertig;
		}

	// for(t = 0;t <= 9;t++)
	// 	{
	// 	printf("%ld:%ldW, ",t,powermeter_watt[t]);
	// 	}
	// printf("\n");

	sql_update_sensorvalues(sql_var_handle);							// in tmp-DB schreiben
	schreibe_datenfile_xml_ips();													// alte XML-Datei fr IP-Symcon erzeugen
	schreibe_datenfile_xml();															// XML-Datei erzeugen

	summe_count++;																				// Zhler fr Messungen erhhen
	for (t = 0;t <= 9;t++)
		{
		summe_watt[t] += powermeter_watt[t];								// Leistung summieren
		}

	time_lt = time(NULL);																	// Aktuelle Zeit ermitteln
	summe_time_diff = (signed long int)difftime(time_lt,last_summe_timestamp); // Differenz in Sekunden seit letzter Messung
	// printf("Zeit seit letzter Messung: %ld Sekunden\n",summe_time_diff);
	if (summe_time_diff < 0) summe_time_diff = -summe_time_diff;
	if(summe_time_diff >= sqlite_db_timeinterval)
		{
		if (summe_time_diff > sqlite_db_timeinterval + 60)
			{
			// Zeit ist um mehr als 60 Sekunden berzogen, da kann was nicht stimmen
			// z.B. Timserver korrupt, Datumsumstellung etc - also diesen Mewert verwerfen
			printf("Zeitdifferenz seit letzter Messung: %ld Sekunden, Mewerte ignoriert !\n",summe_time_diff);
			goto vergiss_es;
			}


		sql_lese_s0ticks(sql_var_handle);										// S0-Zhlerstnde aus der tmp-DB lesen

		s0_delta = s0ticks[0] - s0ticks_old[0];							// Differenz zum alten Zhlerstand ermitteln
		s0_kwh = (double)s0_delta / s0ticks_per_kwh[0];			// Energiemenge in KWh ausrechnen
		s0_joule = s0_kwh * 3600000;												// KW zu Watt: 1000, Stunde zu Sekunden: 3600 - nun kennen wir die Wattsekunden
		s0_watt[0] = s0_joule / summe_time_diff;						// wir kennen die Zeitdifferenz, somit knnen wir die durchschnittlichen Watt errechnen
		// printf("S0_0: delta=%ld, %1.4fKWh, %1.2fJ, %lds, %1.2fW\n",s0_delta,s0_kwh,s0_joule,summe_time_diff,s0_watt[0]);

		s0_delta = s0ticks[1] - s0ticks_old[1];							// Differenz zum alten Zhlerstand ermitteln
		s0_kwh = (double)s0_delta / s0ticks_per_kwh[1];			// Energiemenge in KWh ausrechnen
		s0_joule = s0_kwh * 3600000;												// KW zu Watt: 1000, Stunde zu Sekunden: 3600 - nun kennen wir die Wattsekunden
		s0_watt[1] = s0_joule / summe_time_diff;						// wir kennen die Zeitdifferenz, somit knnen wir die durchschnittlichen Watt errechnen
		// printf("S0_1: delta=%ld, %1.4fKWh, %1.2fJ, %lds, %1.2fW\n",s0_delta,s0_kwh,s0_joule,summe_time_diff,s0_watt[1]);


		sql_lese_d0zaehler(sql_var_handle);									// D0-Zhlerstnde aus der tmp-DB lesen

		if(d0_kwh_old[1] != 0)
			{
			d0_kwhx = d0_kwh[0] - d0_kwh_old[0];							// Differenz zum alten Zhlerstand ermitteln
			d0_joule = d0_kwhx * 3600000;											// KW zu Watt: 1000, Stunde zu Sekunden: 3600 - nun kennen wir die Wattsekunden
			d0_watt[0] = d0_joule / summe_time_diff;					// wir kennen die Zeitdifferenz, somit knnen wir die durchschnittlichen Watt errechnen
			sprintf(ep,"D0_0: %1.4fKWh, %1.2fJ, %lds, %1.2fW",d0_kwhx,d0_joule,summe_time_diff,d0_watt[0]);
			log_append(4,ep);
			}
		else
			{
			d0_kwhx = 0;																			// Differenz zum alten Zhlerstand ermitteln
			d0_joule = 0;																			// KW zu Watt: 1000, Stunde zu Sekunden: 3600 - nun kennen wir die Wattsekunden
			d0_watt[0] = 0;																		// wir kennen die Zeitdifferenz, somit knnen wir die durchschnittlichen Watt errechnen
			sprintf(ep,"D0_0: %1.4fKWh, %1.2fJ, %lds, %1.2fW (old=0)",d0_kwhx,d0_joule,summe_time_diff,d0_watt[0]);
			log_append(4,ep);
			}

		if(d0_kwh_old[1] != 0)
			{
			d0_kwhx = d0_kwh[1] - d0_kwh_old[1];							// Differenz zum alten Zhlerstand ermitteln
			d0_joule = d0_kwhx * 3600000;											// KW zu Watt: 1000, Stunde zu Sekunden: 3600 - nun kennen wir die Wattsekunden
			d0_watt[1] = d0_joule / summe_time_diff;					// wir kennen die Zeitdifferenz, somit knnen wir die durchschnittlichen Watt errechnen
			sprintf(ep,"D0_1: %1.4fKWh, %1.2fJ, %lds, %1.2fW",d0_kwhx,d0_joule,summe_time_diff,d0_watt[1]);
			log_append(4,ep);
			}
		else
			{
			d0_kwhx = 0;																			// Differenz zum alten Zhlerstand ermitteln
			d0_joule = 0;																			// KW zu Watt: 1000, Stunde zu Sekunden: 3600 - nun kennen wir die Wattsekunden
			d0_watt[1] = 0;																		// wir kennen die Zeitdifferenz, somit knnen wir die durchschnittlichen Watt errechnen
			sprintf(ep,"D0_1: %1.4fKWh, %1.2fJ, %lds, %1.2fW (old=0)",d0_kwhx,d0_joule,summe_time_diff,d0_watt[1]);
			log_append(4,ep);
			}

		schreibe_datenfile_sqlite(summe_watt[0] / summe_count,
															summe_watt[1] / summe_count,
															summe_watt[2] / summe_count,
															summe_watt[3] / summe_count,
															summe_watt[4] / summe_count,
															summe_watt[5] / summe_count,
															s0_watt[0],
															s0_watt[1],
															d0_watt[0],
															d0_watt[1],
															summe_time_diff);

vergiss_es:
		for (t = 0;t <= 9;t++)
			{
			summe_watt[t] = 0;
			}
		summe_count = 0;
		if ((s0ticks_old[0] != s0ticks[0]) || (s0ticks_old[1] != s0ticks[1]))
			{
			sql_schreibe_s0ticks_permanent();			
			s0ticks_old[0] = s0ticks[0];
			s0ticks_old[1] = s0ticks[1];
			}

		d0_kwh_old[0] = d0_kwh[0];
		d0_kwh_old[1] = d0_kwh[1];

		last_summe_timestamp = time_lt;
		}

	process_alerts();																			// Alarmberprfungen
	process_notifications();															// ggfs Benachrichtigungen versenden
	
fertig:
	goto hauptschleife;																		// ... und wieder dasselbe Spiel von vorne

// ende:
	sprintf(ep,"----------   powermeter_demon terminated   ----------");
	log_append(1,ep);																			// Endmeldung ins Logfile schreiben
	sqlite3_close(sql_var_handle);												// tmp-Datenbank schlieen
	return(0);
	}

