// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   rgb_program.c   -   Demon zum Ausfhren von Farbablufen ("Programmen") auf dem ALL365^1                      **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2011 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   Eintrge in der Konfigurations-Datenbank, die vom Demon ausgewertet werden   **********
// TODO: Udaten des Kommentars
// /control/rgb/programs/max_programs										Maximale Anzahl Programme
// /control/rgb/programs/program<x>/max_steps						Maximale Anzahl Zeilen in diesem Programm
// /control/rgb/programs/program<x>/name								Benennung des Programms
// /control/rgb/programs/program<x>/description					Blatext
// /control/rgb/programs/program<x>/step<y>/cmdtoken		Befehlstoken
// /control/rgb/programs/program<x>/step<y>/r0					Rot 0
// /control/rgb/programs/program<x>/step<y>/g0					Grn 0
// /control/rgb/programs/program<x>/step<y>/b0					Blau 0
// /control/rgb/programs/program<x>/step<y>/w0					Wei 0
// /control/rgb/programs/program<x>/step<y>/dim0				Dimspeed 0
// /control/rgb/programs/program<x>/step<y>/r1					Rot 1
// /control/rgb/programs/program<x>/step<y>/g1					Grn 1
// /control/rgb/programs/program<x>/step<y>/b1					Blau 1
// /control/rgb/programs/program<x>/step<y>/w1					Wei 1
// /control/rgb/programs/program<x>/step<y>/dim1				Dimspeed 1
// /control/rgb/programs/program<x>/step<y>/timer				Timer



#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <time.h>
#include <sys/time.h>
#include <sys/shm.h>
#include "at91gpio.h"
#include "all_i2c.h"
#include "i2c_chips.h"
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "rgb_program.h"
#include "sqldb.h"
#include "logging.h"
#include "shm_use.h"



// *****   Globale Variablen   *****

#define CONFIG_DATABASE_FILE "/etc/allnetenv/config.s3db"
#define VAR_DATABASE_FILE "/tmp/var.s3db"


signed short int	programm_enabled = 0;									// Programm aktiv 0/1
signed short int	programm_nummer = -1;									// Nummer des gerade aktiven Programms
signed short int	programm_az = -1;											// gerade abgearbeitete Zeilennummer
signed short int	programm_status = -1;									// Status (-2:muss geladen werden, -1:angehalten, 1: in Delay innerhalb der Zeile, 0:Zeile muss bearbeitet werden)
signed short int	programm_delaycounter = 0;						// Delayzhler innerhalb der Zeile
signed short int	programm_zeilennummer = -1;						// aktuelle Zeilennummer
signed short int	programm_max_steps = 16;							// Anzahl zulssiger Zeilennummern
signed short int	fernbedienung = -1;										// Typ der Fernbedienung (-1: Keine, 0: alte kleine FB mit altem Tiny85,
																												//                        1: PortaLuce FB mit neuem Tiny85,
																												//                        2: alte kleine FB mit neuem Tiny85)
signed short int	old_tiny_major = 0;										// IR-Empfnger Major ID
signed short int	old_tiny_minor = 0;										// IR-Empfnger Minor ID
signed long int		p_timeslice = 10;											// Zeitbasis fr Delay 1/x Sekunde

struct programmzeile
	{
	signed short int 		cmdtoken;													// 0:NOP, 1:RGB_Set, 2:Restart 3:End
	signed short int		r0;
	signed short int		g0;
	signed short int		b0;
	signed short int		w0;
	signed short int		dim0;
	signed short int		r1;
	signed short int		g1;
	signed short int		b1;
	signed short int		w1;
	signed short int		dim1;
	double							delay;
	signed short int		target;
	signed short int		switchtype;
	} pz;
struct programmzeile programm_zeile[1001];


/*
// *****   Zufall - Ermitteln einer Zufallszahl zwischen 0...1   *****

	double Zufall() 
	{
	long x;
	long y;
	double rw;

	x = Wert_A * (Letzte_ZahlX & 65535) + (Letzte_ZahlX >> 16);
	y = Wert_B * (Letzte_ZahlY & 65535) + (Letzte_ZahlY >> 16);

	rw = (double)((x << 16) + (y & 65525));
	Letzte_ZahlX = x;
	Letzte_ZahlY = y;
	return (rw / 4294967296.0 + 0.5);
	}
*/


// *****   zufall_long - Ermitteln einer Zufallszahl zwischen 0...4294967295   *****

unsigned long int zufall_long() 
	{
	static long Wert_A = 26004;														// Random Seed A
	static long Wert_B = 26088;														// Random Seed B
	static long Letzte_ZahlX = 26004;											// old seed A
	static long Letzte_ZahlY = 26088;											// old seed B
	long x;
	long y;
	unsigned long int rw;

	x = Wert_A * (Letzte_ZahlX & 65535) + (Letzte_ZahlX >> 16);
	y = Wert_B * (Letzte_ZahlY & 65535) + (Letzte_ZahlY >> 16);

	rw = (x << 16) + (y & 65525);
	Letzte_ZahlX = x;
	Letzte_ZahlY = y;
	return (rw);
	}



// *****   sql_write_rgbvalues   -   Schreiben der RGB-Output-Werte in die tmpDB   *****

void sql_write_rgbvalues(sqlite3 *ptr,unsigned char r0,unsigned char g0,unsigned char b0,unsigned char w0,unsigned char d0,unsigned char r1,unsigned char g1,unsigned char b1,unsigned char w1,unsigned char d1)
	{
	char sql[255];

	sprintf(sql,"UPDATE rgboutput SET r0=%d,g0=%d,b0=%d,w0=%d,d0=%d,r1=%d,g1=%d,b1=%d,w1=%d,d1=%d WHERE id = 0;",r0,g0,b0,w0,d0,r1,g1,b1,w1,d1);
	// printf("SQL: \"%s\"\n",sql);
	do_sqlyy(ptr,sql,0);
	}



// *****   programm_load(nummer) - RGB-Steuerprogramm laden   *****

void programm_load(unsigned short int nummer)
	{
	char tmpstr[80];																			// Blafasel
	char zahlenstring[80];


	sprintf(tmpstr,"SELECT [enabled] AS [value] FROM [programs] WHERE [id]='%d'",nummer);
	sqldb_read_string3(sql_config_handle,tmpstr,zahlenstring);
	programm_enabled = atol(zahlenstring);

	sprintf(tmpstr,"SELECT count(*) AS [value] FROM [programsteps] WHERE [program_id]='%d'",nummer);
	// printf("SQL=\"%s\"\n",tmpstr);
	sqldb_read_string3(sql_config_handle,tmpstr,zahlenstring);
	programm_max_steps = atol(zahlenstring);


	printf("Loading Program #%d (enabled:%d, %d Steps)...",nummer,programm_enabled,programm_max_steps);
	if (programm_enabled > 0) programm_max_steps = programm_load_zeilen(sql_config_handle,nummer);
	printf(" %d Steps loaded.\n",programm_max_steps);
	// print_listing();
	}



// *****   programm_load_zeilen   -   Programmzeilen einlesen   *****

signed long int programm_load_zeilen(sqlite3 *sql_config_handle,unsigned short int nummer)
	{
	signed long int rw = 0;
	// sqlite3 *sql_config_handle;														// Handle fr die DB
	sqlite3_stmt *vm;
	char *sql_query;
	signed short int i = 0;
	char zs[96];
	char sql[255];
	signed long int anzahl = 0;


	sprintf(sql,"SELECT [id],[r0],[g0],[b0],[w0],[d0],[r1],[g1],[b1],[w1],[d1],[command],[delay],[actor],[switch] FROM programsteps WHERE [program_id]='%d' ORDER BY [id]",nummer);

beginn:

	// printf("SQL:\"%s\"\n",sql);
	sql_query = sqlite3_mprintf("%s",sql);								// Vorbereitung des SQL-Querys
	
	// Umwandlung des SQl Querys in Byte-code
	rw = sqlite3_prepare(sql_config_handle,sql_query,strlen(sql_query),&vm,NULL);
	if (rw != SQLITE_OK)
		{
		// printf("(Error %d bei \"%s\")\n",rw,sql);
		// printf("SQLITE Read lock problem !!!!!\n");
		sqlite3_finalize(vm);																// Loeschen des Statements
		sqlite3_free(sql_query);														// freigeben des Bytecodes
		usleep(25000);																			// 25 ms warten
		goto beginn;
		}

	// Ausgabe der Datenstze
	while(sqlite3_step(vm) != SQLITE_DONE)
		{
		i++;
		if(sqlite3_column_text(vm,0) == NULL)
			{
			printf("NULL Pointer als Result bei \"%s\"\n",sql);
			goto final;
			}
		// printf("ID: %s\n",sqlite3_column_text(vm,0));				// Inhalt der Datensatz-ID ausgeben

		strcpy(zs,(char *)sqlite3_column_text(vm,1));				// String aus dem DB-Recordset kopieren
		programm_zeile[anzahl].r0 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,2));				// String aus dem DB-Recordset kopieren
		programm_zeile[anzahl].g0 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,3));				// String aus dem DB-Recordset kopieren
		programm_zeile[anzahl].b0 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,4));				// String aus dem DB-Recordset kopieren
		programm_zeile[anzahl].w0 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,5));				// String aus dem DB-Recordset kopieren
		programm_zeile[anzahl].dim0 = atol(zs);

		strcpy(zs,(char *)sqlite3_column_text(vm,6));				// String aus dem DB-Recordset kopieren
		programm_zeile[anzahl].r1 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,7));				// String aus dem DB-Recordset kopieren
		programm_zeile[anzahl].g1 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,8));				// String aus dem DB-Recordset kopieren
		programm_zeile[anzahl].b1 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,9));				// String aus dem DB-Recordset kopieren
		programm_zeile[anzahl].w1 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,10));			// String aus dem DB-Recordset kopieren
		programm_zeile[anzahl].dim1 = atol(zs);

		strcpy(zs,(char *)sqlite3_column_text(vm,11));			// String aus dem DB-Recordset kopieren
		programm_zeile[anzahl].cmdtoken = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,12));			// String aus dem DB-Recordset kopieren
		programm_zeile[anzahl].delay = atof(zs) * p_timeslice; // Zeitraster 1/timeslice Sekunde (50 ms)
		strcpy(zs,(char *)sqlite3_column_text(vm,13));			// String aus dem DB-Recordset kopieren
		programm_zeile[anzahl].target = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,14));			// String aus dem DB-Recordset kopieren
		programm_zeile[anzahl].switchtype = atol(zs);

		anzahl++;
		}
	if(i < 1)																							// wenn kein Datensatz, 0 zurckgeben
		{
		anzahl = 0;
		}

final:
	sqlite3_finalize(vm);																	// Loeschen des Statements
	sqlite3_free(sql_query);															// freigeben des Bytecodes


	return(anzahl);
	}



// *****   print_listing   -   Listing des geladenen Programms als Pseudocode ausgeben   *****

void print_listing(void)
	{
	signed long int t;


	for (t = 0;t < programm_max_steps;t++)
		{
		print_einzelzeile(t);
		}
	}



// *****   print_einzelzeile   -   Ausgeben einer einzelnen Programmzeile als Pseudocode   *****

void print_einzelzeile(unsigned long int t)
	{
	printf("%ld: ",t);																		// Zeilennummer ausgeben
	if (programm_zeile[t].cmdtoken == 0)
		{
		printf ("-\n");
		goto zeilenende;
		}
	if (programm_zeile[t].cmdtoken == 1)
		{
		printf ("RGB_SET (R0:%d, G0:%d, B0:%d, W0:%d, Dim0:%d) (R1:%d, G1:%d, B1:%d, W1:%d, Dim1:%d)",
						programm_zeile[t].r0,programm_zeile[t].g0,programm_zeile[t].b0,programm_zeile[t].w0,programm_zeile[t].dim0,
						programm_zeile[t].r1,programm_zeile[t].g1,programm_zeile[t].b1,programm_zeile[t].w1,programm_zeile[t].dim1);
		if (programm_zeile[t].delay > 0) printf(" Delay:%1.2f s",programm_zeile[t].delay / (double)p_timeslice);
		printf("\n");
		goto zeilenende;
		}
	if (programm_zeile[t].cmdtoken == 2)
		{
		printf ("RESTART\n");
		goto zeilenende;
		}
	if (programm_zeile[t].cmdtoken == 3)
		{
		printf ("END\n");
		goto zeilenende;
		}
	if (programm_zeile[t].cmdtoken == 4)
		{
		printf ("ACTOR_SET (Target:%d, Switch:%d)",programm_zeile[t].target,programm_zeile[t].switchtype);
		if (programm_zeile[t].delay > 0) printf(" Delay:%1.1f s",programm_zeile[t].delay);
		printf("\n");
		goto zeilenende;
		}
	printf ("(unknown cmdtoken: %d)\n",programm_zeile[t].cmdtoken);
zeilenende: ;
	}



// *****   Hauptschleife   *****

int main(int argc,char **argv)
	{
	signed long int bootdefault_program;									// Programm, das beim Start ausgefhrt werden soll (oder -1 wenn keines)
	unsigned char 	old_r0 = 0;
	unsigned char 	old_g0 = 0;
	unsigned char 	old_b0 = 0;
	unsigned char 	old_w0 = 0;
	unsigned char 	old_r1 = 0;
	unsigned char 	old_g1 = 0;
	unsigned char 	old_b1 = 0;
	unsigned char 	old_w1 = 0;
	unsigned char 	old_dim0 = 254;
	unsigned char 	old_dim1 = 255;
	unsigned long		ti = 0;																// Zhler, geht bei jedem Durchlauf um 1 hoch
	unsigned long		az = 0;																// nderungszhler RGBW + Dim
	signed long int	programm_kommando = -1;								// Befehl von der Bedienoberflche
	char						kommando[255];												// Kommandostring
	char						lstring[255];													// Puffer fr Logging
	signed long int	rpart;																// tmpvar fr Zufallsgenerierung
	char						ep[1024];															// Puffer fr Logfile-Meldungen
	// signed long int	uhr_last_second = -1;									// letzte Sekunde
	// signed long int	uhr_sekunde;													// aktuelle Sekunde
	char						sql[512];															// fr SQL-Strings
	char						rstring[80];													// Rckgabestring fr DB-Queries


	// auf existierendes Shared Memory fr Demonstatus zugreifen
	if (use_my_demonstatus_shm(4) != 1) goto errorende;		// MY_DEMON_ID = 4: rgb_program

	// Status von Demon in SHM eintragen
	my_demonstatus_data[my_demon_id].demon_id = my_demon_id;														// 0 ist Default, hier wird die eigene ID eingetragen
	sprintf(my_demonstatus_data[my_demon_id].name_string,"rgb_program");								// Name
	sprintf(my_demonstatus_data[my_demon_id].version_string,"%s",RGBPROGRAM_VERSION);		// Versionsnummer
	my_demonstatus_data[my_demon_id].last_boot = time(0);																// Timestamp(UTC) des Starts
	my_demonstatus_data[my_demon_id].last_activity = time(0);														// Timestamp(UTC) der letzten Aktion
	my_demonstatus_data[my_demon_id].ti = ti;																						// Counter
	sprintf(my_demonstatus_data[my_demon_id].debug_message,"Start");										// letzte Debug-Meldung
	my_demonstatus_data[my_demon_id].error_message[0] = 0;															// letzte Fehlermeldung
	my_demonstatus_data[my_demon_id].status_long = 0;																		// numerischer Status
	sprintf(my_demonstatus_data[my_demon_id].status_string,"startup");									// String-Status

	// Logging-System initialisieren
	strcpy (log_filename,"/www/reports/rgb_program.log");	// Logfilenamen setzen
	log_level = 4;																				// Debuglevel setzen
	// log_kill();																						// Logfile loeschen
	sprintf(ep,"----------   rgb_program V %s started (debuglevel=%d)   ----------",RGBPROGRAM_VERSION,log_level);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben


	sqlite3_open(VAR_DATABASE_FILE,&sql_var_handle);			// SQLITE3-Datenbank ffnen
	sqlite3_open(CONFIG_DATABASE_FILE,&sql_config_handle);// SQLITE3-Datenbank ffnen


	// Start loggen
	sprintf(lstring,"**********   rgb_program V %s started   **********",RGBPROGRAM_VERSION);
	log_append(1,lstring);


	printf("RGB program V %s started. ",RGBPROGRAM_VERSION);


	// Bootdefault-Programm laden, wenn eines existiert
	bootdefault_program = sqldb_read_long("/control/rgb/bootdefault/program_number");
	if (bootdefault_program >= 0)
		{
		printf("Bootdefault Programm: %ld\n",bootdefault_program);
		programm_load(bootdefault_program);
		programm_status = 0;
		programm_az = 0;
		}
	else
		{
		printf("kein Bootdefault-Programm\n");
		programm_status = -1;
		programm_az = 0;
		}
	// Status und Programmzeile in tmpDB schreiben
	sprintf(sql,"UPDATE [program] set [loaded_program]='%ld',[running_line]='%d',[running_status]='%d',[new_program]='-1',[new_command]='0' where [id]='0';",
					bootdefault_program,programm_az,programm_status);
	// printf("SQL=\"%s\"\n",sql);
	do_sqlyy(sql_var_handle,sql,0);
	programm_nummer = bootdefault_program;
	log_append(1,"done. Ready to work.");


	update_demondebug("running");													// letzte Debug-Meldung
hauptschleife:
	update_demonstatus(ti,1,"main loop");									// Status von Demon in SHM aktualisieren
	usleep(1000000 / p_timeslice);												// 50 ms (1/20s) warten
hauptschleife_nodelay:
	zufall_long();																				// Zufall immer schn mitlaufen lassen
	ti++;																									// Schleifenzhler erhhen

	// uhr_sekunde = time(0);																// Uhrzeit holen
	// printf("s=%ld,  ti=%ld\n",uhr_sekunde,ti);



	// ----- Programmcode-Interpreter -----
	// if (uhr_last_second != uhr_sekunde)
		{
		update_demonstatus(ti,2,"exec");										// Status von Demon in SHM aktualisieren
		// printf("**TP(Z%d): Start, Status:%d **\n",programm_az,programm_status);
		// uhr_last_second = uhr_sekunde;
		sprintf(sql,"SELECT [new_command] as [value] FROM [program] WHERE [id]='0'");
		sqldb_read_string3(sql_var_handle,sql,rstring);
		programm_kommando = atol(rstring);
		// if (programm_kommando != 0) printf("programm_kommando: %ld\n",programm_kommando);
		if (programm_kommando == -2)
			{
			update_demondebug("Program Stop");								// Status von Demon in SHM aktualisieren
			programm_status = -1;															// Programm stoppen
			// sqldb_write_long("/control/rgb/var/program_cmd",-1);
			sprintf(sql,"UPDATE [program] set [new_command]='0',[running_status]='%d' where [id]='0';",programm_status);
			// printf("SQL=\"%s\"\n",sql);
			do_sqlyy(sql_var_handle,sql,0);
			}
		if (programm_kommando >= 10000)
			{
			sprintf(ep,"Load:%ld",programm_kommando - 10000);
			update_demondebug(ep);														// Status von Demon in SHM aktualisieren
			programm_nummer = programm_kommando - 10000;			// Programmnummer ermitteln
			programm_load(programm_nummer);
			programm_status = 0;
			programm_az = 0;
			// Status und Programmzeile in tmpDB schreiben
			sprintf(sql,"UPDATE [program] set [loaded_program]='%d',[running_line]='%d',[running_status]='%d',[new_program]='-1',[new_command]='0' where [id]='0';",
							programm_nummer,programm_az,programm_status);
			//printf("SQL=\"%s\"\n",sql);
			do_sqlyy(sql_var_handle,sql,0);
			}

		if (programm_nummer >= 0)
			{
			// if (programm_status != -1) printf("programm_status=%d, programm_az=%d, Token=%d, Delay=%d\n",programm_status,programm_az,programm_zeile[az].cmdtoken,programm_delaycounter);
			if (programm_status == -1)												// Programm angehalten (inaktiv)
				{
				goto programm_weiter;
				}

// programm_machdas:
			if (programm_status == 0)													// Programm luft, Zeile muss abgearbeitet werden
				{
				sprintf(ep,"P:%d,Line:%d",programm_nummer,programm_az);
				update_demondebug(ep);													// Status von Demon in SHM aktualisieren

				if (programm_zeile[programm_az].cmdtoken == 0) // NOP
					{
					// printf("(exec)--NOP\n");
					programm_status = -1;
					goto programm_engine_finished;
					}
				if (programm_zeile[programm_az].cmdtoken == 1) // RGB set
					{
					// printf("**TP(Z%d): RGB_set, Delay:%d **\n",programm_az,programm_zeile[programm_az].delay);
					if (programm_zeile[programm_az].r0 >= 0)
						{
						t_r0 = programm_zeile[programm_az].r0;
						}
					else
						{
						rpart = -programm_zeile[programm_az].r0;
						if(rpart < 2) rpart = 2;
						if(rpart > 255) rpart = 255;
						t_r0 = (zufall_long() % rpart) * (255 / (rpart - 1));
						}
					if (programm_zeile[programm_az].g0 >= 0)
						{
						t_g0 = programm_zeile[programm_az].g0;
						}
					else
						{
						rpart = -programm_zeile[programm_az].g0;
						if(rpart < 2) rpart = 2;
						if(rpart > 255) rpart = 255;
						t_g0 = (zufall_long() % rpart) * (255 / (rpart - 1));
						}
					if (programm_zeile[programm_az].b0 >= 0)
						{
						t_b0 = programm_zeile[programm_az].b0;
						}
					else
						{
						rpart = -programm_zeile[programm_az].b0;
						if(rpart < 2) rpart = 2;
						if(rpart > 255) rpart = 255;
						t_b0 = (zufall_long() % rpart) * (255 / (rpart - 1));
						}
					if (programm_zeile[programm_az].w0 >= 0)
						{
						t_w0 = programm_zeile[programm_az].w0;
						}
					else
						{
						rpart = -programm_zeile[programm_az].w0;
						if(rpart < 2) rpart = 2;
						if(rpart > 255) rpart = 255;
						t_w0 = (zufall_long() % rpart) * (255 / (rpart - 1));
						}
					t_dim0 = programm_zeile[programm_az].dim0;
					if (programm_zeile[programm_az].r1 >= 0)
						{
						t_r1 = programm_zeile[programm_az].r1;
						}
					else
						{
						rpart = -programm_zeile[programm_az].r1;
						if(rpart < 2) rpart = 2;
						if(rpart > 255) rpart = 255;
						t_r1 = (zufall_long() % rpart) * (255 / (rpart - 1));
						}
					if (programm_zeile[programm_az].g1 >= 0)
						{
						t_g1 = programm_zeile[programm_az].g1;
						}
					else
						{
						rpart = -programm_zeile[programm_az].g1;
						if(rpart < 2) rpart = 2;
						if(rpart > 255) rpart = 255;
						t_g1 = (zufall_long() % rpart) * (255 / (rpart - 1));
						}
					if (programm_zeile[programm_az].b1 >= 0)
						{
						t_b1 = programm_zeile[programm_az].b1;
						}
					else
						{
						rpart = -programm_zeile[programm_az].b1;
						if(rpart < 2) rpart = 2;
						if(rpart > 255) rpart = 255;
						t_b1 = (zufall_long() % rpart) * (255 / (rpart - 1));
						}
					if (programm_zeile[programm_az].w1 >= 0)
						{
						t_w1 = programm_zeile[programm_az].w1;
						}
					else
						{
						rpart = -programm_zeile[programm_az].w1;
						if(rpart < 2) rpart = 2;
						if(rpart > 255) rpart = 255;
						t_w1 = (zufall_long() % rpart) * (255 / (rpart - 1));
						}
					t_dim1 = programm_zeile[programm_az].dim1;
					// printf("(exec)--RGB_set %d %d %d %d  %d %d %d %d  %d %d\n",t_r0,t_g0,t_b0,t_w0, t_r1,t_g1,t_b1,t_w1, t_dim0,t_dim1);
					sprintf(lstring,"(Control_Debug) Program(Line %d): r0:%d,g0:%d,b0:%d,w0:%d, r1:%d,r2:%d,r3:%d,w1:%d Dim0:%d,Dim1:%d",programm_az,t_r0,t_g0,t_b0,t_w0,t_r1,t_g1,t_b1,t_w1,t_dim0,t_dim1);
					log_append(4,lstring);
					programm_delaycounter = programm_zeile[programm_az].delay;
					if (programm_delaycounter > 0)
						{
						programm_status = 1;
						goto programm_engine_finished;
						}
					goto programm_nextline;
					}
				if (programm_zeile[programm_az].cmdtoken == 2) // Restart
					{
					// printf("(exec)--Restart\n");
					programm_az = 0;
					// goto programm_weiter;
					goto hauptschleife_nodelay;
					}
				if (programm_zeile[programm_az].cmdtoken == 3) // End
					{
					// printf("(exec)--STOP\n");
					programm_status = -1;
					programm_delaycounter = -2;
					goto programm_weiter;
					}
				if (programm_zeile[programm_az].cmdtoken == 4) // logischen Aktor schalten
					{
					// printf("(exec)--ACTOR_W Target:%d, Switch: %d\n",programm_zeile[programm_az].target,programm_zeile[programm_az].switchtype);		
					sprintf(kommando,"/usr/sbin/actor_w_write \"%d\" \"%d\" >/dev/null",programm_zeile[programm_az].target,programm_zeile[programm_az].switchtype);
					// printf("cmd=%s\n",kommando);
					system(kommando);
					programm_delaycounter = programm_zeile[programm_az].delay;
					if (programm_delaycounter > 0)
						{
						programm_status = 1;
						goto programm_engine_finished;
						}
					goto programm_nextline;
					}

				update_demonerror("unknown OP");								// Status von Demon in SHM aktualisieren
				printf("(exec)--Unknown operand - TERMINATING\n");
				log_append(3,"(exec)--Unknown operand - TERMINATING");
				programm_status = -1;
				programm_delaycounter = -2;


programm_engine_finished:
				// printf("**TP(Z%d): finished, Status:%d **\n",programm_az,programm_status);
				goto programm_weiter;
				}

			if (programm_status == 1)													// Programm luft, Zeile ist abgearbeitet und hngt im Delay-Loop
				{
				// printf("**TP(Z%d): Delay **\n",programm_az);
				programm_delaycounter--;
				if (programm_delaycounter <= 0)
					{
					programm_status = 0;
					goto programm_nextline;
					}
				goto programm_weiter;
				}

programm_nextline:
			programm_az++;
			if (programm_az >= programm_max_steps) programm_status = -1;
programm_weiter:
			;
			// printf("**TP(Z%d): weiter, Status:%d **\n",programm_az,programm_status);
			}
		}






	// ----- Prfen, ob sich gegenber dem letztenmal etwas gendert hat -----
	az = 0;
	if (old_r0				!= t_r0				) az++;
	if (old_g0				!= t_g0				) az++;
	if (old_b0				!= t_b0				) az++;
	if (old_w0				!= t_w0				) az++;
	if (old_r1				!= t_r1				) az++;
	if (old_g1				!= t_g1				) az++;
	if (old_b1				!= t_b1				) az++;
	if (old_w1				!= t_w1				) az++;
	if (old_dim0			!= t_dim0			) az++;
	if (old_dim1			!= t_dim1			) az++;

	// bei nderung diese RGB-Werte in die tmp-DB schreiben
	if (az > 0)
		{
		update_demonstatus(ti,10,"RGB write");							// Status von Demon in SHM aktualisieren
		sql_write_rgbvalues(sql_var_handle,t_r0,t_g0,t_b0,t_w0,t_dim0,t_r1,t_g1,t_b1,t_w1,t_dim1);
		} 

	// Alte Werte merken
	old_r0 = t_r0;
	old_g0 = t_g0;
	old_b0 = t_b0;
	old_w0 = t_w0;
	old_r1 = t_r1;
	old_g1 = t_g1;
	old_b1 = t_b1;
	old_w1 = t_w1;
	old_dim0 = t_dim0;
	old_dim1 = t_dim1;

	goto hauptschleife;

// ende:
	sqlite3_close(sql_config_handle);											// Datenbank schlieen
	sqlite3_close(sql_var_handle);												// Datenbank schlieen
errorende:
	return(0);
	}
