// **********   shm_use.h - Include-File fr die Verwendung des Shared-Memory-Bereichs   **********


// *****   Globale Variablen   ******

#define											SHM_VERSION "0.55"

// Dummy-Speicher zum Testen
#define											SHM_ID_DUMMY					42000
#define											SHM_DUMMY_SIZE				65536

// Sensor-Daten
#define											SHM_ID_SENSORDATA			42001
#define											SHM_ID_SENSORCOUNT		42002

// Aktor-Daten Read und Write
#define											SHM_ID_ACTOR_R_DATA		42003
#define											SHM_ID_ACTOR_R_COUNT	42004
#define											SHM_ID_ACTOR_W_DATA		42005
#define											SHM_ID_ACTOR_W_COUNT	42006

// DMX512-Daten
#define											SHM_ID_DMX						42007
#define											SHM_DMX_SIZE					 8192

// Statusfelder fr Demons
#define											SHM_ID_DEMONSTATUS		42008
#define											SHM_ID_DEMONCOUNT			  128
signed long int							my_demon_id = 0;

// --- verwendete IDs for my_demon_id ---
//   0: shm_create					Das Tool, das die shm-Bereiche anlegt
//   1: rgb_demon						Farbsteuerdemon fr ALL3651
//   2: rgb_listener				UDP-Empfnger fr ALL3651
//   3: rgb_remote					IR-Fernbedienungsempfnger fr ALL3651
//   4: rgb_program					Ausfhrungseinheit fr Programme (ALL3651)
//   5: all3075_demon				Steuerung fr ALL3075-Steckdosen
//   6: plugwise_demon			Steuerung fr Plugwise-Steckdosen
//   7: timer_demon					Zeitsteuerung fr logische Aktoren
//   8: cv_demon						Kontrolle der Poti-Eingnge am ALL3651
//   9: update_demon				Updater fr alle Gerte zum automatischen Einspielen von Patches


// Medaten vom ALL2075V2 ADC
#define											SHM_ID_ADC						42009
#define											SHM_ID_ADCCOUNT				65536


struct s_shm_sensordata
	{
	double value_float;
	signed long int value_long;
	char value_string[150];
	} ;

struct s_shm_actordata
	{
	double value_float;
	signed long int value_long;
	char value_string[150];
	} ;

struct s_shm_dmxdata
	{
	char dmx_array[513];																	// 0: Startbyte, 1...512: Datenbytes
	unsigned short int dmx_laenge;												// Anzahl zu sendender Bytes inkl. Startbyte (norm. 513)
	} ;

struct s_shm_demonstatus
	{
	signed long int demon_id;															// eindeutige ID des Demons (sollte immer gleich dem Array-Index sein)
	char name_string[50];																	// Name
	char version_string[10];															// Versionsnummer
	signed long int last_boot;														// Timestamp(UTC) der letzten Aktion
	signed long int last_activity;												// Timestamp(UTC) der letzten Aktion
	signed long int ti;																		// Counter
	char debug_message[256];															// letzte Debug-Meldung
	char error_message[256];															// letzte Fehlermeldung
	signed long int status_long;													// Status numerisch
	char status_string[80];																// Status als String
	} ;

struct s_shm_adc
	{
	unsigned short int adc[6];
	} ;


// Sensor-Datenstructs
struct s_shm_sensordata			shm_sensordata_dummy;
#define											SHM_SENSORDATA_SIZE (sizeof(shm_sensordata_dummy) * MAX_LOGICAL_SENSORS)
unsigned long int						shm_sensorcount_dummy;
#define											SHM_SENSORCOUNT_SIZE sizeof(shm_sensorcount_dummy)


// DMX512-Datenstruct
struct s_shm_dmxdata				shm_dmxdata_dummy;
#define											SHM_DMXDATA_SIZE sizeof(shm_dmxdata_dummy)


// Demonstatus-Datenstructs
struct s_shm_demonstatus		shm_demonstatus_dummy;
#define											SHM_DEMONSTATUS_SIZE (sizeof(shm_demonstatus_dummy) * SHM_ID_DEMONCOUNT)

struct s_shm_demonstatus* 	my_demonstatus_data;				// Demonstatus-Array im Shared-RAM


// Aktoren
struct s_shm_actordata			shm_actordata_dummy;
#define											SHM_ACTOR_R_DATA_SIZE (sizeof(shm_actordata_dummy) * MAX_LOGICAL_ACTORS)
#define											SHM_ACTOR_W_DATA_SIZE (sizeof(shm_actordata_dummy) * MAX_LOGICAL_ACTORS)
unsigned long int						shm_actorcount_dummy;
#define											SHM_ACTOR_R_COUNT_SIZE sizeof(shm_actorcount_dummy)
#define											SHM_ACTOR_W_COUNT_SIZE sizeof(shm_actorcount_dummy)

// ADC-Datenstructs
struct s_shm_adc						shm_adc_dummy;
#define											SHM_ADC_SIZE (sizeof(shm_adc_dummy) * SHM_ID_ADCCOUNT)



// *****   Funktions-Prototypen   *****

// void							cache_setzedefault(void);
signed long int			use_my_demonstatus_shm			(signed long int nummer);
void								update_demonstatus					(signed long int ti,signed long int numstatus,char stringstatus[]);
void								update_demondebug						(char stringdebug[]);
void								update_demonerror						(char stringerror[]);


// *****   use_my_demonstatus_shm   -   SHM attachen und zur Benutzung herrichten   *****

signed long int use_my_demonstatus_shm(signed long int nummer)
	{
	signed long int rw = 0;
	int shID;


	my_demon_id = nummer;																	// Demon-ID in globale Variable schreiben

	// auf existierendes Shared Memory fr Demonstatus zugreifen
	shID = shmget(SHM_ID_DEMONSTATUS,SHM_DEMONSTATUS_SIZE,0666);
	if (shID >= 0)
		{
		my_demonstatus_data = shmat(shID,0,0);
		if (my_demonstatus_data == (struct s_shm_demonstatus *)-1)
			{
			printf("Problem beim Zugriff auf DEMONSTATUS shared memory !\n");
			goto ende;
			}
		else
			{
			rw = 1;
			// printf("SHM_DEMONSTATUS_data OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_demonstatus_data war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		rw = -1;
		goto ende;
    }
ende: return(rw);
	}



// *****   update_demonstatus   -   Status numerisch und als String updaten   *****

void update_demonstatus(signed long int ti,signed long int numstatus,char stringstatus[])
	{
	my_demonstatus_data[my_demon_id].last_activity = time(0);														// Timestamp(UTC) der letzten Aktion
	my_demonstatus_data[my_demon_id].ti = ti;																						// Counter
	my_demonstatus_data[my_demon_id].status_long = numstatus;														// numerischer Status
	strcpy(my_demonstatus_data[my_demon_id].status_string,stringstatus);								// String-Status
	// sprintf(my_demonstatus_data[my_demon_id].debug_message,"Start");									// letzte Debug-Meldung
	// my_demonstatus_data[my_demon_id].error_message[0] = 0;														// letzte Fehlermeldung
	}



// *****   update_demondebug   -   Debugmessage als String updaten   *****

void update_demondebug(char stringdebug[])
	{
	strcpy(my_demonstatus_data[my_demon_id].debug_message,stringdebug);									// Debug-Meldung
	}



// *****   update_demonerror   -   Errormessage als String updaten   *****

void update_demonerror(char stringerror[])
	{
	strcpy(my_demonstatus_data[my_demon_id].error_message,stringerror);									// Error-Meldung
	}
